#!/usr/local/bin/perl56 

=head1 NAME
sam.pl - A user interface for WGS sequence assembly projects

=head1 SYNOPSIS
Just run it from your favorite browser

=head1 DESCRIPTION
wgs_assembly.pl allows users to keep track of assembly progress, query the database for specific genes, compare assemblies and assemblers and much more

=head1 EXAMPLES

=head1 NOTES

=head1 AUTHOR
Rene Warren April 2003
=cut

use lib "/home/rwarren/perl/Production/SAM/lib";
use lib "/usr/local/lib/perl5/site_perl/5.005";
use lib "/usr/local/ulib/prod/alDente/lib/perl";


use CGI qw/:all/;
use local::gscweb;
use strict;
use Date::Manip;
use Data::Dumper;
use Cipher;
use wgsDATA;
use wgsHTML;

####Variable
my ($leading, $lagging, $orf_hash, $orf_stat, $graph_value);
my ($supertig_coord, $contig_coord, $read_coord, $ultracontig_size);
my ($tig_scroll, $orf_scroll, $contig_size, $orf_tig, $maptig_scroll);
my $dbconfig_file="userinfo.txt";
my $htdoc_config="config.txt";
my $page = 'gscweb'->new();

my ($featureorf, $zoom, $vertmove, $horizmove);

my $BLASTABLE_ID=1;

my (%cache_key);

####Parameter
my $menu=param('menu');
my $assembly_id=param('assembly_id');
my $compare_id=param('compare_id');
my $report_menu=param('reportmenu');
my @summary_list=param('summary');
my $assembler_name=param('assembler');
my $assembly_type=param('assembly_type');
my @library_list=param('library');
my $insert_library=param('insertlibrary');
my $count_gap=param('countgap');
my $count_overlap=param('countoverlap');
my $orf_name=param('orf_name');
my $sort=param('sort');
my $fuzzykey=param('fuzzykey');
my $orf_id=param('orf_id');
my $tig_selection=param('contig');
my $gap_stats=param('gap_stats');
my $gene_finder_id=param('gene_finder_id');
my $map_id=param('map_id');
my $gene_finder_select=param('gene_finder_select');
my $clone_name=param('clone_name');
my $buried=param('buried');
my $maptig_selection=param('maptig');

my $username=param('username');
my $password=param('access');
my $sam_selection=param('samselect');
my @delete_job=param('checkdeleted');
my $file=param('filename');
my $email=param('email');
my $start_sequence=param('startseq');
my $end_sequence=param('endseq');

###Defaults

#-------------------------------
#set the buried value
if(! param('buried')){$buried=0;}else{$buried=1;}

#-------------------------------
#set the orf value
if(! param('orf')){$featureorf='';}else{$featureorf=param('orf');} 

#-------------------------------
#set the default zoom selection
if(! param('zoom')){$zoom=100;}else{$zoom=param('zoom');}

#-------------------------------
#get the vertmove value
if(! param('vertmove')){$vertmove=0;}else{$vertmove=param('vertmove');}

#-------------------------------
#get the horizmove value
if(! param('horizmove')){$horizmove=0;}else{$horizmove=param('horizmove');}

#-------------------------------
#set the move value
if (param('move') eq 'l'){$horizmove+=200;}#fast left
elsif (param('move') eq 'r'){$horizmove-=200;}#fast right
elsif (param('move') eq 'u'){$vertmove+=200;}#fast up
elsif (param('move') eq 'd'){$vertmove-=200;}#fast down

#-------------------------------
#key for cache
if (! param('fuzzykey')){$cache_key{'key'}=getppid();$fuzzykey=$cache_key{'key'};}
elsif (param ('fuzzykey')){$cache_key{'key'}=$fuzzykey;}

###Initial Values
my $scriptname=script_name();
my ($cgipath, $htdocpath)=wgsDATA::getScriptLocation($scriptname, $htdoc_config);
my $user_info=wgsDATA::getUserInfo($dbconfig_file);
my $project=wgsDATA::getProject($user_info);
$assembly_id=wgsDATA::getDefaultAssembly($user_info) if (! $assembly_id);
$gene_finder_id=wgsDATA::getDefaultGeneFinder($user_info, $assembly_id) if (!$gene_finder_id);
$map_id=wgsDATA::getDefaultMapVersion($user_info, $assembly_id) if (!$map_id);

my %conf=%$user_info;

####SAM
    
my $sam_credential;

if  (param('data')){
   my $tmpdata_ecz=param('data');
   $sam_credential=Cipher::unwrapData($tmpdata_ecz, $user_info->{'password'});
}else{
   $sam_credential={username=>$username, password=>$password, database=>$conf{'assembly_database'}, host_name=>$conf{'assembly_host'}, pass=>-1};
}

my $updated_user=wgsDATA::checkDatabaseConnection($sam_credential);

####pack the data
my $compressed_userinfo=Cipher::wrapData($updated_user, $user_info->{'password'});

####Graphs
if (param('progressgraph')){
   wgsDATA::createProgressGraph(\%cache_key);
}
if (param('distributiongraph')){
   wgsDATA::createDistributionGraph(\%cache_key);
}
if (param('insertgraph')){
#   wgsDATA::createInsertGraph($insert_library, $user_info, $assembly_id, $gene_finder_id);
   wgsDATA::createInsertGraph($insert_library, \%cache_key);
}
if (param('gapgraph')){
   wgsDATA::createGapDistribution(\%cache_key);
}


#---------------------------------DATA FOR GRAPH----------------------------------

if ($menu==3){#deals with the data--->avoid caching
    ($tig_scroll, $orf_scroll, $contig_size, $orf_tig)=wgsDATA::getTigScrollBoxInfo($user_info, $assembly_id, $gene_finder_id);
    my @tig_scroll=@$tig_scroll;

    if (! defined param('contig')){                      #essential otherwise doesn't allow arachne contig0 to be displayed
       $tig_selection=$tig_scroll[$#tig_scroll];         #arachne tigs nomenclature starts with 0.
       $zoom=100;
    }

    if (param('orf')){
       $tig_selection=$orf_tig->{$featureorf};
    }

    ($leading, $lagging, $orf_hash, $orf_stat)=wgsDATA::getSpecificOrfInfo($user_info, $assembly_id, $gene_finder_id, $tig_selection, $contig_size->{$tig_selection});

   if (param('showcontig')){      #script has been run once
      my ($tmp_selection, $build)=wgsDATA::getAssemblerNBuild($user_info, $assembly_id);
      wgsDATA::computeTigImage($tig_selection, $contig_size, $orf_hash, $zoom, $vertmove, $tmp_selection, $build, $featureorf, $leading, $lagging);
   }
}elsif($menu==4 && defined $map_id){

   $maptig_scroll=wgsDATA::getMapScrollBoxInfo($user_info, $assembly_id, $map_id);
   my @maptig_scroll=@$maptig_scroll;

   if (! defined param('maptig')){
      $maptig_selection=$maptig_scroll[$#maptig_scroll];
      $zoom=8000;
   }

   $graph_value=wgsDATA::initializeMapViewerData;
   my $supertig_arr=wgsDATA::getSupertigIntrinsicData($user_info, $assembly_id, $map_id, $maptig_selection, $buried);
   my $contig_arr=wgsDATA::getContigCoord($user_info, $assembly_id, $map_id, $maptig_selection, $buried);
   my $read_arr=wgsDATA::getReadCoord($user_info, $assembly_id, $map_id, $maptig_selection, $buried);

   ($supertig_coord, $contig_coord, $read_coord, $ultracontig_size)=wgsDATA::rearrangeMapCoord($supertig_arr, $contig_arr, $read_arr);###here is where I want to allow flipping
   
   if (param('showmap')){         #script has been run once
      wgsDATA::computeMapImage($maptig_selection, $zoom, $horizmove, $ultracontig_size, $supertig_coord, $contig_coord, $read_coord, $clone_name, $graph_value);
   }
}

#####Make Menu
wgsHTML::setHTML($page, $htdocpath);
wgsHTML::printTitle($htdocpath, $project, $updated_user->{'pass'}, $fuzzykey, $scriptname);
wgsHTML::printLink($scriptname, $assembly_id, $gene_finder_id, $htdocpath, $fuzzykey, $compressed_userinfo, $menu);

####Get common
my $tmp_build=wgsDATA::getAssemblyList($user_info);
my $tmp_map=wgsDATA::getMapList($user_info);
my ($tmp_selection, $build)=wgsDATA::getAssemblerNBuild($user_info, $assembly_id);
my $gene_finder_used=wgsDATA::getGeneFinderUsed($user_info, $assembly_id);
my $map_version_used=wgsDATA::getMapVersionUsed($user_info, $assembly_id);

&Menu();

$page->BottomBar();

exit;

#---------------------------
sub Menu{

   ####Menu

   if (! param('menu')){        #main menu (home page)
      my $assembly_list=wgsDATA::getAssemblyInfo($user_info, $assembly_id);
      #my $true=wgsDATA::checkSequenceCache($user_info);
      my $true=1;
      wgsDATA::processSequenceInfo($user_info) if (! $true); #only get this info once, cache for subsequent access
      my ($library_list, $seqclone_list, $unique_read)=wgsDATA::getSequenceCache($user_info);
      my ($orf_info, $init_codon)=wgsDATA::getOrfInfo($user_info, $assembly_id, $gene_finder_id);
      my $download_config=wgsDATA::getDownloadConfig($user_info, $assembly_id, $BLASTABLE_ID, $gene_finder_id, $htdocpath);
      wgsHTML::printHomePage($assembly_id, $library_list, $seqclone_list, $unique_read, $assembly_list, $orf_info, $init_codon, $tmp_build, $scriptname, $fuzzykey, $download_config, $compressed_userinfo, $gene_finder_id, $gene_finder_used, $project);
   }elsif ($menu==1){           #Assembly-specifics
      if (! $report_menu){
         wgsHTML::reportMenu($scriptname, $menu, $assembly_id, $gene_finder_id, $fuzzykey, $compressed_userinfo);
      }elsif ($report_menu==1){ #assembly progress
         if (param('progress')){ 
            my ($tmp_summary, $progress_title, $max_option)=wgsDATA::getProgress($user_info, \@summary_list, $assembler_name, \%cache_key);
            wgsHTML::printProgressGraph($scriptname, $menu, $report_menu, $tmp_summary, $progress_title, $assembler_name, $max_option, $fuzzykey, $compressed_userinfo);
         }else{
            my $tmp_assembler=wgsDATA::getAssemblerList($user_info);
            wgsHTML::progressMenu($tmp_assembler, $tmp_selection, $scriptname, $menu, $report_menu, $fuzzykey, $compressed_userinfo);
         }
      }elsif($report_menu==2){ #Read distribution
         if (param('distribution')){
            my ($all_account, $missing_lib, $pass)=wgsDATA::getDistribution($user_info, \@library_list, $assembly_id, $tmp_selection, $build, \%cache_key);
            if ($pass){
               wgsHTML::printDistributionGraph($scriptname, $menu, $report_menu, $all_account, $missing_lib, $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo);
            }else{
               wgsHTML::errorDistribution($scriptname, $menu, $report_menu, $missing_lib, $fuzzykey, $compressed_userinfo);
            }
         }else{
            my $tmp_library=wgsDATA::getLibraryList($user_info);
            wgsHTML::distributionMenu($tmp_build, $assembly_id, $gene_finder_id, $tmp_library, $scriptname, $menu, $report_menu, $fuzzykey, $compressed_userinfo);
         }
      }elsif($report_menu==3){ #insert-size distribution
         my $insert_list=wgsDATA::getInsertSizeData($user_info, $assembly_id, $tmp_selection, $build, \%cache_key);
         wgsHTML::insertMenu($tmp_build, $assembly_id, $gene_finder_id, $scriptname, $menu, $report_menu, $fuzzykey, $compressed_userinfo);
         wgsHTML::printInsertGraph($scriptname, $insert_list, $fuzzykey, $compressed_userinfo);
      }elsif($report_menu==4){ #gap clones
         if (! $count_gap && ! $gap_stats){
            wgsHTML::gapMenu($tmp_build, $assembly_id, $gene_finder_id, $scriptname, $menu, $report_menu, $fuzzykey, $compressed_userinfo);
            my ($classify_count)=wgsDATA::getGapData($user_info, $assembly_id, $tmp_selection, $build, \%cache_key);
            wgsHTML::showAllGapTable($classify_count, $scriptname, $menu, $report_menu, $assembly_id, $gene_finder_id, $fuzzykey, $compressed_userinfo);
         }elsif ($count_gap && ! $gap_stats){
            wgsHTML::gapMenu($tmp_build, $assembly_id, $gene_finder_id, $scriptname, $menu, $report_menu, $fuzzykey, $compressed_userinfo);
            my ($tmp_gap_list, $tmp_gap_stat)=wgsDATA::getGapCache(\%cache_key);
            wgsHTML::showSpecificGap($count_gap, $scriptname, $menu, $report_menu, $assembly_id, $gene_finder_id, $htdocpath, $tmp_gap_list, $tmp_gap_stat, $fuzzykey, $compressed_userinfo);
         }elsif ($gap_stats){###Huge Table showing all the Gap Stats (Sequence Gaps identified by us)
            my ($tmp_gap_list, $tmp_gap_stat)=wgsDATA::getGapCache(\%cache_key);
            wgsHTML::showGapSummaryOnly($scriptname, $menu, $tmp_selection, $build, $report_menu, $assembly_id, $gene_finder_id, $tmp_gap_list, $tmp_gap_stat, $fuzzykey, $compressed_userinfo);
         }
      }elsif($report_menu==5){ #overlap clones
         wgsHTML::overlapMenu($tmp_build, $assembly_id, $gene_finder_id, $scriptname, $menu, $report_menu, $fuzzykey, $compressed_userinfo);
         if ($count_overlap eq ''){
            my $overlap_count_list=wgsDATA::getOverlapData($user_info, $assembly_id, \%cache_key);
            wgsHTML::showAllOverlapTable($overlap_count_list, $scriptname, $menu, $report_menu, $assembly_id, $gene_finder_id, $fuzzykey, $compressed_userinfo);
         }else{
            my $tmp_overlap_list=wgsDATA::getOverlapCache(\%cache_key);
            wgsHTML::showSpecificOverlap($count_overlap, $scriptname, $menu, $report_menu, $assembly_id, $gene_finder_id, $tmp_overlap_list, $fuzzykey, $compressed_userinfo);
         }
      }
   }elsif ($menu==2){        #compare assemblies
      if (! $compare_id){
         $compare_id=$assembly_id;  
      }
      my $compare_list=wgsDATA::getComparisonData($user_info, $assembly_id, $compare_id);
      wgsHTML::showCompare($tmp_build, $assembly_id, $gene_finder_id, $compare_id, $compare_list, $scriptname, $menu, $fuzzykey, $compressed_userinfo, $project);

   }elsif ($menu==3){        #contig viewer

      wgsHTML::contigViewerMenu($tmp_build, $assembly_id, $gene_finder_id, $gene_finder_used, $scriptname, $menu, $fuzzykey, $compressed_userinfo);
      wgsHTML::contigViewerForm($tig_scroll, $scriptname, $tig_selection, $featureorf, $orf_scroll, $assembly_id, $gene_finder_id, $fuzzykey, $menu, $zoom, $compressed_userinfo);
      wgsHTML::contigViewerDisplay($scriptname, $tig_selection, $zoom, $contig_size, $orf_hash, $vertmove, $assembly_id, $gene_finder_id, $featureorf, $fuzzykey, $menu, $compressed_userinfo);
      wgsHTML::contigViewerStats($orf_stat);
   }elsif ($menu==4){        #map viewer

      wgsHTML::mapViewerMenu($tmp_build, $assembly_id, $map_id, $map_version_used, $scriptname, $menu, $fuzzykey, $compressed_userinfo);
      if(defined $map_id){
         wgsHTML::mapViewerForm($maptig_scroll, $map_id, $clone_name, $maptig_selection, $buried, $menu, $zoom, $horizmove, $assembly_id, $scriptname, $fuzzykey, $compressed_userinfo);
         wgsHTML::mapViewerDisplay($scriptname, $maptig_selection, $zoom, $horizmove, $map_id, $clone_name, $buried, $read_coord, $htdocpath, $graph_value, $assembly_id, $fuzzykey, $menu, $compressed_userinfo);
      }
   }elsif ($menu==5){        #orflist

      if (! $orf_id){
         wgsHTML::orfMenu($tmp_build, $assembly_id, $gene_finder_id, $gene_finder_used, $scriptname, $menu, $fuzzykey, $compressed_userinfo);
         my ($orf_list, $total_orf)=wgsDATA::getOrfList($user_info, $assembly_id, $orf_name, $gene_finder_id);
         wgsHTML::displayOrfList($scriptname, $menu, $assembly_id, $gene_finder_id, $orf_name, $fuzzykey, $orf_list, $total_orf, $sort, $compressed_userinfo);
      }elsif($orf_id){
         my $all_orf=wgsDATA::getOneOrfInfo($user_info, $assembly_id, $orf_id, $gene_finder_id);
         wgsHTML::displayOrfSpecific($scriptname, $menu, $assembly_id, $gene_finder_id, $gene_finder_used, $orf_name, $fuzzykey, $orf_id, $all_orf, $tmp_selection, $build, $compressed_userinfo);
      }#come from graph

   }elsif ($menu==6){        #blast

      my $blast_config=wgsDATA::getBlastConfig($user_info, $assembly_id, $BLASTABLE_ID, $gene_finder_id);
      wgsHTML::blastMenu($tmp_build, $assembly_id, $gene_finder_id, $gene_finder_used, $scriptname, $menu, $fuzzykey, $compressed_userinfo);
      wgsHTML::blastSetParameter($blast_config, $tmp_selection, $build, $cgipath, $gene_finder_id);

   }elsif ($menu==8){        #SAM
 
      if($updated_user->{'pass'}!=1){
         wgsHTML::SAMlogin($scriptname, $htdocpath, $updated_user, $menu, $fuzzykey, $assembly_id, $gene_finder_id);
      }else{
        
         wgsHTML::SAMstarttable();
         wgsHTML::SAMhome($scriptname, $htdocpath, $updated_user, $menu, $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo, $sam_selection);

         if($sam_selection==1){
           wgsHTML::filePrompt($scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo) if (! param('filename'));; 
           wgsDATA::checkXMLFileExistence($file, $email, $updated_user, $scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo) if (param('filename'));#selection becomes cmd id
         }elsif($sam_selection==2){
            if (! param('request')){
               my ($tmp_assembler_list, $tmp_library_list, $sequence_date, $most_recent_date, $least_recent_date)=wgsDATA::getAssemblerTraceNLibrary($updated_user);
               my ($gene_finder_available)=wgsDATA::getGeneFinderList($updated_user);
               wgsHTML::displayAssembleForm($scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo, $tmp_assembler_list, $tmp_library_list, $sequence_date, $most_recent_date, $least_recent_date, $gene_finder_available);
             }elsif(param('request')){
                wgsDATA::writeTask($start_sequence, $end_sequence, \@library_list, $assembler_name, $assembly_type,$gene_finder_select, $email, $scriptname, $sam_selection, $updated_user);
              }
         }elsif($sam_selection==3){
           wgsHTML::filePrompt($scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo) if (! param('filename'));;
           wgsDATA::checkXMLFileExistence($file, $email, $updated_user, $scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo) if (param('filename'));#selection becomes cmd id
         }elsif($sam_selection==4){
           if (! param('tiemap')){
              my $tmp_map=wgsDATA::getMapList($user_info);
              wgsHTML::displayTieMapForm($scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $compressed_userinfo, $tmp_build, $tmp_map, $map_id);
           }else{
              wgsDATA::addTieMapTask($assembly_id, $map_id, $email, $sam_selection, $updated_user);
           }
         }elsif($sam_selection==5){
            if (! param('annotate')){
               my ($gene_finder_available)=wgsDATA::getGeneFinderList($updated_user);
               wgsHTML::displayAnnotateForm($scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo, $gene_finder_available, $gene_finder_used, $tmp_build);
            }else{
               wgsDATA::addAnnotationTask($assembly_id, $gene_finder_select, $email, $sam_selection, $updated_user);
            }      
         }elsif($sam_selection==9){
            wgsDATA::deleteJob($updated_user, \@delete_job) if (param('delete'));
            my $task_reference=wgsDATA::retrieveTasks($updated_user);
            wgsHTML::displayTasks($task_reference, $scriptname, $htdocpath, $sam_selection, $menu,  $fuzzykey, $assembly_id, $gene_finder_id, $compressed_userinfo);  
         }else{
            wgsHTML::SAMwelcome();
         }
         wgsHTML::SAMendtable();
     }
  }elsif ($menu==99){        #detail info on sequence

      my $true=wgsDATA::checkSeqLibraryCache($user_info);
      wgsDATA::processSeqLibraryInfo($user_info) if (! $true); #only get this info once, cache for subsequent access
      my ($seq_library, $seq_total)=wgsDATA::getSeqLibraryCache($user_info);
      my ($library_list, $clone_list, $unique_read)=wgsDATA::getSequenceCache($user_info);
      wgsHTML::displaySeqLibraryInfo($seq_library, $seq_total, $clone_list, $unique_read);
   }
}
