#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

BlastParser.pl -

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item lib '/home/martink/export/extern/perl'

=item strict

=item Bio::Tools::BPlite

=item Number::Format

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item Orf2Database

=back

=cut

#Mar 26th, 2003
use lib '/home/martink/export/extern/perl';
use strict;
use Bio::Tools::BPlite;   
use Number::Format;   
use FindBin;
use lib "$FindBin::Bin/../lib";
use Orf2Database;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
Author:Rene Warren
Script:BlastParser.pl
Purpose:Parses Blast Output using BioPerl's BPlite library
my $orf_id=$ARGV[0];
my $blast_config_id=$ARGV[1];
my %user_info;
$user_info{'username'}=$ARGV[2];
$user_info{'password'}=$ARGV[3];
$user_info{'database'}=$ARGV[4];
$user_info{'host_name'}=$ARGV[5];
my $blast_result=&parseBlast();
Orf2Database::addBlastRecord($orf_id, \%user_info, $blast_result, $blast_config_id);

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#parseBlast'>parseBlast; </A></strong></font><BR>

=over

=for html <A NAME='parseBlast'>

=item parseBlast

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#------------------------------------------------------
sub parseBlast{

    my %blast;

    my $report = Bio::Tools::BPlite->new(-fh=>\*STDIN);
    do{

       my $savequery=$report->query;
       my $querylength=$report->qlength;

       my $sbjct = $report->nextSbjct; 

       if (defined $sbjct){

          my $savehit=$sbjct->name;
          my $hsp= $sbjct->nextHSP;
          my $score=$hsp->score;
          my $evalue=$hsp->P;
          my $bits=$hsp->bits;
          my $match=$hsp->match;
          my $hitlength=$hsp->length;
          my $positive=$hsp->positive;
          my $querysequence=$hsp->querySeq;
          my $hitsequence=$hsp->sbjctSeq;              
          my $query_start=$hsp->query->start;
          my $query_stop=$hsp->query->end;
          my $hit_start=$hsp->hit->start;
          my $hit_stop=$hsp->hit->end;
          my $hit_protein_length=$hsp->hit->seqlength;

          my $orf_similarity=($positive/$querylength)*100;  
          my $hit_similarity=($positive/$hit_protein_length)*100;            
          my $hit_identity=($match/$hit_protein_length)*100;
         
 
          $savehit=~s/[\'\"]//gi;###remove any quotations globally          

          my ($ac, $protein_name, $organism)=split(/\*{3}/, $savehit);
          $blast{'accession'}=$ac;
          $blast{'protein'}=$protein_name;
          $blast{'organism'}=$organism;
          $blast{'orf_similarity'}=$orf_similarity;
          $blast{'score'}=$score;
          $blast{'evalue'}=$evalue;
          $blast{'bits'}=$bits;
          $blast{'match'}=$match;
          $blast{'hit_start'}=$hit_start;
          $blast{'hit_stop'}=$hit_stop;
          $blast{'query_start'}=$query_start;
          $blast{'query_stop'}=$query_stop;
          $blast{'positive'}=$positive;
          $blast{'hit_length'}=$hitlength;
          $blast{'hit_sequence'}=$hitsequence;
          $blast{'query_sequence'}=$querysequence;
          $blast{'hit_protein_length'}=$hit_protein_length;
          $blast{'hit_similarity'}=$hit_similarity;
          $blast{'hit_identity'}=$hit_identity;

       }
       else{#if no hit found
          $blast{'accession'}="NA";
          $blast{'protein'}="NA";
          $blast{'organism'}="NA";
          $blast{'orf_similarity'}=0;
          $blast{'score'}=0;
          $blast{'evalue'}=0;
          $blast{'hit_start'}=0;
          $blast{'hit_stop'}=0;
          $blast{'query_start'}=0;
          $blast{'query_stop'}=0;
          $blast{'positive'}=0;
          $blast{'hit_length'}=0;
          $blast{'hit_sequence'}="NA";
          $blast{'query_sequence'}="NA";
          $blast{'bits'}=0;
          $blast{'match'}=0;
          $blast{'hit_protein_length'}=0;
          $blast{'hit_similarity'}=0;
          $blast{'hit_identity'}=0;

       }

     } while ($report->_parseHeader != -1); # go to next BLAST report

   return \%blast;
} 

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>

=cut

