#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

DumpSequencesNmakeBlastDB.pl - 

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Author:Rene Warren<BR>Script:DumpSequencesNmakeBlastDB.pl<BR>Purpose:Will make blastable database of ORFS for a specific assembly and gene finder in a user-specified directory (Blastable_DB and Config Tables, SAMdb)<BR>Oct 30, 2002/Feb 14 2003/Apr 4th 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item Date::Manip

=item SQLdb

=item CommonTask

=item Data::Dumper

=item XMLparser

=item CustomAnalysis

=item Orf2Database

=item Cache::FileCache

=back

=cut

#Author:Rene Warren
#Script:DumpSequencesNmakeBlastDB.pl
#Purpose:Will make blastable database of ORFS for a specific assembly and gene finder in a user-specified directory (Blastable_DB and Config Tables, SAMdb)
#Oct 30, 2002/Feb 14 2003/Apr 4th 2003
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use Date::Manip;
use SQLdb;
use CommonTask;
use Data::Dumper;
use XMLparser;
use CustomAnalysis;
use Orf2Database;
use Cache::FileCache;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=item $opt_p

=item $opt_u

=item $opt_d

=item $opt_a

=item $opt_o

=item $opt_i

=item $opt_m

=item $opt_g

=back

=cut

use vars qw($opt_p $opt_u $opt_d $opt_a $opt_o $opt_i $opt_m $opt_g);
##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
require "getopts.pl";
&Getopts('p:u:d:a:o:i:g:m:');
####Arguments
if (! $opt_p || ! $opt_u || ! $opt_d || ! $opt_a || ! $opt_o  || ! $opt_i || ! $opt_g ){
   die "Usage: DumpSequencesNmakeBlastDB.pl -u username -p password -d database -a assembly -o host name -g genefinder id -i job queue id -m e-mail\n";
}
####Making the cache
my %cache_options = (namespace=>"gene",
                     cache_root=>"/home/tmp/rwarren",
                     default_expires_in=>1500);
my $cache = new Cache::FileCache(\%cache_options);
my @date_format=("%Y-%m-%d %H:%M:%S");
my %user_info;
$user_info{'username'}=$opt_u;
$user_info{'password'}=$opt_p;
$user_info{'database'}=$opt_d;
$user_info{'host_name'}=$opt_o;
my $email=$opt_m;
my $jobqueue_id=$opt_i;
my $assembly_id=$opt_a;
my $gene_finder_id=$opt_g;
#********************
my $blastable_tigid=1;
my $blastable_orfid=2;
#********************
my $configuration=&getConfigurationInfo(\%user_info, $assembly_id);
my $gf_config=Orf2Database::getGeneFinderConfiguration(\%user_info, $gene_finder_id);
my $blastprot_config=&getBlastableDBConfig(\%user_info, $blastable_orfid);
my $blastdna_config=&getBlastableDBConfig(\%user_info, $blastable_tigid);
my ($message_db, $status)=&createOrfDatabase(\%user_info, $configuration, $assembly_id, $blastprot_config, $gf_config->{'protein_file'}, $gf_config->{'cds_file'}, $gf_config->{'everygene_file'}, $gene_finder_id, $blastdna_config);
exit(0);

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#getBlastableDBConfig'>getBlastableDBConfig; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#createOrfDatabase'>createOrfDatabase; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#sendEmail'>sendEmail; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getConfigurationInfo'>getConfigurationInfo; </A></strong></font><BR>

=over

=for html <A NAME='getBlastableDBConfig'>

=item getBlastableDBConfig

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$blastable_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub getBlastableDBConfig{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $blastable_id=shift;

   ####Get BLAST DB CREATOR Configuration
   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly Information
   my @field=("Blastable_DB_Exec", "Blastable_DB_Exec_Dir", "Blastable_DB_Location", "Blastable_DB_Node", "Blastable_DB_Wrapper", "Blastable_DB_Wrapper_Dir");
   my @table=("Blastable_DB");
   my @condition=("Blastable_DB_ID=$blastable_id");####1 for blastp
   my $select_blastable=SQLdb::GetRecord(\@field, \@table, \@condition);
   my $tmp_blastable=SQLdb::ExecuteUnique($dbh, $select_blastable);

   SQLdb::Disconnect($dbh);

   return $tmp_blastable;
}

=for html <A NAME='createOrfDatabase'>

=item createOrfDatabase

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_blastable</B>():  [Default = <B>shift</B>]<BR>
- <B>$orf_protein_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$orf_cds_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$everygene_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_blastable_nt</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub createOrfDatabase{

   my $user_info=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $assembly_id=shift;
   my $tmp_blastable=shift;
   my $orf_protein_file=shift;
   my $orf_cds_file=shift;
   my $everygene_file=shift;
   my $gene_finder_id=shift;
   my $tmp_blastable_nt=shift;

   my $type_aa="ORF_protein";
   my $type_nt="ORF_CDS";

   my ($message, $status);

   ####output
   my @blastable=@$tmp_blastable;
   my @blastable_nt=@$tmp_blastable_nt;

   my $exec_cmd="$blastable[1]/$blastable[0]";
   my $exec_cmd_nt="$blastable_nt[1]/$blastable_nt[0]";

   if (! -d "$blastable[2]/$assembly_id"){
      `mkdir $blastable[2]/$assembly_id`;
   }
   my $aa_output="$blastable[2]/$assembly_id/$orf_protein_file";#htdocs/assembly
   my $cds_output="$blastable[2]/$assembly_id/$orf_cds_file";
   my $everygene_output="$blastable_nt[2]/$everygene_file";

   eval{

      ####input
      CommonTask::dumpOrf($user_info, $aa_output, $cds_output, $assembly_id, $gene_finder_id);
      CommonTask::dumpEveryOrf($user_info, $everygene_output);

      ####Launch DB maker//Orf for specific build//protein
      my $cmd="$blastable[5]/$blastable[4] \'$exec_cmd\' \'$aa_output\' \'$type_aa\' \'$configuration{'Assembler_Name'}\' \'$configuration{'run_dir'}\' \'$configuration{'project_name'}\'";
      #print "$cmd\n";
      system("$cmd")==0 or die "$blastable[4] failed.\n";

      ####Launch DB maker//Orf for specific build//cds
      my $cmd="$blastable[5]/$blastable[4] \'$exec_cmd_nt\' \'$cds_output\' \'$type_nt\' \'$configuration{'Assembler_Name'}\' \'$configuration{'run_dir'}\' \'$configuration{'project_name'}\'";
      #print "$cmd\n\n";
      system("$cmd")==0 or die "$blastable[4] failed.\n";

      ####Launch DB maker//Every Orf in Database
      my $cmd="$blastable_nt[5]/$blastable_nt[4] \'$exec_cmd_nt\' \'$everygene_output\' \'$type_nt\' \'All\' \'s\' \'$configuration{'project_name'}\'";
      system("$cmd")==0 or die "$blastable_nt[4] failed.\n";
   };
   if($@){$status='failed';$message="Some problem making file databases.  ";}else{$status='completed';$message="File databases created.  ";}

   return $message, $status;

}

=for html <A NAME='sendEmail'>

=item sendEmail

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$email</B>():  [Default = <B>shift</B>]<BR>
- <B>$message</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$jobqueue_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$status</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub sendEmail{

   my $email=shift;
   my $message=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $jobqueue_id=shift;
   my $status=shift;

   ###must send the Job_Queue to the user

   my $comment="Your Job#$jobqueue_id:\nAnnotation of Assembly $configuration{'Assembler_Name'} Build$configuration{'run_dir'} using Gene Finder ID=$gene_finder_id $status.\n$message";
   CommonTask::sendMail($email, $comment, $jobqueue_id) if ($email ne '');

}

=for html <A NAME='getConfigurationInfo'>

=item getConfigurationInfo

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub getConfigurationInfo{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembly_id=shift;

   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly Information
   my @assembly_field=("Assembly_Build_ID", "Assembly_Type");
   my @assembly_table=("Assembly");
   my @assembly_condition=("Assembly_ID=$assembly_id");
   my $select_assembly=SQLdb::GetRecord(\@assembly_field, \@assembly_table, \@assembly_condition);
   my $buildref=SQLdb::ExecuteUnique($dbh, $select_assembly);

   my @build_arr=@$buildref;
   my $tmp_build=$build_arr[0];

   ####Get config info for given assembly
   my @config_field=("Assembler_Name", "Config_Data_Dir", "Config_Read_File", "Config_Qual_File", "Config_Xml_Dir", "Config_Exec_Dir", "Config_Exec_Cmd", "Config_Assembler_Dir", "Config_Xml_File", "Config_Assembler_Exec", "Assembler_ID", "Config_Analysis_File", "Config_AssemblyXml_File", "Config_SummaryXml_File", "Config_Tig_Fasta_File", "Config_Tig_Qual_File", "Config_Output_Dir");
   my @config_table=("Config", "Assembler", "Assembly");
   my @config_condition=("Assembler.Assembler_ID=Config.FK_Assembler__ID", "Assembler.Assembler_ID=Assembly.FK_Assembler__ID","Assembly_ID=$assembly_id");#can't select under project id, because above may return nothing for max build
   my $select_config=SQLdb::GetRecord(\@config_field, \@config_table, \@config_condition);
   my $data_config=SQLdb::ExecuteUnique($dbh, $select_config);

   #print "$select_config\n";

   my @data_config=@$data_config;
   if (!defined $data_config[0]){die "Problem getting config information\n";} #if problem with query (likely be an assembler name mismatch)

   ####Get Project Information
   my @project_field=("Project_ID", "Project_Name");
   my @project_table=("Project");
   my @project_condition=();
   my $select_project=SQLdb::GetRecord(\@project_field, \@project_table, \@project_condition);
   my $data_project=SQLdb::ExecuteUnique($dbh, $select_project);
   my @data_project=@$data_project;

   if (defined $build_arr[1]){
      $tmp_build=$build_arr[0]."_".$build_arr[1];
   }

   ####Get all info in one place
   my @configuration=("Assembler_Name", "$data_config[0]", "Config_Data_Dir", "$data_config[1]", "Config_Read_File", "$data_config[2]", "Config_Qual_File","$data_config[3]", "Config_Xml_Dir","$data_config[4]", "Config_Exec_Dir","$data_config[5]", "Config_Exec_Cmd", "$data_config[6]", "Config_Assembler_Dir", "$data_config[7]", "Config_Xml_File", "$data_config[8]", "Config_Assembler_Exec", "$data_config[9]", "Assembler_ID", "$data_config[10]", "run_dir", "$tmp_build", "project_name", "$data_project[1]", "project_id", "$data_project[0]", "Config_Analysis_File", "$data_config[11]", "Config_AssemblyXml_File", "$data_config[12]", "Config_SummaryXml_File", "$data_config[13]", "Config_Tig_Fasta_File", "$data_config[14]", "Config_Tig_Qual_File", "$data_config[15]", "Config_Output_Dir", "$data_config[16]");
   my %configuration=@configuration;

   SQLdb::Disconnect($dbh);

   return \%configuration;
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>

=cut

