#!/usr/local/bin/perl 
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

arachne_analyze.pl - 

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
 <BR>Author:Rene Warren<BR>Script:arachne_analyze.pl<BR>Purpose:Wrapper to analyze Arachne's output<BR>November 2002<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item AnalyzeAssembly

=item CreateAssemblyXML

=item Data::Dumper

=back

=cut

#Author:Rene Warren
#Script:arachne_analyze.pl
#Purpose:Wrapper to analyze Arachne's output
#November 2002
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use AnalyzeAssembly;
use CreateAssemblyXML;
use Data::Dumper;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
my @data_config=($ARGV[0], $ARGV[1], $ARGV[2], $ARGV[3], $ARGV[4]);
my @user_info=($ARGV[5], $ARGV[6], $ARGV[7], $ARGV[8]);
my @condition=($ARGV[9], $ARGV[10], $ARGV[11], $ARGV[12]);
my $qual="assembly.quals";
my $link="assembly.links";
my $read="assembly.reads";
my $discarded="reads.deleted";
my $work_dir="work";
my $tig="assembly.bases";
my $zipqual="assembly.quals.gz";
my $ziptig="assembly.bases.gz";
my ($super_tig, $supertig_info);
my $output_dir="$data_config[0]/$data_config[1]/$data_config[2]";
my $link_file="$output_dir/$link";
my $read_file="$output_dir/$read";
my $discardedreads_file="$output_dir/$work_dir/$discarded";
my $qual_file="$output_dir/$qual";
my $tig_file="$output_dir/$tig";
my $zip_qual_file="$output_dir/$zipqual";
my $zip_tig_file="$output_dir/$ziptig";
my $date=`date`;
chomp($date);
&unzipFile($zip_qual_file, $zip_tig_file);
CreateAssemblyXML::startRecord($date, \@data_config, $data_config[3]);
my $initial_stats=&parseArachneOutput($link_file, $read_file, $qual_file, $tig_file, \@data_config);
CreateAssemblyXML::endRecord(\@data_config, $data_config[3]);
my $addedread_stats=AnalyzeAssembly::getSinglet(\@user_info, \@condition, $initial_stats, $discardedreads_file);
my $addedgenome_stats=AnalyzeAssembly::getGenomicAssemblyStats(\@user_info, \@condition, $addedread_stats);
CreateAssemblyXML::assemblySummary($date, $addedgenome_stats, \@data_config);
exit;

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#unzipFile'>unzipFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachneOutput'>parseArachneOutput; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeValue'>initializeValue; </A></strong></font><BR>

=over

=for html <A NAME='unzipFile'>

=item unzipFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$zip_qual_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$zip_tig_file</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#--------------------------------
sub unzipFile{

   my $zip_qual_file=shift;
   my $zip_tig_file=shift;

   if (-e  $zip_qual_file){
      `gunzip $zip_qual_file`;
   }
   if (-e $zip_tig_file){
      `gunzip $zip_tig_file`;
   }
}

=for html <A NAME='parseArachneOutput'>

=item parseArachneOutput

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$link_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$read_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$qual_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$tig_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$data_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#--------------------------------
sub parseArachneOutput{

   my $link_file=shift;
   my $read_file=shift;
   my $qual_file=shift;
   my $tig_file=shift;
   my $data_config=shift;
 
   my $init_stats=&initializeValue();
   my ($read_per_tig, $read_stats)=AnalyzeAssembly::parseArachneReadFile($read_file, $init_stats);
   my ($super_tig, $supertig_stats)=AnalyzeAssembly::parseArachneLinkFile($link_file, $read_stats);
   my ($qual_stats, $contig_qual)=AnalyzeAssembly::parseArachneContigQuality($qual_file, $supertig_stats);
   my ($stats)=AnalyzeAssembly::parseArachneContig($tig_file, $read_per_tig, $contig_qual, $super_tig, $qual_stats, $data_config);

   return $stats;
   
}#end sub

=for html <A NAME='initializeValue'>

=item initializeValue

=for html
initialize values (essential)<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#--------------------------------
sub initializeValue{

   #initialize values (essential)
   my %init;
   $init{'smallest_tig'}=100000000;#have to set here since we loop through
   $init{'biggest_tig'}=0;
   $init{'all_a'}=0;
   $init{'all_c'}=0;
   $init{'all_g'}=0;
   $init{'all_t'}=0;
   $init{'consensus'}=0;
   $init{'totalbp'}=0;
   $init{'error_rate'}=0;
   $init{'average_phred'}=0;
   $init{'phred20'}=0;
   $init{'phred30'}=0;
   $init{'phred40'}=0;
   $init{'contigs'}=0;
   $init{'tigsize'}=0;
   $init{'assembled_reads'}=0;
   $init{'1'}=0;
   $init{'2'}=0;
   $init{'3'}=0;
   $init{'4'}=0;
   $init{'5'}=0;
   $init{'6+'}=0;
   $init{'10+'}=0;
   $init{'20+'}=0;

   return \%init;

}#end sub

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>

=cut

