#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

insilico_digest.pl -

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Author:Rene Warren<BR>Script:insilico_digest.pl<BR>Purpose:will perform in-silico digest of a specific region (contig/clone) and produce a virtual gel-image containing the band patterns<BR>May 2004<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item GD

=item strict

=item SQLdb

=item SAMQueryAPI

=item CommonTask

=item Data::Dumper

=item POSIX qw(log10)

=back

=cut

#Author:Rene Warren
#Script:insilico_digest.pl
#Purpose:will perform in-silico digest of a specific region (contig/clone) and produce a virtual gel-image containing the band patterns
#May 2004
use FindBin;
use lib "$FindBin::Bin/../lib";
use GD;
use strict;
use SQLdb;
use SAMQueryAPI;
use CommonTask;
use Data::Dumper;
use POSIX qw(log10);

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=item $opt_p

=item $opt_u

=item $opt_d

=item $opt_a

=item $opt_o

=item $opt_f

=item $opt_g

=item $opt_c

=item $opt_t

=item $opt_e

=item $opt_s

=item $opt_i

=back

=cut

use vars qw($opt_p $opt_u $opt_d $opt_a $opt_o $opt_f $opt_g $opt_c $opt_t $opt_e $opt_s $opt_i);
##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
require "getopts.pl";
&Getopts('p:u:d:a:o:f:m:g:c:t:e:s:z:q:i:');
if (!$opt_u || !$opt_p || !$opt_d || !$opt_o || !$opt_a || !$opt_f || !$opt_g || !$opt_t){
   print "Usage: insilico_digest.pl\n";
   print "-u username\n";
   print "-p password\n";
   print "-d database\n";
   print "-o host name\n";
   print "-a assembly id\n";
   print "-f map id\n";
   print "-g directory to store gif (full path)\n";
   print "-t Clone type (new, single, buried, etc.)\n";
   print "=" x 50, "\n";
   print "-c /fullpath/fof clones to digest OPTIONAL\n";
   print "--------------->OR<--------------\n";
   print "=" x 50, "\n";
   print "-i contig number to perform digest on OPTIONAL\n";
   print "-s start coordinate for fake fingerprint OPTIONAL\n";
   print "-e stop coordinate for fake fingerprint OPTIONAL\n";
   die "*Note: If you use the option -i, you MUST set the value for -s AND -e\n"; 
}
my $user_info;
$user_info->{'username'}=$opt_u;
$user_info->{'password'}=$opt_p;
$user_info->{'host_name'}=$opt_o;
$user_info->{'database'}=$opt_d;
my $assembly_id=$opt_a;
my $map_id=$opt_f;
my $clones_fof=$opt_c;
my $gif_dir=$opt_g;
my $clone_type=$opt_t;
####Constants
my $M=-400;#original:-250 ###slope=-0.002, 1/x=-500/2(respect image size)
my $K=4.7; #original:4.5  ###migration coefficient
my $EXP_START=45;
my $EXP_END=95;
my $BAND_THICK=1;
my $EXP_TXT_START=1;
my $GEL_HEAD=40;
my $SIL_START=105;
my $SIL_END=155;
my $SIL_TXT_START=165;
my $SIL_SPC_START=225;
my $TXT_HEAD1=2;
my $TXT_HEAD2=16;
my $TXT_HEAD3=30;
my $SIZE_LIMIT=700;
my $ERROR=0.1;      #+/- error allowed on band size
my $SIL_ARROW_START=185;
my $STEP_INCREASE=15;
my $TEXT_TAIL=880;
my $RESOLUTION_BOUNDARY=($M*(log10($SIZE_LIMIT)-$K)+$GEL_HEAD);
my $WIDTH=360;
my $LENGTH=900;
my @date_format=("%Y-%m-%d %H:%M:%S");
my ($clone_select, $flag_clone);
if($opt_c){
   $clone_select=&readFOF($clones_fof);
   print Dumper($clone_select);
   $flag_clone=1;
}elsif($opt_i){
   $clone_select->{'contig_number'}=$opt_i;
   $clone_select->{'contig_start'}=$opt_s;
   $clone_select->{'contig_stop'}=$opt_e;
   if($opt_s>=$opt_e){
      die "-s option must be smaller than -e\n";
   }
   $flag_clone=0;
}else{
   die "You must select the -c or -n option, not both or none.\n";
}
my ($contig, $template, $position, $map_clone, $contig_sequence, $total)=&getInfo($user_info, $assembly_id, $map_id, $clone_select, $clone_type, $flag_clone);
$map_clone=&identifyProperFosmidClone($contig, $template, $position, $total, $map_clone, $contig_sequence);
$map_clone=&digestFosmid($map_clone);
&processDigestData($map_clone, $gif_dir);
exit(0);

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#readFOF'>readFOF; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#processDigestData'>processDigestData; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#compareXtoY'>compareXtoY; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#printInvalidGel'>printInvalidGel; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#printValidGel'>printValidGel; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeDrawing'>initializeDrawing; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeColor'>initializeColor; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#digestFosmid'>digestFosmid; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#identifyProperFosmidClone'>identifyProperFosmidClone; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#sendEmail'>sendEmail; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getInfo'>getInfo; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeEnzyme'>reorganizeEnzyme; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#readConfigFile'>readConfigFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeRead'>reorganizeRead; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeFakeClone'>reorganizeFakeClone; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeClone'>reorganizeClone; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeTig'>reorganizeTig; </A></strong></font><BR>

=over

=for html <A NAME='readFOF'>

=item readFOF

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub readFOF{

   my $file=shift;

   my $clone_select;

   open (OUT, $file) || die "Can't open $file.\n";
   while(<OUT>){
      chomp;
      if(/^(\S+)\s?/){
         $clone_select->{$1}++;
      }
   }

   close OUT;
   return $clone_select;
}

=for html <A NAME='processDigestData'>

=item processDigestData

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub processDigestData{

   my ($map_clone, $image_dir)=@_;

   foreach my $clone (keys %$map_clone){

      my $image_file=$clone.".gif";

      if(! -d $image_dir){
         my $cmd="mkdir $image_dir";
         system($cmd);
      }

      if ($map_clone->{$clone}{'paired_ok'}){
         my $band_color_exp=&compareXtoY($map_clone->{$clone}{'experimental'}, $map_clone->{$clone}{'insilico'});
         my $band_color_sil=&compareXtoY($map_clone->{$clone}{'insilico'}, $map_clone->{$clone}{'experimental'});
         ###more analysis here
         &printValidGel($clone, $map_clone->{$clone}, $band_color_exp, $band_color_sil, $image_dir, $image_file);
      }else{
         &printInvalidGel($clone, $map_clone->{$clone}, $image_dir, $image_file);
      }
    }
}

=for html <A NAME='compareXtoY'>

=item compareXtoY

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub compareXtoY{

   my ($upper_array, $lower_array)=@_;

   my $band_color;
   foreach my $band_up(@$upper_array){
      my $match=0;
      my $high_boundary=$band_up+($ERROR*$band_up);   ###Error Allowed +/- $ERROR     
      my $low_boundary=$band_up-($ERROR*$band_up);

      foreach my $band_lo(@$lower_array){
         if($band_lo >$low_boundary && $band_lo <$high_boundary){
            $match=1;  ###there's a band with close M.W., even if don't match in numbers
         }
      }
      
      if($match){$band_color->{$band_up}="black";}else{$band_color->{$band_up}="red";}
   }

   return $band_color;
}

=for html <A NAME='printInvalidGel'>

=item printInvalidGel

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub printInvalidGel{

   my ($clone, $map_clone_data, $image_dir, $image_file)=@_;

   my $image=&initializeDrawing($WIDTH,$LENGTH); #Define Drawing Object
   my $color=&initializeColor($image);

   open (IMG, ">$image_dir/$image_file") || die "can't open $image_dir/$image_file for writing.\n";
   binmode IMG;

   ####START MAKING GRAPHIC

   $image->string(gdLargeFont,$EXP_TXT_START, $TXT_HEAD1, $clone, $color->{$map_clone_data->{'buried'}{'color'}});
   $image->string(gdSmallFont,$SIL_START, $TXT_HEAD1, "$map_clone_data->{'buried'}{'name'} / $map_clone_data->{'enzyme'} Digest", $color->{$map_clone_data->{'buried'}{'color'}});
   $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD2, "$map_clone_data->{'comment'}", $color->{'black'});
   $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD3, "DATA NOT AVAILABLE", $color->{'red'});

   #-----------------------
   #Convert the image to PNG and print it on standard output
   print IMG $image->gif;#->png#(not supported in current version)
   close IMG;

}

=for html <A NAME='printValidGel'>

=item printValidGel

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub printValidGel{

    my ($clone, $map_clone_data, $band_color_exp, $band_color_sil, $image_dir, $image_file)=@_;

    my $experimental=$map_clone_data->{'experimental'};
    my $insilico=$map_clone_data->{'insilico'};

    my $image=&initializeDrawing($WIDTH,$LENGTH); #Define Drawing Object
    my $color=&initializeColor($image);

    open (IMG, ">$image_dir/$image_file") || die "can't open $image_dir/$image_file for writing.\n";
    binmode IMG;

    ####START MAKING GRAPHIC

    $image->string(gdLargeFont,$EXP_TXT_START, $TXT_HEAD1, $clone, $color->{$map_clone_data->{'buried'}{'color'}});
    $image->string(gdSmallFont,$SIL_START, $TXT_HEAD1, "$map_clone_data->{'buried'}{'name'} / $map_clone_data->{'enzyme'} Digest", $color->{$map_clone_data->{'buried'}{'color'}});
    $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD2, "Contig $map_clone_data->{'contig_number'} at $map_clone_data->{'left_coord'}-$map_clone_data->{'right_coord'} bp  InsertSize=$map_clone_data->{'stop'} bp", $color->{'black'});
    $image->dashedLine(0,$RESOLUTION_BOUNDARY,$WIDTH,$RESOLUTION_BOUNDARY, $color->{'grey'});

    #-----------------------
    my ($exp_band_seen, $silico_band_seen);

    $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD3, "EXPERIMENTAL", $color->{'navy'});
    foreach my $band(@$experimental){###experimental bands

       my $pixel = $M*(log10($band)-$K); ###will return migration distance for every fragment
       my $migration=$GEL_HEAD+$pixel;
       $image->filledRectangle($EXP_START,$migration,$EXP_END,$migration+$BAND_THICK, $color->{$band_color_exp->{$band}});

       if(defined $exp_band_seen->{$band}){
           my $current_count=$exp_band_seen->{$band}+1;
           my $previous=$band." ($exp_band_seen->{$band})";
           my $current=$band." ($current_count)";
           $image->string(gdTinyFont,$EXP_TXT_START, $migration, $previous, $color->{'white'});##erases previous entry
           $image->string(gdTinyFont,$EXP_TXT_START, $migration, $current, $color->{'black'});
       }else{
           $image->string(gdTinyFont,$EXP_TXT_START, $migration, $band, $color->{'black'});
       }

       $exp_band_seen->{$band}++; ###keeps track of multiplet bands
    }
    my $ladder=$TEXT_TAIL;
    $image->string(gdSmallFont,$SIL_START, $TXT_HEAD3, "IN-SILICO", $color->{'forrest'});

    my $alternate="salmon";
    my $flag=0;

    foreach my $band(@$insilico){###in silico digest
       my $pixel = $M*(log10($band)-$K); ###will return migration distance for every fragment
       my $migration=$GEL_HEAD+$pixel;

       $image->filledRectangle($SIL_START,$migration,$SIL_END,$migration+$BAND_THICK, $color->{$band_color_sil->{$band}});

       if($migration<=$RESOLUTION_BOUNDARY){
          $ladder-=$STEP_INCREASE;

          $image->line($SIL_ARROW_START,$migration,$SIL_SPC_START, $ladder, $color->{$alternate});      ####matching line
          $image->string(gdTinyFont,$SIL_SPC_START, $ladder, "($band) $map_clone_data->{'fragment_position'}{$band}", $color->{'black'}); ###more info

          if($flag){
             $flag=0;
             $alternate="salmon"
          }else{
             $flag=1;
             $alternate="lightblue";
          }
       }
       if(defined $silico_band_seen->{$band}){
           my $current_count=$silico_band_seen->{$band}+1;
           my $previous=$band." ($silico_band_seen->{$band})";
           my $current=$band." ($current_count)";
           $image->string(gdTinyFont,$SIL_TXT_START, $migration, $previous, $color->{'white'});##erases previous entry
           $image->string(gdTinyFont,$SIL_TXT_START, $migration, $current, $color->{'black'});
       }else{
           $image->string(gdTinyFont,$SIL_TXT_START, $migration, $band, $color->{'black'});
       }

       $image->string(gdTinyFont,$SIL_TXT_START, $migration, $band, $color->{'black'});   ####band size
       $silico_band_seen->{$band}++; ###keeps track of multiplet bands
    }

    #-----------------------
    #Convert the image to PNG and print it on standard output
    print IMG $image->gif;#->png#(not supported in current version)
    close IMG;

}

=for html <A NAME='initializeDrawing'>

=item initializeDrawing

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-------------------------------------
sub initializeDrawing{

   my ($width, $length)=@_;

   my $image = new GD::Image($width,$length);

   return $image;
}

=for html <A NAME='initializeColor'>

=item initializeColor

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$image</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub initializeColor{

   my $image=shift;
   my $color;

   # allocate some colors
   $color->{white} = $image->colorAllocate(255,255,255);
   $color->{black} = $image->colorAllocate(0,0,0);
   $color->{swamp} = $image->colorAllocate(150,150,30);
   $color->{blue} = $image->colorAllocate(0,102,204);
   $color->{yellow}= $image->colorAllocate(255,255,0);
   $color->{cyan}=$image->colorAllocate(0,255,255);
   $color->{purple}=$image->colorAllocate(255,0,255);
   $color->{green}=$image->colorAllocate(100,250,25);
   $color->{red}=$image->colorAllocate(250,25,75);
   $color->{forrest}= $image->colorAllocate(25,175,0);
   $color->{dirtyred}=$image->colorAllocate(200,0,120);
   $color->{navy}=$image->colorAllocate(0,0,150);
   $color->{dirtyyellow}=$image->colorAllocate(200,200,75);
   $color->{grey}=$image->colorAllocate(153, 153, 153);
   $color->{lightgrey}=$image->colorAllocate(220, 220, 220);
   $color->{salmon}=$image->colorAllocate(255,153,153);
   $color->{lightblue}=$image->colorAllocate(153,204,255);
   $color->{orange}=$image->colorAllocate(255,153,51);

   return $color;
}

=for html <A NAME='digestFosmid'>

=item digestFosmid

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>($map_clone)</B>():  [Default = <B>@_</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub digestFosmid{

   my ($map_clone)=@_;

   foreach my $clone (keys %$map_clone){

      if($map_clone->{$clone}{'paired_ok'}){

         my $enzyme=$map_clone->{$clone}{'enzyme'};
         my $pattern=$map_clone->{$clone}{'pattern'};
         my @experimental=split(" ", $map_clone->{$clone}{'digest'});
         my @sites=();
         while($map_clone->{$clone}{'sequence'} =~ /$pattern/ig){    ###look for position of specific pattern
            @sites = (@sites, pos $map_clone->{$clone}{'sequence'});
         }
         my $previous=$map_clone->{$clone}{'start'};
         my @all_fragment;
         foreach my $position(@sites){
            my $size=($position+$map_clone->{$clone}{'cut_site'})-$previous;
            push @all_fragment, $size;
            my $fragment_position;
            if(! $previous){###this is the first fragment
               $fragment_position="V_".($map_clone->{$clone}{'left_coord'}+$previous)."-".($map_clone->{$clone}{'left_coord'}+$position);
            }else{          ###other fragments
               $fragment_position=($map_clone->{$clone}{'left_coord'}+$previous)."-".($map_clone->{$clone}{'left_coord'}+$position);
            }
            $map_clone->{$clone}{'fragment_position'}{$size}=$fragment_position;
            $previous=$position+$map_clone->{$clone}{'cut_site'};
         }
         my $last_size=$map_clone->{$clone}{'stop'}-$previous;

         my $fragment_position=$map_clone->{$clone}{'left_coord'}+$previous."-".$map_clone->{$clone}{'right_coord'}."_V";
         $map_clone->{$clone}{'fragment_position'}{$last_size}=$fragment_position;

         push @all_fragment, $last_size;               ###convert band string into array
         my @insilico=sort { $a <=> $b} @all_fragment; ###order array

         $map_clone->{$clone}{'experimental'}=\@experimental;
         $map_clone->{$clone}{'insilico'}=\@insilico;

         foreach my $band(@insilico){
            $map_clone->{$clone}{'fragment_size_string'}.=$band." ";
            $map_clone->{$clone}{'fragment_position_string'}=$map_clone->{$clone}{'fragment_position'}{$band}." ";
         }
      }
   }
   return $map_clone;
}

=for html <A NAME='identifyProperFosmidClone'>

=item identifyProperFosmidClone

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub identifyProperFosmidClone{

   my ($contig, $template, $position, $total, $map_clone, $contig_sequence)=@_;
 
   my $ct=0;
   my ($been_there, $orientation);

   ###graphical view of assembled reads
   $orientation->{0}="<--";
   $orientation->{1}="-->";

   TEMPLATE:
   foreach my $template_a (sort {$template->{$a}<=>$template->{$b}} keys %$template){
      $ct++;
      my $percent=($ct/$total*100);
      print "$ct out of $total ($percent%)\n";

      $been_there->{$template_a}++;###ensures that only check 1 paired-end

      my  $reverse;

      my $direction=$template->{$template_a}{'direction'};
      my $clone_name=$template->{$template_a}{'clone'};

      my $tig_a=$template->{$template_a}{'contig'};

      if ($direction eq 'F'){$reverse="R";}elsif($direction eq 'R'){$reverse="F";}

      my $template_b=$clone_name.$reverse;###get the partner

      if (! defined $been_there->{$template_b} && defined $template->{$template_b}{'contig'}){###the order here is important for speeding purposes
         my $tig_b=$template->{$template_b}{'contig'};

         if ($tig_a == $tig_b){####Paired Reads located on same contig
            ###check whether the reads are facing each other -->    <---
            ###if it's the case, grab the sequence, note the clone 
                my $dir_a=$position->{$template_a}{'direction'};
                my $dir_b=$position->{$template_b}{'direction'};

                if ($dir_a != $dir_b){
          #         print "$template_a:$position->{$template_a}{'start'}-$position->{$template_a}{'stop'} $orientation->{$dir_a}\n";
          #         print "$template_b:$position->{$template_b}{'start'}-$position->{$template_b}{'stop'} $orientation->{$dir_b}\n";
                   my ($left_coord, $right_coord)=(0,0);
                   ####START READ
                   if ($position->{$template_a}{'start'}<$position->{$template_b}{'start'}){
                      $left_coord=$position->{$template_a}{'start'};
                   }else{
                      $left_coord=$position->{$template_b}{'start'};
                   }

                   ####END READ
                   if ($position->{$template_a}{'stop'}>$position->{$template_b}{'stop'}){
                      $right_coord=$position->{$template_a}{'stop'};
                   }else{
                      $right_coord=$position->{$template_b}{'stop'};
                   }
                   $map_clone->{$clone_name}{'stop'}=abs($right_coord-$left_coord);
                   $map_clone->{$clone_name}{'left_coord'}=$left_coord;
                   $map_clone->{$clone_name}{'right_coord'}=$right_coord;
                   $map_clone->{$clone_name}{'sequence'}=substr($contig_sequence->{$tig_a}, $left_coord, $map_clone->{$clone_name}{'stop'});  
                   $map_clone->{$clone_name}{'start'}=0;
                   $map_clone->{$clone_name}{'paired_ok'}=1;
                   $map_clone->{$clone_name}{'comment'}="Valid and logical paired-end reads.  Insert size is $map_clone->{$clone_name}{'stop'}."; 
                }else{###both reads point in the same direction
                   $map_clone->{$clone_name}{'paired_ok'}=0; ###no sequence since problematic clone, but keep track of clone
                   $map_clone->{$clone_name}{'comment'}="Illogical:$template_a is $orientation->{$dir_a}/$template_b $orientation->{$dir_b}";
                   next TEMPLATE;
                }

         }else{###paired on different contigs
            $map_clone->{$clone_name}{'paired_ok'}=0; ###no sequence since problematic clone fo insilico digests, but keep track of clone
            $map_clone->{$clone_name}{'comment'}="$template_a on tig$tig_a and $template_b on different tig$tig_b";
         }
      }else{###check pairs only once
         if(! defined $template->{$template_b}{'contig'}){
            $map_clone->{$clone_name}{'paired_ok'}=0; ###no sequence since problematic clone, but keep track of clone
            $map_clone->{$clone_name}{'comment'}="$template_b not found in the sequence assembly";
         }
      }
   }
   return $map_clone;
}

=for html <A NAME='sendEmail'>

=item sendEmail

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$email</B>():  [Default = <B>shift</B>]<BR>
- <B>$message</B>():  [Default = <B>shift</B>]<BR>
- <B>$map_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$jobqueue_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$status</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>###must send the Job_Queue to the user</Font><BR>
</Font><BR>

=cut

#-------------------------------
sub sendEmail{

   my $email=shift;
   my $message=shift;
   my $map_id=shift;
   my $assembly_id=shift;
   my $jobqueue_id=shift;
   my $status=shift;

   ###must send the Job_Queue to the user

   my $comment="Your Job#$jobqueue_id:\nBinding the fingerprint map (Map_ID=$map_id) to the sequence assembly (Assembly_ID=$assembly_id)  $status.\n$message";
   CommonTask::sendMail($email, $comment, $jobqueue_id) if ($email ne '');

}

=for html <A NAME='getInfo'>

=item getInfo

=for html
Gather all the info pertaining to the map and seq. assembly<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#------------------------------------
sub getInfo{
   #Gather all the info pertaining to the map and seq. assembly

   my ($user_info, $assembly_id, $map_id, $clone_select, $clone_type, $flag_clone)=@_;
 
   my $dbh=SQLdb::Initialize($user_info->{'password'}, $user_info->{'username'},$user_info->{'database'},$user_info->{'host_name'});

   my $tmp_tig=SAMQueryAPI::getContigNSupertigStats($dbh, $assembly_id);
   my $tmp_config=SAMQueryAPI::getFileDataConfig($dbh, $assembly_id);
   my $tmp_enzyme=SAMQueryAPI::getEnzyme($dbh, $map_id);                    ###Will grab enzyme info from the map
   
   my ($contig)=&reorganizeTig($tmp_tig);
   my ($enzyme_info)=&reorganizeEnzyme($tmp_enzyme);
   my ($contig_sequence)=&readConfigFile($tmp_config);

   my ($template, $position, $map_clone, $total);

   if ($flag_clone){
      my $tmp_spec=SAMQueryAPI::getReadCoordBuried($dbh, $assembly_id, $map_id);
      my $tmp_all=SAMQueryAPI::getReadCoordAllClone($dbh, $assembly_id);       ###Will grab all clones sequenced
      my ($fosmid_digest)=&reorganizeRead($tmp_spec);
      ($template, $position, $map_clone, $total)=&reorganizeClone($tmp_all, $fosmid_digest, $enzyme_info, $clone_select, $clone_type);
   }else{
      ($template, $position, $map_clone, $total)=&reorganizeFakeClone($enzyme_info, $clone_select, $clone_type);
   }
   SQLdb::Disconnect($dbh);

   return $contig, $template, $position, $map_clone, $contig_sequence, $total;
}

=for html <A NAME='reorganizeEnzyme'>

=item reorganizeEnzyme

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_enzyme</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub reorganizeEnzyme{

   my $tmp_enzyme=shift;
   my @enzyme_array=@$tmp_enzyme;

   my $enzyme_info;
   foreach my $record(@$tmp_enzyme){
      $enzyme_info->{'name'}=$enzyme_array[0];
      $enzyme_info->{'pattern'}=$enzyme_array[1];
      $enzyme_info->{'cut_site'}=$enzyme_array[2];
   }
   return $enzyme_info;
}

=for html <A NAME='readConfigFile'>

=item readConfigFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub readConfigFile{

   my $tmp_config=shift;

   my @record=@$tmp_config;

   my $build=$record[3]."_".$record[4];
   my $contig_file=$record[0]."/".$record[1]."/".$build."/".$record[2]."/".$record[5];

   open (TIG, $contig_file) || die "can't open $contig_file\n";

   my $contig_sequence;
   my $sequence;
   my $contig_number=-1;
   while(<TIG>){
      chomp;
 
      if(/^\>\D+(\d+)(\D+)?/){
         my $new_tig=$1;
         if($contig_number != -1){
            $contig_sequence->{$contig_number}=$sequence;
         }
         $sequence='';
         $contig_number=$new_tig;

      }elsif(/^([agctnx]*[acgtnx])$/i){
         $sequence.=$1;
      }
   }
   if($contig_number != -1){
      $contig_sequence->{$contig_number}=$sequence;
      #print "$contig_number $sequence\n";
   }

   close TIG;
   return $contig_sequence;
}

=for html <A NAME='reorganizeRead'>

=item reorganizeRead

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_spec</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub reorganizeRead{

   my $tmp_spec=shift;
   
   my $fosmid_digest;

   foreach my $record (@$tmp_spec){

      my ($buried_type, $buried_color);
      if($record->[10]){$buried_type="buried";$buried_color="grey";}else{$buried_type="not buried";$buried_color="dirtyred"}

      $fosmid_digest->{$record->[0]}{'digest'}=$record->[12];
      $fosmid_digest->{$record->[0]}{'enzyme'}=$record->[13];
      $fosmid_digest->{$record->[0]}{'pattern'}=$record->[14];          ###Enzyme Recognition site
      $fosmid_digest->{$record->[0]}{'cut_site'}=$record->[15];         ###Position of enzyme cut site
      $fosmid_digest->{$record->[0]}{'buried'}{'name'}=$buried_type;    ###Is it a buried clone?
      $fosmid_digest->{$record->[0]}{'buried'}{'color'}=$buried_color;
   }

   return $fosmid_digest;
}

=for html <A NAME='reorganizeFakeClone'>

=item reorganizeFakeClone

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub reorganizeFakeClone{
 
   my ($enzyme_info, $clone_select, $clone_type)=@_;

   my $total=0;
   my ($template, $position, $map_clone);

   $total=2;

   ###First "Read"

   my $template_name="RegionF";           ###RF00123aB02.F/R/U
   $template->{$template_name}{'contig'}=$clone_select->{'contig_number'};
   $template->{$template_name}{'direction'}="F"; ###F/R/U
   $template->{$template_name}{'clone'}="Region";

   $position->{$template_name}{'start'}=$clone_select->{'contig_start'};
   $position->{$template_name}{'stop'}=$clone_select->{'contig_start'}+700;
   $position->{$template_name}{'direction'}=1; ###1/0 (1 being true, --> relative to contig orientation)

   ###Second "Read"
   $template_name="RegionR";

   $template->{$template_name}{'contig'}=$clone_select->{'contig_number'};
   $template->{$template_name}{'direction'}="R"; ###F/R/U
   $template->{$template_name}{'clone'}="Region";

   $position->{$template_name}{'start'}=$clone_select->{'contig_stop'}-700;
   $position->{$template_name}{'stop'}=$clone_select->{'contig_stop'};
   $position->{$template_name}{'direction'}=0; ###1/0 (1 being true, --> relative to contig orientation)   

   ###Clone info
   my $clone="Region";

   $map_clone->{$clone}{'digest'}="";
   $map_clone->{$clone}{'enzyme'}=$enzyme_info->{'name'};
   $map_clone->{$clone}{'pattern'}=$enzyme_info->{'pattern'};          ###Enzyme Recognition site
   $map_clone->{$clone}{'cut_site'}=$enzyme_info->{'cut_site'};
   $map_clone->{$clone}{'buried'}{'name'}=$clone_type;
   $map_clone->{$clone}{'buried'}{'color'}="lightblue";
   $map_clone->{$clone}{'contig_number'}=$clone_select->{'contig_number'};     ###for image generating purposes

   return $template, $position, $map_clone, $total;
}

=for html <A NAME='reorganizeClone'>

=item reorganizeClone

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub reorganizeClone{

   my ($tmp_all, $fosmid_digest, $enzyme_info, $clone_select, $clone_type)=@_;

   my $total=0;
   my ($template, $position, $map_clone);
   foreach my $record (@$tmp_all){

    if(defined $clone_select->{$record->[0]}){####Is it one of the chosen close from the fof?

      $total++;
      
      my $template_name=$record->[0].$record->[2];           ###RF00123aB02.F/R/U
      $template->{$template_name}{'contig'}=$record->[1];
      $template->{$template_name}{'direction'}=$record->[2]; ###F/R/U
      $template->{$template_name}{'clone'}=$record->[0];

      $position->{$template_name}{'start'}=$record->[3];
      $position->{$template_name}{'stop'}=$record->[4];
      $position->{$template_name}{'direction'}=$record->[5]; ###1/0 (1 being true, --> relative to contig orientation)

      if(defined $fosmid_digest->{$record->[0]}){

         $map_clone->{$record->[0]}{'digest'}=$fosmid_digest->{$record->[0]}{'digest'};
         $map_clone->{$record->[0]}{'enzyme'}=$fosmid_digest->{$record->[0]}{'enzyme'};
         $map_clone->{$record->[0]}{'pattern'}=$fosmid_digest->{$record->[0]}{'pattern'};          ###Enzyme Recognition site
         $map_clone->{$record->[0]}{'cut_site'}=$fosmid_digest->{$record->[0]}{'cut_site'};         ###Position of enzyme cut site
         $map_clone->{$record->[0]}{'buried'}{'name'}=$fosmid_digest->{$record->[0]}{'buried'}{'name'};    ###Is it a buried clone?
         $map_clone->{$record->[0]}{'buried'}{'color'}=$fosmid_digest->{$record->[0]}{'buried'}{'color'};  
         $map_clone->{$record->[0]}{'contig_id'}=$record->[7];        ###for database insert
         $map_clone->{$record->[0]}{'contig_number'}=$record->[1];     ###for image generating purposes

      }else{

         $map_clone->{$record->[0]}{'digest'}='';
         $map_clone->{$record->[0]}{'enzyme'}=$enzyme_info->{'name'};
         $map_clone->{$record->[0]}{'pattern'}=$enzyme_info->{'pattern'};          ###Enzyme Recognition site
         $map_clone->{$record->[0]}{'cut_site'}=$enzyme_info->{'cut_site'};         ###Position of enzyme cut site
         $map_clone->{$record->[0]}{'buried'}{'name'}=$clone_type;    ###Is it a buried clone?
         $map_clone->{$record->[0]}{'buried'}{'color'}="forrest";
         $map_clone->{$record->[0]}{'contig_id'}=$record->[7];        ###for database insert
         $map_clone->{$record->[0]}{'contig_number'}=$record->[1];     ###for image generating purposes
      }
    }
   }

   return $template, $position, $map_clone, $total;
}

=for html <A NAME='reorganizeTig'>

=item reorganizeTig

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_tig</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------
sub reorganizeTig{

   my $tmp_tig=shift;
 
   my $contig;
   foreach my $record (@$tmp_tig){
      $contig->{$record->[0]}{'supertig'}=$record->[1];
      $contig->{$record->[0]}{'order'}=$record->[3];
      $contig->{$record->[0]}{'size'}=$record->[4];
      $contig->{$record->[0]}{'gap'}=$record->[5];
      $contig->{$record->[0]}{'id'}=$record->[7];
   }

   return $contig;
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>

=cut

