#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

map2xml.pl - 

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Author:Rene Warren<BR>Script:map2xml.pl<BR>Purpose:Creates an xml file (model file) for inputing fingerprint data in SAMdb<BR>June 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item Date::Manip

=item Plate

=back

=cut

#Author:Rene Warren
#Script:map2xml.pl
#Purpose:Creates an xml file (model file) for inputing fingerprint data in SAMdb
#June 2003
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use Date::Manip;
use Plate;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=item $opt_f

=item $opt_v

=item $opt_c

=item $opt_a

=item $opt_r

=item $opt_i

=item $opt_s

=item $opt_e

=item $opt_l

=back

=cut

use vars qw($opt_f $opt_v $opt_c $opt_a $opt_r $opt_i $opt_s $opt_e $opt_l);
##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
require "getopts.pl";
&Getopts('f:v:c:a:r:i:s:e:l:');
#####TRAP ERRORS#####
#INVALID OPTION
if (! $opt_f || ! $opt_v || ! $opt_c || ! $opt_a || ! $opt_r || ! $opt_i || ! $opt_l){
   die "Usage: map2xml.pl -f xml file (to be created) -i overlap file -l library fingerprinted (e.g.RF001) -s experimental digests file (optional) -e enzyme used (optional) -v version -c comment -a date -r directory to store xml\n";
}elsif (! -d $opt_r){
   die "$opt_r invalid.\n";
}elsif (! -e "$opt_r/$opt_i"){
   print "$opt_r/$opt_i\n";
   die "Please Enter a valid file in the $opt_r directory.\n";
}
#####Do not make the digest file mandatory#####
my ($ignore_digest,$digest_file, $enzyme);
if($opt_s){
   if(! -e "$opt_r/$opt_s"){
      print "$opt_r/$opt_s\n";
      die "Please Enter a valid file in the $opt_r directory.\n";
   }else{
      $ignore_digest=0;
      $digest_file=$opt_s;
   }
   if(! $opt_e){
      die "You must enter a value for the enzyme used to generate the fragments.\n";
   }else{
      $enzyme=$opt_e;
   }
}else{
   $opt_e='';
   $ignore_digest=1;
}
my $file2create=$opt_f;
my $overlap_file=$opt_i;
my $comment=$opt_c;
my $version=$opt_v;
my $date=$opt_a;
my $filedir=$opt_r;
my $finger_lib=$opt_l;
my %plate_layout;
my $plate=new Plate(\%plate_layout);
my $digest;
$plate->transform384to96();
&initializeXml($file2create, $filedir, $date, $comment, $version);
$digest=getDigests($digest_file, $filedir) if (! $ignore_digest);
my ($map, $size)=&getInfo($overlap_file, $filedir, $plate, $digest, $enzyme, $finger_lib);
&createCoreXml($map, $size, $file2create, $filedir);
&endXml($file2create, $filedir);
exit;

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#getDigests'>getDigests; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getInfo'>getInfo; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#endXml'>endXml; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeXml'>initializeXml; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#createCoreXml'>createCoreXml; </A></strong></font><BR>

=over

=for html <A NAME='getDigests'>

=item getDigests

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-------------------------------------------
sub getDigests{

   my ($digest_file, $filedir)=@_;
   my ($digest, $name, $fragment);

   open (DIG, "$filedir/$digest_file") or die "Can't open $filedir/$digest_file.\n";
   
   while (<DIG>){
      chomp;
      if(/^\s+(\S+)\s+/){
         #print "$1\n";
         $name=$1;
      }elsif(/^(\d+)\s?$/){
         $fragment.=$1." ";   ###Will add a space after each fragment sizes
      }elsif(/^\-/){
         $digest->{$name}=$fragment if ($name ne '');
         print "$name..$fragment\n";
         $name='';
         $fragment='';
      }
   }
   close DIG;

   return $digest;
}

=for html <A NAME='getInfo'>

=item getInfo

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$filedir</B>():  [Default = <B>shift</B>]<BR>
- <B>$plate</B>():  [Default = <B>shift</B>]<BR>
- <B>$digest</B>():  [Default = <B>shift</B>]<BR>
- <B>$enzyme</B>():  [Default = <B>shift</B>]<BR>
- <B>$finger_lib</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------
sub getInfo{

   my $file=shift;
   my $filedir=shift;
   my $plate=shift;
   my $digest=shift;
   my $enzyme=shift;
   my $finger_lib=shift;

   my %hash;

   open (OUT, "$filedir/$file") or die "Can't open $file!\n";

   my $contig;
   my (%map, %size);
   my $order=0;

   while (<OUT>){
      chomp;
      if(/\>(\d+)$/){
         $contig=$1;
         $order=0;
      }elsif(/(.*)(\D{1})(\d+)(\-)?/){
         $order++;

         my $padded_plate=$1;
         my $well_row=$2;
         my $extra_char=$4;       
         my $well_number=$3;

         my $concat=$well_row.$well_number;

         my $well_96=$plate->{$concat};
         
         my $name_plate=$finger_lib.$1 if ($padded_plate=~/0+(\d+)/);      ###Can do this because it's not part of the distribution
         my $clone=$name_plate.$well_96;
         my $altname=$padded_plate.$concat;

         my $buried=0; 
         if($extra_char){$buried=1};
         $map{$contig}{$clone}{'buried'}=$buried;
         $map{$contig}{$clone}{'order'}=$order;
         $map{$contig}{$clone}{'altname'}=$altname;
         $map{$contig}{$clone}{'digest'}=$digest->{$altname};
         $map{$contig}{$clone}{'enzyme'}=$enzyme;
         #print "$contig---$clone---$altname---$order---$buried  $name_plate\n";
      }
   }
   close OUT;

   return \%map, \%size;
}

=for html <A NAME='endXml'>

=item endXml

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$filedir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------
sub endXml{

   my $file=shift;
   my $filedir=shift;

   open (IN, ">>$filedir/$file") or die "Can't open $file\n";

   print IN "\t</map_volume>";   

   close IN;

}

=for html <A NAME='initializeXml'>

=item initializeXml

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$filedir</B>():  [Default = <B>shift</B>]<BR>
- <B>$date</B>():  [Default = <B>shift</B>]<BR>
- <B>$comment</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------
sub initializeXml{

   my $file=shift;
   my $filedir=shift;
   my $date=shift;
   my $comment=shift;

   open (IN, ">$filedir/$file") or die "Can't open $file\n";

   #Create XML according to the NCBI's trace archive RFC
   print IN "<?xml version=\"1.0\"?>\n";   
   print IN "\t<map_volume>\n";
   print IN "\t<volume_name>Fingerprint Map Layout</volume_name>\n";
   print IN "\t<map_date>$date</map_date>\n";
   print IN "\t<map_comment>$comment</map_comment>\n";
   print IN "\t<map_version>$version</map_version>\n";

   close IN;

} #end sub

=for html <A NAME='createCoreXml'>

=item createCoreXml

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$map</B>():  [Default = <B>shift</B>]<BR>
- <B>$size</B>():  [Default = <B>shift</B>]<BR>
- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$filedir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------
sub createCoreXml{

   my $map=shift;
   my %map=%$map;
   my $size=shift;
   my %size=%$size;
   my $file=shift;
   my $filedir=shift;

   open (IN, ">>$filedir/$file") or die "Can't open $file\n";

   foreach my $contig (sort {$a<=>$b} keys %map){
   
      print IN "\t\t<contig number=\"$contig\">\n";
      print IN "\t\t\t<contig_number>$contig</contig_number>\n";
      print IN "\t\t\t<contig_size>$size{$contig}{'size'}</contig_size>\n";
      
      my $tmp_clone=$map{$contig};
      my %clone=%$tmp_clone;

      foreach my $clone (sort {$clone{$a}{'order'}<=>$clone{$b}{'order'}} keys %clone){
         print "$clone{$clone}{'altname'}...$clone{$clone}{'buried'}...$clone{$clone}{'order'}...$clone\n";
         print IN "\t\t\t<clone_name altname=\"$clone{$clone}{'altname'}\" buried=\"$clone{$clone}{'buried'}\" order=\"$clone{$clone}{'order'}\" overlap=\"$clone{$clone}{'overlap'}\" digest=\"$clone{$clone}{'digest'}\" enzyme=\"$clone{$clone}{'enzyme'}\">$clone</clone_name>\n";
      }  
      print IN "\t\t</contig>\n";
   }
   
   close IN;

}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut

