#!/usr/local/bin/perl 
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

phrap_analyze.pl -

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Author:Rene Warren<BR>Script:phrap_analyze.pl<BR>Purpose:Program specific to Phrap for analyzing ace files<BR>January 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item AnalyzeAssembly

=item CreateAssemblyXML

=item Data::Dumper

=back

=cut

#Author:Rene Warren
#Script:phrap_analyze.pl
#Purpose:Program specific to Phrap for analyzing ace files
#January 2003
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use AnalyzeAssembly;
use CreateAssemblyXML;
use Data::Dumper;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
my @data_config=($ARGV[0], $ARGV[1], $ARGV[2], $ARGV[3], $ARGV[4]);
my @user_info=($ARGV[5], $ARGV[6], $ARGV[7], $ARGV[8]);
my @condition=($ARGV[9], $ARGV[10], $ARGV[11], $ARGV[12]);
my ($stats);
my $date=`date`;
chomp($date);
my $acename="$data_config[2]".".fasta.screen.ace.";                                #tentative-->have to be put in the db
my @acefiles=<$data_config[0]/$data_config[1]/$data_config[2]/edit_dir/$acename*> ;#need to get this from config
my $max=0;
foreach my $ace (sort {$b cmp $a} @acefiles){
   my $version=$1 if ($ace=~/fasta\.screen\.ace\.(\d+)/);
   $max=$version if ($version>$max);
}
my $current_ace="$acename$max";
my $acefile="$data_config[0]/$data_config[1]/$data_config[2]/edit_dir/$current_ace";
my $singlet_file="$data_config[2]".".fasta.screen.singlets";
my $discardedreads_file="$data_config[0]/$data_config[1]/$data_config[2]/edit_dir/$singlet_file";
CreateAssemblyXML::startRecord($date, \@data_config, $data_config[3]);#this is essential to specify in which xml file the data will be created
my $initial_stats=&parseAceFile($acefile, \@data_config);
CreateAssemblyXML::endRecord(\@data_config, $data_config[3]);
my $addedread_stats=AnalyzeAssembly::getSinglet(\@user_info, \@condition, $initial_stats, $discardedreads_file);
my $addedgenome_stats=AnalyzeAssembly::getGenomicAssemblyStats(\@user_info, \@condition, $addedread_stats);
CreateAssemblyXML::assemblySummary($date, $addedgenome_stats, \@data_config);
exit;

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#parseAceFile'>parseAceFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeValue'>initializeValue; </A></strong></font><BR>

=over

=for html <A NAME='parseAceFile'>

=item parseAceFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$acefile</B>():  [Default = <B>shift</B>]<BR>
- <B>$data_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#--------------------------------
sub parseAceFile{

   my $acefile=shift;
   my $data_config=shift;
    
   my %tig_length=();
   my ($stats, $tig_hash);

   my $init_stats=initializeValue();
   print "Parsing $acefile...\n";
   ($stats, $tig_hash)=AnalyzeAssembly::parseAce($acefile, $init_stats, \%tig_length, $data_config);

   my %stats=%$stats; 

   $stats{'mean_tig'}=int($stats{'totalbp'}/$stats{'contigs'}) if ($stats{'contigs'} != 0);

   ####call to N50 calculation
   my $N50=AnalyzeAssembly::N_fifty($tig_hash);#call to N50 calculation
   $stats{"N50_length"}=$N50;

   return \%stats;
   
}#end sub

=for html <A NAME='initializeValue'>

=item initializeValue

=for html
initialize values (essential)<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#--------------------------------
sub initializeValue{

   #initialize values (essential)
   my %stats;

   $stats{'smallest_tig'}=100000000;#have to set here since we loop through
   $stats{'biggest_tig'}=0;
   $stats{'all_a'}=0;
   $stats{'all_c'}=0;
   $stats{'all_g'}=0;
   $stats{'all_t'}=0;
   $stats{'consensus'}=0;
   $stats{'totalbp'}=0;
   $stats{'error_rate'}=0;
   $stats{'average_phred'}=0;
   $stats{'phred20'}=0;
   $stats{'phred30'}=0;
   $stats{'phred40'}=0;
   $stats{'contigs'}=0;
   $stats{'tigsize'}=0;
   $stats{'assembled_reads'}=0;
   $stats{'1'}=0;
   $stats{'2'}=0;
   $stats{'3'}=0;
   $stats{'4'}=0;
   $stats{'5'}=0;
   $stats{'6+'}=0;
   $stats{'10+'}=0;
   $stats{'20+'}=0;

   return \%stats;

}#end sub

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut

