#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

run_assembler.pl -

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Author:Rene Warren<BR>Script:run_assembler.pl<BR>Purpose:orchestrates assembler run, xml import, assembly analyses, gene prediction & annotation, blastable databases creation<BR>Oct 30, 2002/Feb 14 2003/Apr 4th 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item Date::Manip

=item SQLdb

=item CommonTask

=item Data::Dumper

=item XMLparser

=item CustomAnalysis

=item Orf2Database

=item Cache::FileCache

=back

=cut

#Author:Rene Warren
#Script:run_assembler.pl
#Purpose:orchestrates assembler run, xml import, assembly analyses, gene prediction & annotation, blastable databases creation
#Oct 30, 2002/Feb 14 2003/Apr 4th 2003
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use Date::Manip;
use SQLdb; 
use CommonTask;
use Data::Dumper;
use XMLparser;
use CustomAnalysis;
use Orf2Database;
use Cache::FileCache;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=item $opt_p

=item $opt_u

=item $opt_d

=item $opt_a

=item $opt_o

=item $opt_s

=item $opt_e

=item $opt_l

=item $opt_m

=item $opt_i

=item $opt_v

=item $opt_g

=back

=cut

use vars qw($opt_p $opt_u $opt_d $opt_a $opt_o $opt_s $opt_e $opt_l $opt_m $opt_i $opt_v $opt_g);
##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
require "getopts.pl";
&Getopts('p:u:d:a:s:e:o:l:m:i:v:g:');
####Making the cache
my %cache_options = (namespace=>"gene",
                     cache_root=>"/home/tmp/rwarren",
                     default_expires_in=>1500);
my $cache = new Cache::FileCache(\%cache_options);
my @date_format=("%Y-%m-%d %H:%M:%S");
####Arguments
if (! $opt_p || ! $opt_u || ! $opt_d || ! $opt_a || ! $opt_o || ! $opt_l || ! $opt_s || ! $opt_e || ! $opt_i || !$opt_v || !$opt_g){
   die "Usage: run_assembler.pl -u username -p password -d database -a assembler -o host name -s start date condition -e end date condition -l library list (coma seperated e.g. RM001,RM002) -i job queue id -v new build (1:true -1:false) -g gene finder id -m e-mail\n";
}
my %user_info;
$user_info{'username'}=$opt_u;
$user_info{'password'}=$opt_p;
$user_info{'host_name'}=$opt_o;
$user_info{'database'}=$opt_d;
my $assembler=$opt_a;
my $date_start=$opt_s;
my $date_end=$opt_e;
my $library_list=$opt_l;
my $email=$opt_m;
my $jobqueue_id=$opt_i;
my $new_build=$opt_v;
my $gene_finder_id=$opt_g;
#******************** 
my $blastable_tigid=1; ###will always be these ids for the programs
my $blastable_orfid=2;
my $assembly_type="WholeAssembly";
my $configuration=&getConfigurationInfo(\%user_info, $assembler, $new_build);
my ($date_condition, $library_condition)=&buildCondition($date_start, $date_end, $library_list);
#my ($message_assembly, $new_run_dir)=("new", "4.00_WholeAssembly");
#my $status="completed";
my ($message_assembly, $new_run_dir)=&startAssembly(\%user_info, $assembler, $date_condition, $library_condition, $configuration, \@date_format, $jobqueue_id, $assembly_type);
my ($status, $message_analysis)=&startAnalysis(\%user_info, $configuration, $new_run_dir);
my $assembly_id=&getAssemblyID(\%user_info, $assembler, $configuration);####
#added->
if ($status=~/complete/){
   &parseDataIntoDatabase(\%user_info, $configuration, $assembly_id, $date_condition, $new_run_dir);#only parse if assembly was complete
   &updateAssemblyStatus(\%user_info, $assembly_id, $status, \@date_format);#####
   my $blastdna_config=&getBlastableDBConfig(\%user_info, $blastable_tigid);
   &createTigDatabase($configuration, $assembler, $assembly_id, $blastdna_config, $new_run_dir);
   &customAnalyses(\%user_info, $assembly_id);
   if ($gene_finder_id != -1){
      my ($tmp_new_orf, $orf_protein_file, $orf_cds_file, $everygene_file)=&findGene(\%user_info, $configuration, $assembly_id, $gene_finder_id, $new_run_dir);
      &annotateNovelGenes(\%user_info, $tmp_new_orf);
      my $blastprot_config=&getBlastableDBConfig(\%user_info, $blastable_orfid);
      &createOrfDatabase(\%user_info, $configuration, $assembler, $assembly_id, $blastprot_config, $orf_protein_file, $orf_cds_file, $everygene_file, $gene_finder_id, $blastdna_config, $new_run_dir);
   }
}
CommonTask::updateJobStatus(\%user_info, $jobqueue_id, $status, \@date_format);
&sendEmail($email, $message_assembly, $message_analysis, $configuration, $jobqueue_id, $status, $new_run_dir) if ($opt_m);          ###only send email if there seems to be an address
exit(0);

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#getAssemblyID'>getAssemblyID; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#buildCondition'>buildCondition; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getBlastableDBConfig'>getBlastableDBConfig; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#createTigDatabase'>createTigDatabase; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#createOrfDatabase'>createOrfDatabase; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#findGene'>findGene; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#annotateNovelGenes'>annotateNovelGenes; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#customAnalyses'>customAnalyses; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#sendEmail'>sendEmail; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getConfigurationInfo'>getConfigurationInfo; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#startAssembly'>startAssembly; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#updateAssemblyStatus'>updateAssemblyStatus; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#startAnalysis'>startAnalysis; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseDataIntoDatabase'>parseDataIntoDatabase; </A></strong></font><BR>

=over

=for html <A NAME='getAssemblyID'>

=item getAssemblyID

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembler</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub getAssemblyID{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembler=shift;
   my $configuration=shift;
   my %configuration=%$configuration;

   ####re-initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly_ID for specific Build number
   my @assembly_field=("Assembly_ID");
   my @assembly_table=("Assembly", "Assembler");
   my @assembly_condition=("Assembly_Build_ID=$configuration{'run_dir'}", "Assembler_ID=FK_Assembler__ID", "Assembler_Name='$assembler'");
   my $select_assembly=SQLdb::GetRecord(\@assembly_field, \@assembly_table, \@assembly_condition);
   my $tmp_assembly=SQLdb::ExecuteUnique($dbh, $select_assembly);
   my @assembly_arr=@$tmp_assembly;

   SQLdb::Disconnect($dbh);

   return $assembly_arr[0];
}

=for html <A NAME='buildCondition'>

=item buildCondition

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$date_start</B>():  [Default = <B>shift</B>]<BR>
- <B>$date_end</B>():  [Default = <B>shift</B>]<BR>
- <B>$library_list</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub buildCondition{

   my $date_start=shift;
   my $date_end=shift;
   my $library_list=shift;
 
   my $library_condition;

   ####Build date condition
   my $date_condition="Trace_Date_Sequenced>=\'$date_start\' AND Trace_Date_Sequenced<=\'$date_end 23:59:59\'";    #otherwise doesn't include that day

   ####Build library condition
   my @chosen_library=split (/,/, $library_list);#was a coma, but replace by white space (for web display reasons)
   my $library_count=0;
   foreach my $lib (sort @chosen_library){
       $library_condition.=" OR " if ($library_count>0);
       $library_condition.="Library_Name=\'$lib\'";
       $library_count++;
   }

   return $date_condition, $library_condition;
}

=for html <A NAME='getBlastableDBConfig'>

=item getBlastableDBConfig

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$blastable_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub getBlastableDBConfig{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $blastable_id=shift;

   ####Get BLAST DB CREATOR Configuration
   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly Information
   my @field=("Blastable_DB_Exec", "Blastable_DB_Exec_Dir", "Blastable_DB_Location", "Blastable_DB_Node", "Blastable_DB_Wrapper", "Blastable_DB_Wrapper_Dir");
   my @table=("Blastable_DB");
   my @condition=("Blastable_DB_ID=$blastable_id");####1 for blastp
   my $select_blastable=SQLdb::GetRecord(\@field, \@table, \@condition);
   my $tmp_blastable=SQLdb::ExecuteUnique($dbh, $select_blastable);

   SQLdb::Disconnect($dbh);

   return $tmp_blastable; 
}

=for html <A NAME='createTigDatabase'>

=item createTigDatabase

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembler</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_blastable</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_run_dir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub createTigDatabase{

   my $configuration=shift;
   my %configuration=%$configuration;
   my $assembler=shift;
   my $assembly_id=shift;
   my $tmp_blastable=shift;
   my $new_run_dir=shift;   

   my $type="Contig";

   ####input
   my $output_dir="$configuration{'Config_Data_Dir'}/$configuration{'project_name'}/$new_run_dir";
   my $data_output_dir="$output_dir/$configuration{'Config_Output_Dir'}";
   my $tig_fasta_file=$configuration{'Config_Tig_Fasta_File'};
   my $input="$data_output_dir/$tig_fasta_file";

   ####output
   my @blastable=@$tmp_blastable;
   my $exec_cmd="$blastable[1]/$blastable[0]";
   if (! -d "$blastable[2]/$assembly_id"){
      `mkdir $blastable[2]/$assembly_id`;
   }
   my $output="$blastable[2]/$assembly_id/$tig_fasta_file";#htdocs/assembly

   ####Rearrange Fasta file (remove Xs)
   CommonTask::reArrangeFasta($input, $output);
   
   ####Launch DB maker
   my $cmd="nice -12 $blastable[5]/$blastable[4] \'$exec_cmd\' \'$output\' \'$type\' \'$assembler\' \'$new_run_dir\' \'$configuration{'project_name'}\'";
   system("$cmd")==0 or die "$blastable[4] failed.\n"; 
}

=for html <A NAME='createOrfDatabase'>

=item createOrfDatabase

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembler</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_blastable</B>():  [Default = <B>shift</B>]<BR>
- <B>$orf_protein_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$orf_cds_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$everygene_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_blastable_nt</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_run_dir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub createOrfDatabase{

   my $user_info=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $assembler=shift;
   my $assembly_id=shift;
   my $tmp_blastable=shift;
   my $orf_protein_file=shift;
   my $orf_cds_file=shift;
   my $everygene_file=shift;
   my $gene_finder_id=shift;
   my $tmp_blastable_nt=shift;
   my $new_run_dir=shift;

   my $type_aa="ORF_protein";
   my $type_nt="ORF_CDS";

   ####output
   my @blastable=@$tmp_blastable;
   my @blastable_nt=@$tmp_blastable_nt;

   my $exec_cmd="$blastable[1]/$blastable[0]";
   my $exec_cmd_nt="$blastable_nt[1]/$blastable_nt[0]";   

   if (! -d "$blastable[2]/$assembly_id"){
      `mkdir $blastable[2]/$assembly_id`;
   }
   my $aa_output="$blastable[2]/$assembly_id/$orf_protein_file";#htdocs/assembly
   my $cds_output="$blastable[2]/$assembly_id/$orf_cds_file";
   my $everygene_output="$blastable_nt[2]/$everygene_file";
   
   ####input
   CommonTask::dumpOrf($user_info, $aa_output, $cds_output, $assembly_id, $gene_finder_id);
   CommonTask::dumpEveryOrf($user_info, $everygene_output);

   ####Launch DB maker//Orf for specific build//protein
   my $cmd="$blastable[5]/$blastable[4] \'$exec_cmd\' \'$aa_output\' \'$type_aa\' \'$assembler\' \'$new_run_dir\' \'$configuration{'project_name'}\'";
   system("$cmd")==0 or die "$blastable[4] failed.\n"; 

   ####Launch DB maker//Orf for specific build//cds
   my $cmd="$blastable[5]/$blastable[4] \'$exec_cmd_nt\' \'$cds_output\' \'$type_nt\' \'$assembler\' \'$new_run_dir\' \'$configuration{'project_name'}\'";
   system("$cmd")==0 or die "$blastable[4] failed.\n";

   ####Launch DB maker//Every Orf in Database
   my $cmd="$blastable_nt[5]/$blastable_nt[4] \'$exec_cmd_nt\' \'$everygene_output\' \'$type_nt\' \'All\' \'s\' \'$configuration{'project_name'}\'";
   system("$cmd")==0 or die "$blastable_nt[4] failed.\n";

}

=for html <A NAME='findGene'>

=item findGene

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_run_dir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub findGene{

   my $user_info=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $assembly_id=shift;
   my $gene_finder_id=shift;
   my $new_run_dir=shift;

   my $output_dir="$configuration{'Config_Data_Dir'}/$configuration{'project_name'}/$new_run_dir";
   my $data_output_dir="$output_dir/$configuration{'Config_Output_Dir'}";
   my $tig_fasta_file=$configuration{'Config_Tig_Fasta_File'};
   my $tig_qual_file=$configuration{'Config_Tig_Qual_File'};

   my $gf_config=Orf2Database::getGeneFinderConfiguration($user_info, $gene_finder_id);
   my %gf_config=%$gf_config;

   my $cmd="$gf_config{'wrapper_dir'}/$gf_config{'wrapper'} \'$tig_fasta_file\' \'$tig_qual_file\' \'$data_output_dir\' \'$gf_config{'directory'}\' \'$gf_config{'low_qual'}\' \'$gf_config{'maxphred'}\' \'$gf_config{'topology'}\' \'$gf_config{'smallest_gene'}\' \'$gf_config{'cds_file'}\' \'$gf_config{'protein_file'}\' \'$assembly_id\' \'$gf_config{'overlap'}\' \'$gf_config{'training_set'}\'";
   #print "$cmd\n";
   system("$cmd")==0 or die "system $gf_config{'wrapper'} failed.\n";

   ####get cache content
   my $orf_key="quality".$assembly_id;
   my $quality_orf=$cache->get($orf_key);

   ####Update Orf
   my $tmp_new_orf=Orf2Database::updateOrf($user_info, $assembly_id, $quality_orf, $gene_finder_id);

   return $tmp_new_orf, $gf_config{'protein_file'}, $gf_config{'cds_file'}, $gf_config{'everygene_file'};
}

=for html <A NAME='annotateNovelGenes'>

=item annotateNovelGenes

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_new_orf</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub annotateNovelGenes{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $tmp_new_orf=shift;
   my %new_orf=%$tmp_new_orf;   

   #******************** 
   my $blastp_config_id=1;####1 for blastp (no need to put in config, because it's always the value expected here
   #####################

   ####Get Blast Configuration
   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly Information
   my @field=("Blast_Config_Program", "Blast_Config_Exec", "Blast_Config_Dir", "Blast_Config_Database", "Blast_Config_Processor", "Blast_Config_Blastall");
   my @table=("Blast_Config");
   my @condition=("Blast_Config_ID=$blastp_config_id");####1 for blastp
   my $select_blast_config=SQLdb::GetRecord(\@field, \@table, \@condition);
   my $tmp_config=SQLdb::ExecuteUnique($dbh, $select_blast_config);

   SQLdb::Disconnect($dbh);

   my @blast_config=@$tmp_config;

   my $blast_parser="$blast_config[2]/$blast_config[1]";

   ####Start Blasting
   foreach my $orf_id (sort {$a<=>$b} keys %new_orf){
      #print "$orf_id\n$new_orf{$orf_id}\n\n";#temporary
      my $blastall="| $blast_config[5] -a $blast_config[4] -p $blast_config[0] -d $blast_config[3] | $blast_parser $orf_id $blastp_config_id $user_info{'username'} $user_info{'password'} $user_info{'database'} $user_info{'host_name'}";
      open (BLAST, $blastall) or die "Can't open $blastall\n";
      print BLAST $new_orf{$orf_id};
      close BLAST;
   }

}

=for html <A NAME='customAnalyses'>

=item customAnalyses

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub customAnalyses{

   my $user_info=shift;
   my $assembly_id=shift;

   CustomAnalysis::libraryAnalysis($user_info, $assembly_id);
   CustomAnalysis::cloneAnalysis($user_info, $assembly_id);

}

=for html <A NAME='sendEmail'>

=item sendEmail

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$email</B>():  [Default = <B>shift</B>]<BR>
- <B>$message_assembly</B>():  [Default = <B>shift</B>]<BR>
- <B>$message_analysis</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$jobqueue_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$status</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_run_dir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub sendEmail{

   my $email=shift;
   my $message_assembly=shift;
   my $message_analysis=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $jobqueue_id=shift;
   my $status=shift;
   my $new_run_dir=shift;

   ###must send the Job_Queue to the user

   my $comment="Your Job#$jobqueue_id:\nAssembly $configuration{'Assembler_Name'} Build$new_run_dir $status.\n$message_assembly\n$message_analysis";
   CommonTask::sendMail($email, $comment, $jobqueue_id) if ($email ne '');
   
}

=for html <A NAME='getConfigurationInfo'>

=item getConfigurationInfo

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembler</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_build</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub getConfigurationInfo{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembler=shift;
   my $new_build=shift;

   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly Information
   my @assembly_field=("max(Assembly_Build_ID)");
   my @assembly_table=("Assembly", "Assembler","Project");
   my @assembly_condition=("Assembler_ID=FK_Assembler__ID", "Project_ID=FK_Project__ID", "Assembler_Name='$assembler'");
   my $special_instruction="GROUP BY Project_ID";#always same Assembler_ID, what about many projects?
   my $select_assembly=SQLdb::GetRecord(\@assembly_field, \@assembly_table, \@assembly_condition, $special_instruction);
#   print "$select_assembly\n";

   my $buildref=SQLdb::ExecuteUnique($dbh, $select_assembly);
   my @build_arr=@$buildref;

   ####Changes to new build Vs. New version #
   if (defined $build_arr[0]){
      print "actual:$build_arr[0]\n";
      my $integer=$1 if ($build_arr[0] =~/(\d+)\.?/);
      print "integer:$integer\n";

      if($new_build==1){
         $integer++;###increment to new build
         $build_arr[0]="$integer".".00";

      }elsif($new_build==-1){###new version
         $build_arr[0]+=0.01;
      }

      print "new build:$build_arr[0]\n";

   }else{###First build ever, even if new version selected: must create a new build if none exist
      $build_arr[0]="1.00";
   }

   ####Get config info for given assembler
   my @config_field=("Assembler_Name", "Config_Data_Dir", "Config_Read_File", "Config_Qual_File", "Config_Xml_Dir", "Config_Exec_Dir", "Config_Exec_Cmd", "Config_Assembler_Dir", "Config_Xml_File", "Config_Assembler_Exec", "Assembler_ID", "Config_Analysis_File", "Config_AssemblyXml_File", "Config_SummaryXml_File", "Config_Tig_Fasta_File", "Config_Tig_Qual_File", "Config_Output_Dir", "Config_Assembler_Node");
   my @config_table=("Config", "Assembler");
   my @config_condition=("Assembler_ID=FK_Assembler__ID", "Assembler_Name like '%$assembler%'");#can't select under project id, because above may return nothing for max build
   my $select_config=SQLdb::GetRecord(\@config_field, \@config_table, \@config_condition);
   my $data_config=SQLdb::ExecuteUnique($dbh, $select_config);

#   print "$select_config\n";

   my @data_config=@$data_config;
   if (!defined $data_config[0]){die "Problem getting config information\n";} #if problem with query (likely be an assembler name mismatch)

   ####Get Project Information
   my @project_field=("Project_ID", "Project_Name");
   my @project_table=("Project");
   my @project_condition=();
   my $select_project=SQLdb::GetRecord(\@project_field, \@project_table, \@project_condition);
   my $data_project=SQLdb::ExecuteUnique($dbh, $select_project);
   my @data_project=@$data_project;

   ####Get all info in one place
   my @configuration=("Assembler_Name", "$data_config[0]", "Config_Data_Dir", "$data_config[1]", "Config_Read_File", "$data_config[2]", "Config_Qual_File","$data_config[3]", "Config_Xml_Dir","$data_config[4]", "Config_Exec_Dir","$data_config[5]", "Config_Exec_Cmd", "$data_config[6]", "Config_Assembler_Dir", "$data_config[7]", "Config_Xml_File", "$data_config[8]", "Config_Assembler_Exec", "$data_config[9]", "Assembler_ID", "$data_config[10]", "run_dir", "$build_arr[0]", "project_name", "$data_project[1]", "project_id", "$data_project[0]", "Config_Analysis_File", "$data_config[11]", "Config_AssemblyXml_File", "$data_config[12]", "Config_SummaryXml_File", "$data_config[13]", "Config_Tig_Fasta_File", "$data_config[14]", "Config_Tig_Qual_File", "$data_config[15]", "Config_Output_Dir", "$data_config[16]", "AssemblerNode", "$data_config[17]");
   my %configuration=@configuration;

   SQLdb::Disconnect($dbh);

   return \%configuration;
}

=for html <A NAME='startAssembly'>

=item startAssembly

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembler</B>():  [Default = <B>shift</B>]<BR>
- <B>$date_condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$library_condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$date_format</B>():  [Default = <B>shift</B>]<BR>
- <B>$jobqueue_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_type</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub startAssembly{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembler=shift;
   my $date_condition=shift;
   my $library_condition=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $date_format=shift;
   my @date_format=@$date_format; 
   my $jobqueue_id=shift;
   my $assembly_type=shift;

   my $message_assembly;
   my $new_run_dir;

   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Write to Assembly table
   my $unixdate=`date`;
   chomp($unixdate);
   my $startdate=&UnixDate($unixdate,@date_format);
   my $uniquetable="Assembly";
   my @assembly_field=("Assembly_ID", "FK_Project__ID", "FK_Assembler__ID", "FK_Job_Queue__ID", "Assembly_Build_ID", "Assembly_Type", "Assembly_DateStart", "Assembly_DateEnd", "Assembly_Status");
   my @assembly_value=("", "$configuration{'project_id'}", "$configuration{'Assembler_ID'}", "$jobqueue_id", "$configuration{'run_dir'}", "$assembly_type", "$startdate", "", "initiated");
   SQLdb::InsertRecord($dbh, $uniquetable, \@assembly_field, \@assembly_value);

   ###Run wrapper for given assembler via system calls

   eval{#Catch error if system calls fail

      ####Create input files for assemblers
      if (! -d "$configuration{'Config_Data_Dir'}/$configuration{'project_name'}"){
        print "$configuration{'Config_Data_Dir'}/$configuration{'project_name'} does not exist...creating directory\n";
        `mkdir $configuration{'Config_Data_Dir'}/$configuration{'project_name'}`;
      }

      my @trace_condition=("Clone_ID=FK_Clone__ID", "Library_ID=FK_Library__ID", "$library_condition", "$date_condition");
     
      #files created from cmds below are essential to assemblers
      #Leave here.  Useful to create log file for debugging
      print "\nMaking a fasta dump of all read traces and quality...\n";

#commented on June 23rd
      CommonTask::FastaDump($dbh, \%configuration, \@trace_condition);#make fasta dump (seq + qual), returns #of reads
      print "Building XML ancillary file...\n";
#commented on June 23rd
      CommonTask::XmlDump($dbh, \%configuration, \@trace_condition);

      SQLdb::Disconnect($dbh);#disconnect now because assembly is usually long
 
      #####################################
      $new_run_dir=$configuration{'run_dir'}."_".$assembly_type;

      my $cmd="$configuration{'Config_Assembler_Dir'}/$configuration{'Config_Assembler_Exec'} \'$configuration{'Config_Exec_Dir'}\' \'$configuration{'project_name'}\' \'$new_run_dir\' \'$configuration{'Config_Data_Dir'}\' \'$configuration{'Config_Exec_Cmd'}\' \'$user_info{'password'}\' \'$user_info{'username'}\' \'$user_info{'database'}\' \'$user_info{'host_name'}\' \'$date_start\' \'$date_end\' \'$library_list\'";
      print "$cmd\n";
      system("$cmd")==0 or die "system $configuration{'Config_Assembler_Exec'} failed.\n";
      
   };
   ###Must find another way to evaluate failure.  This is because there are 2 systems call in order to execute phrap.  If the second one fails, but running the overall script was successful, nothing can be done
   if($@){$message_assembly=$@;print "\nLaunching Assembly failed:$@\n";}else{print "\nAssembly Launch Successful";}

   return $message_assembly, $new_run_dir;
}

=for html <A NAME='updateAssemblyStatus'>

=item updateAssemblyStatus

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$status</B>():  [Default = <B>shift</B>]<BR>
- <B>$date_format</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub updateAssemblyStatus{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembly_id=shift;
   my $status=shift;
   my $date_format=shift;
   my @date_format=@$date_format;

   ####re-initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Update Assembly table
   my $unixdate=`date`;
   chomp($unixdate);
   my $enddate=&UnixDate($unixdate,@date_format);
   my $uniquetable="Assembly";
   my @assembly_field=("Assembly_DateEnd", "Assembly_Status");
   my @assembly_value=("$enddate", "$status");
   my @assembly_condition=("Assembly_ID=$assembly_id");
   SQLdb::UpdateRecord($dbh, $uniquetable, \@assembly_field, \@assembly_value, \@assembly_condition);

   SQLdb::Disconnect($dbh);
}

=for html <A NAME='startAnalysis'>

=item startAnalysis

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_run_dir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub startAnalysis{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $new_run_dir=shift;

   my ($status, $message_analysis);

   eval{#Assembly Analysis
      my $cmd="$configuration{'Config_Assembler_Dir'}/$configuration{'Config_Analysis_File'} $configuration{'Config_Data_Dir'} $configuration{'project_name'} $new_run_dir $configuration{'Config_AssemblyXml_File'} $configuration{'Config_SummaryXml_File'} \'$user_info{'password'}\' \'$user_info{'username'}\' \'$user_info{'database'}\' \'$user_info{'host_name'}\' \'$date_start\' \'$date_end\' \'$library_list\'";
      print "Executing..$cmd\n$new_run_dir\n";
      system("$cmd")==0 or die "system $configuration{'Config_Analysis_File'} failed.\n";
   };
   if($@){$message_analysis=$@;print "$@\n";}else{print "\nAnalysis program lauched successfully\n";}
 

   #eval{
   #Other Analyses
   #};

   ####Only way I found to detect an error in the assembly since jobs are launched
   ####Check to to see if assembly successsful by making sure some 'key' files are present (xml files produced)
   if (! -e "$configuration{'Config_Data_Dir'}/$configuration{'project_name'}/$new_run_dir/$configuration{'Config_SummaryXml_File'}"){
      $status="failed";
      $message_analysis.=" $@";
      print "$@\n";
      
    }else{
      $status="completed";
   }

   return $status, $message_analysis;
}

=for html <A NAME='parseDataIntoDatabase'>

=item parseDataIntoDatabase

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$configuration</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$date_condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$new_run_dir</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------
sub parseDataIntoDatabase{

   my $user_info=shift;
   my $configuration=shift;
   my %configuration=%$configuration;
   my $assembly_id=shift;
   my $date_condition=shift;
   my $new_run_dir=shift;

   my $xml_dir="$configuration{'Config_Data_Dir'}/$configuration{'project_name'}/$new_run_dir";

   XMLparser::assemblyData($user_info, $xml_dir, $assembly_id, $configuration{'Config_AssemblyXml_File'}, $date_condition);
   XMLparser::summaryData($user_info, $xml_dir, $assembly_id, $configuration{'Config_SummaryXml_File'});
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut

