#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

run_glimmerGeneRBS.pl -

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Author:Rene Warren<BR>Script:runGlimmerGeneRBS.pl<BR>Purpose:runs glimmer using a training set and a Ribosome Binding Site (RBS) finder<BR>Rene Warren 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item SequenceAnalysis

=item Cache::FileCache

=item Data::Dumper

=back

=cut

#Author:Rene Warren
#Script:runGlimmerGeneRBS.pl
#Purpose:runs glimmer using a training set and a Ribosome Binding Site (RBS) finder
#Rene Warren 2003
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use SequenceAnalysis;
use Cache::FileCache;
use Data::Dumper;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
####Making the cache
my %cache_options = (namespace=>"gene",
                     cache_root=>"/home/tmp/rwarren",
                     default_expires_in=>'1500');
my $cache = new Cache::FileCache(\%cache_options);
#global variables
my $ALL_CONTIG="contig"; 
my $PHRED20=20;
my $RBS_FINDER="/home/pubseq/BioSw/glimmer/RBSfinder/rbs_finder.pl";
my %gf_config;
my $tig_fasta_file=$ARGV[0];
my $tig_qual_file=$ARGV[1];
my $data_output_dir=$ARGV[2];
$gf_config{'directory'}=$ARGV[3];
$gf_config{'lowqual'}=$ARGV[4];
$gf_config{'maxphred'}=$ARGV[5];
$gf_config{'topology'}=$ARGV[6];
$gf_config{'smallest_gene'}=$ARGV[7];
$gf_config{'cds_file'}=$ARGV[8];
$gf_config{'protein_file'}=$ARGV[9];
my $assembly_id=$ARGV[10];
$gf_config{'overlap'}=$ARGV[11];
$gf_config{'training_set'}=$ARGV[12];
my $orf_key="quality".$assembly_id;
print "Splitting Contigs...\n";
&splitContig($tig_fasta_file, $data_output_dir);
print "done. Running Glimmer...\n";
my $all_orf=&runGlimmer($data_output_dir, \%gf_config);
print "done. Checking Quality...\n";
my $quality_orf=&checkQuality($tig_qual_file, $data_output_dir, $all_orf, \%gf_config);
print "done.\n";
$cache->set($orf_key, $quality_orf, '2 hours');
exit;

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#splitContig'>splitContig; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#checkQuality'>checkQuality; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getQualityOrfs'>getQualityOrfs; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#loadQualityScore'>loadQualityScore; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#runGlimmer'>runGlimmer; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#checkOrf'>checkOrf; </A></strong></font><BR>

=over

=for html <A NAME='splitContig'>

=item splitContig

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$data_directory</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#----------------------------------
sub splitContig{

   my $file=shift;
   my $data_directory=shift;

   my $contig_file="$data_directory/$file";

   open(FILE, $contig_file) or die "fasta; Can't open $contig_file\n";

   my %contigs;
   my ($contig, $answer, $temp, $countfile);
   my $dir ="$data_directory/$ALL_CONTIG";

   print "="x80;
   print "\nBREAK UP A CONTIG FILE INTO INDIVIDUAL CONTIG FASTA FILES\n";

   print "Removing $dir...please wait\n";
   print "Breaking up $contig_file...";

   `rm -rf $dir`;  #removes directory temp and all its content
   `mkdir $dir`;
    my $id=-1;

    while(<FILE>){                         #go through $file
       chomp;                              #get rid of cr

       if(/\>.*\D+(\d+)\s?$/i){                #if find contig, isolate number
           print "$_ $1 $contig_file\n";
           $id=$1;
           close IN if ($id != -1);
           open (IN, ">$dir/contig$id");   #open a new file contigX for writing
           print IN ">Contig$id\n";        #insert the fasta header with contig number
       }
       elsif (/([ATGCXN]+[ATGCXN])/gis){   #if on sequences line
           $temp=lc($1);                   #convert all sequence in caps
           $temp =~ s/x/n/g;               #replace all X with N (glimmer can't handle Xs)
           print IN "$temp";}              #save that line in the new file contigX (eliminate new line character)
       #elsif ($_ eq '')
       #{   close IN;}                      #close file if nothing
   }
   close IN;
   print "done\n";
   close FILE;                             #close $file
 
}

=for html <A NAME='checkQuality'>

=item checkQuality

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$data_dir</B>():  [Default = <B>shift</B>]<BR>
- <B>$all_orf</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------------
sub checkQuality{

   my $file=shift;
   my $data_dir=shift;
   my $all_orf=shift;
   my $gene_finder_config=shift;

   my $qual_file="$data_dir/$file";

   my $scores=&loadQualityScore($qual_file, $all_orf); 
   my $quality_orf=&getQualityOrfs($scores, $all_orf, $gene_finder_config);

   #print Dumper($quality_orf);

   return $quality_orf;  
}

=for html <A NAME='getQualityOrfs'>

=item getQualityOrfs

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$scores</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_allorf</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------------
sub getQualityOrfs{

   my $scores=shift;
   my %scores=%$scores;
   my $tmp_allorf=shift;
   my %all_orf=%$tmp_allorf;
   my $gene_finder_config=shift;
   my %gene_finder_config=%$gene_finder_config;

   my ($length, $total, $average_phred, $average_badbase, $index_start, $index_stop, $low_base_score, $countarr, $percent)=(0,0,0,0,0,0,0,0,0);
   my ($orf_ct, $qual_ct, $poor_quality_ct)=(0,0,0);
   my %quality_orf;

   print "\nCalculating ORFs quality scores...";
   print "...\n";
   #open (IN, ">test.test");

   foreach my $contig (sort {$a<=>$b} keys %all_orf){

     my $tmp_orf=$all_orf{$contig};
     my %orf_list=%$tmp_orf;

     foreach my $cds (keys %orf_list){
        $orf_ct++;
        my $orf_start=$orf_list{$cds}{'start'};
        my $orf_stop=$orf_list{$cds}{'stop'};

        if ($orf_start>$orf_stop){                                  #start>stop?
            $index_start=$orf_stop-1;                               #array element starts at 0
            $index_stop=$orf_start-1;
        }
        elsif($orf_stop>$orf_start){
            $index_start=$orf_start-1;
            $index_stop=$orf_stop-1;
        }

        my $tmp_quality=$scores{$contig};                           #go to the hash matching contigs in $element name
        my @tig_quality=@$tmp_quality;

        $total=$length=$average_phred=$average_badbase=$low_base_score=0;

        for (my $position=$index_start;$position<=$index_stop;$position++){ #go thru all the quality array for spec. element
           $length++;
           $total+=$tig_quality[$position];                   #calculate total quality
           $low_base_score++ if ($tig_quality[$position]<$PHRED20);      #calculate bad quality <phred20
        }
        $average_phred=($total/$length);
        $average_badbase=($low_base_score/$length);

        if ($average_phred >=$gene_finder_config{'maxphred'} && $average_badbase <= $gene_finder_config{'lowqual'}){   #dual quality screen
           $qual_ct++;
           #print IN ">$contig $orf_start $orf_stop\n$cds\n\n";
           $quality_orf{$contig}{$cds}{'start'}=$orf_start; #transfer good quality sequence orfs to new hash
           $quality_orf{$contig}{$cds}{'stop'}=$orf_stop;
           $quality_orf{$contig}{$cds}{'protein'}=$orf_list{$cds}{'protein'};
           $quality_orf{$contig}{$cds}{'strand'}=$orf_list{$cds}{'strand'};

           my ($perca, $perct, $percg, $percc, $percgc)=SequenceAnalysis::calculateComposition($cds);  #calls to method in module SequenceAnalysis.pm
           my $pos_composition=SequenceAnalysis::compositionPosition($cds);
           my %pos_composition=%$pos_composition;

           $quality_orf{$contig}{$cds}{'percent_a'}=$perca;
           $quality_orf{$contig}{$cds}{'percent_t'}=$perct;
           $quality_orf{$contig}{$cds}{'percent_c'}=$percc;
           $quality_orf{$contig}{$cds}{'percent_g'}=$percg;
           $quality_orf{$contig}{$cds}{'percent_gc'}=$percgc;

           $quality_orf{$contig}{$cds}{'a1'}=$pos_composition{1}{'a'};
           $quality_orf{$contig}{$cds}{'a2'}=$pos_composition{2}{'a'};
           $quality_orf{$contig}{$cds}{'a3'}=$pos_composition{3}{'a'};

           $quality_orf{$contig}{$cds}{'t1'}=$pos_composition{1}{'t'};
           $quality_orf{$contig}{$cds}{'t2'}=$pos_composition{2}{'t'};
           $quality_orf{$contig}{$cds}{'t3'}=$pos_composition{3}{'t'};

           $quality_orf{$contig}{$cds}{'c1'}=$pos_composition{1}{'c'};
           $quality_orf{$contig}{$cds}{'c2'}=$pos_composition{2}{'c'};
           $quality_orf{$contig}{$cds}{'c3'}=$pos_composition{3}{'c'};

           $quality_orf{$contig}{$cds}{'g1'}=$pos_composition{1}{'g'};
           $quality_orf{$contig}{$cds}{'g2'}=$pos_composition{2}{'g'};
           $quality_orf{$contig}{$cds}{'g3'}=$pos_composition{3}{'g'};

           $quality_orf{$contig}{$cds}{'initiation_codon'}=substr($cds,0,3);###get the initiation codon

        }else{
          $poor_quality_ct++;
           #print ">$average_phred >=$gene_finder_config{'maxphred'} && $average_badbase <= $gene_finder_config{'lowqual'}\n";
           #print ">$contig \n$cds\n\n"
        }
        
     }
   }
   #close IN;
   print "Orf:$orf_ct, Qual:$qual_ct, Bad Quality:$poor_quality_ct\n";
   return \%quality_orf;
}

=for html <A NAME='loadQualityScore'>

=item loadQualityScore

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$qual_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp_allorf</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmp</B>():  [Default = <B>$_</B>]<BR>
</Font><BR>

=cut

#-----------------------------------------
sub loadQualityScore{

   my $qual_file=shift;
   my $tmp_allorf=shift;
   my %all_orf=%$tmp_allorf;

   my %tig_quality;
   my @quality;
 
   open (QUAL, $qual_file) or die "Qual;Can't open $qual_file\n";

   my $contig=-1;
   while(<QUAL>){
      chomp;

      if(/\>.*\D+(\d+)\s?$/i){####phrap has extra space after name
         if ($contig != -1 && defined $all_orf{$contig}){
            my @cp_quality=@quality;           #essential to prevent multiple reference to same array
            #print ">$contig\n@cp_quality\n";
            $tig_quality{$contig}=\@cp_quality;
            @quality=();
         }
         $contig=$1;

      }elsif (/^[\s\d][\s\d]+[\s\d]$/g && defined $all_orf{$contig}){ #if series of number
        my $tmp=$_;
        while ($tmp =~ /\s?(\d{1,2})\s?/g){                #isolates every single numbers
           push @quality, $1;                              #saves it in @quality array
        }
     }

   }
  if ($contig != -1 && defined $all_orf{$contig}){
      my @cp_quality=@quality;
      #print "$contig \n @cp_quality\n\n";
      $tig_quality{$contig}=\@cp_quality;
  }
  close QUAL;   

  return \%tig_quality;
}

=for html <A NAME='runGlimmer'>

=item runGlimmer

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$data_dir</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------------
sub runGlimmer{

   my $data_dir=shift;
   my $gene_finder_config=shift;
   my %gene_finder_config=%$gene_finder_config;

   my %all_orf;

   my $genefinder_dir =$gene_finder_config{'directory'};                                      #directory where prgm reside
   my $smallest_contig=$gene_finder_config{'smallest_gene'};

   my $filedir="$data_dir/$ALL_CONTIG"; #where all files output from fastaconvert.pl
   my $outdir="$data_dir";

   my ($currentfile, $cds, $line, $start, $end, $isolatedna, $protein);
   my (@cds, @temp);                 #stores the coding sequence of extraction after glimmer run
   my (%codon2aa, %comprev);

   my @allfiles=<$filedir/*>;        #get all contig files
   my $orf_ct=0;

   foreach my $tigfile (@allfiles){  #cycle through all contigs

      my $contig_number;
      print "\n\nFile is $tigfile\n";
      $currentfile=$tigfile;

      print $currentfile, "\n";

      open (OUT, $currentfile) or die "Can't open file $currentfile";                         #open contigX

      print "\nBuild interpolated context model in  tmp.1.model";                        
     `rm -f $outdir/tmp.1.model`;                       #remove previous model binary file
     `$genefinder_dir/./build-icm <$gf_config{'training_set'}> $outdir/tmp.1.model`;          #run build-icm prgm get IMM (markov model)
     `rm -f $outdir/g2.1.coord`;

      `$genefinder_dir/./glimmer2 $currentfile $outdir/tmp.1.model -l -g $smallest_contig -p $gf_config{'overlap'}| $genefinder_dir/./get-putative > $outdir/g2.1.coord`;
                                                                                              #save results in temporary g2.1.coord file

      ####takes into consideration RBS
      #----(temp directory)

      #rbs_finder.pl <Seq_file> <Whole_Coord_File> <Output_file> <RBS_window_size> [<Consensus_seq>] [-p <Partial_Coord_File>]
      `$RBS_FINDER $currentfile $outdir/g2.1.coord $outdir/rbs.coord 50`;

      ####

      if (-s "$outdir/rbs.coord" >0){                                                        #if no output of glimmer, don't save in g2.coord

         open (GLIMMER_O, "$outdir/rbs.coord");

         ####Load DNA sequence in Memory
         my $whole_seq="";
         while(<OUT>){
            chomp;
            if (/^([agctn]*[acgtn])$/i){$whole_seq.=lc($1);}elsif(/\>\D+(\d+)$/i){$contig_number=$1;}
         }

         while(<GLIMMER_O>){
             chomp;
             $line=$_;
             print "$line\n"; 
 
             if (/\d+\s+\d+\s+\d+/){                                       #allows output from RBSfinder
               my $strand;
               @temp=split(/\s+/,$line);
               if ($temp[2]>$temp[3]){
                  $start=$temp[3]-1;                                      #corrects glimmer mistake / subs -4 to see stop
                  $end=$temp[2];
                  $isolatedna=substr($whole_seq,$start,$end-$start);
                  $cds=SequenceAnalysis::reverseComplement($isolatedna);
                  $protein=SequenceAnalysis::dna2protein($cds);
                  $strand=-9;
               }
               if ($temp[3]>$temp[2]){
                   $start=$temp[2];
                  $end=$temp[3];                                           #add +3 to see stop codon
                  $isolatedna=substr($whole_seq,$start-1,$end-$start+1);
                  $cds=$isolatedna;
                  $protein=SequenceAnalysis::dna2protein($cds);
                  $strand=+9;
                }

              $all_orf{$contig_number}{$cds}{'start'}=$temp[2];           #start and stop to isolate seq only
              $all_orf{$contig_number}{$cds}{'stop'}=$temp[3];
              $all_orf{$contig_number}{$cds}{'protein'}=$protein;
              $all_orf{$contig_number}{$cds}{'strand'}=$strand;
              $orf_ct++;
            }
         }#end while

         close GLIMMER_O;
      }#end coord if
      else {
          print "\nNothing in tmp.1.coord for Contig$contig_number.fasta\n";
      }
        
      close OUT;
   }#for each

   return \%all_orf;
}

=for html <A NAME='checkOrf'>

=item checkOrf

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_orf</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#############################
sub checkOrf{

   my $tmp_orf=shift;
   my %all_orf=%$tmp_orf;

   my $ct=0;
   foreach my $tig (sort {$a<=>$b} keys %all_orf){
      print "$tig-\n";
      my $tmp_orf=$all_orf{$tig};
      my %orf_list=%$tmp_orf;
      foreach my $cds (keys %orf_list){
         $ct++;
         print "\t$ct-\n\t$cds\n\n";
      }
   }
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>



=cut

