#!/usr/local/bin/perl
##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

tiemapNsequence.pl - 

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
<BR>Script: tiemapNsequence.pl<BR>Purpose: Orients supercontigs and joins based on the fingerprint map<BR>Author: Rene Warren<BR>October 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item FindBin

=item lib "$FindBin::Bin/../lib"

=item strict

=item SQLdb

=item SAMQueryAPI

=item CommonTask

=item GD

=item Data::Dumper

=item POSIX qw(log10)

=back

=cut

#Script: tiemapNsequence.pl
#Purpose: Orients supercontigs and joins based on the fingerprint map
#Author: Rene Warren
#October 2003
use FindBin;
use lib "$FindBin::Bin/../lib";
use strict;
use SQLdb;
use SAMQueryAPI;
use CommonTask;
use GD;
use Data::Dumper;
use POSIX qw(log10);

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=item $opt_p

=item $opt_u

=item $opt_d

=item $opt_a

=item $opt_o

=item $opt_m

=item $opt_f

=item $opt_i

=back

=cut

use vars qw($opt_p $opt_u $opt_d $opt_a $opt_o $opt_m $opt_f $opt_i);
##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
require "getopts.pl";
&Getopts('p:u:d:a:o:f:m:i:');
if (!$opt_u || !$opt_p || !$opt_d || !$opt_o || !$opt_a || !$opt_f){
   die "Usage: tiemapNsequence.pl -u username -p password -d database -o host name -a assembly id -f map id -i job queue id -m email\n";
}
my $user_info;
$user_info->{'username'}=$opt_u;
$user_info->{'password'}=$opt_p;
$user_info->{'host_name'}=$opt_o;
$user_info->{'database'}=$opt_d;
my $assembly_id=$opt_a;
my $map_id=$opt_f;
my $job_queue_id=$opt_i;
my $email=$opt_m;
my $DEFAULT_GAP_SIZE=5000;
####Constants
my $M=-400;#original:-250 ###slope=-0.002, 1/x=-500/2(respect image size)
my $K=4.7; #original:4.5  ###migration coefficient
my $EXP_START=45;
my $EXP_END=95;
my $BAND_THICK=1;
my $EXP_TXT_START=1;
my $GEL_HEAD=40;
my $SIL_START=105;
my $SIL_END=155;
my $SIL_TXT_START=165;
my $SIL_SPC_START=225;
my $TXT_HEAD1=2;
my $TXT_HEAD2=16;
my $TXT_HEAD3=30;
my $SIZE_LIMIT=700;
my $ERROR=0.1;      #+/- error allowed on band size
my $SIL_ARROW_START=185;
my $STEP_INCREASE=15;
my $TEXT_TAIL=880;
my $RESOLUTION_BOUNDARY=($M*(log10($SIZE_LIMIT)-$K)+$GEL_HEAD);
my $WIDTH=360;
my $LENGTH=900;
my @date_format=("%Y-%m-%d %H:%M:%S");
my ($ultracontig, $info_orient_notburied, $info_orient_buried);
#####Query SAMdb and extract map N assembly info
#=================================================
print "Getting Info\n";
my ($contig, $supertig, $template, $position, $map_clone, $map, $read, $map_conversion, $total, $contig_sequence, $web_dir)=&getInfo($user_info, $assembly_id, $map_id);
my $image_dir="$web_dir/$assembly_id/$map_id";
if(! -d $image_dir){
   my $cmd="mkdir $image_dir";
   system($cmd);
}
#####Algorithms to tie map onto sequence assembly
#=================================================
print "Collapsing Reads\n";
my ($tigcoord, $readcoord, $finaltigcoord, $finalreadcoord, $status, $info_collapse)=&collapseCoord($contig, $read, $supertig);
my $buried=1;
my ($layout_buried, $orient_buried, $order_buried, $status, $info_compute_buried)=&computeLayout($tigcoord, $readcoord, $map, $supertig, $buried);   #this is the core algorithm
($ultracontig, $status, $info_orient_buried)=&orientJoin($layout_buried, $orient_buried, $tigcoord, $readcoord, $map, $supertig, $buried, $ultracontig);
$buried=0;                                                                                                                     #don't want buried clone now 
my ($layout_notburied, $orient_notburied, $order_notburied, $status, $info_compute_notburied)=&computeLayout($tigcoord, $readcoord, $map, $supertig, $buried);               
($ultracontig, $status, $info_orient_notburied)=&orientJoin($layout_notburied, $orient_notburied, $tigcoord, $readcoord, $map, $supertig, $buried, $ultracontig);
my ($status, $info_writedb)=&writeUltracontig($ultracontig, $finaltigcoord, $finalreadcoord, $order_buried, $map, $map_conversion, $user_info);
my $message=$info_collapse."ALL CLONES: ".$info_compute_buried.$info_orient_buried."UNBURIED CLONES: ".$info_compute_notburied.$info_orient_notburied. $info_writedb;
#####Digest Clones for this Map_ID/Assembly_ID
#===============================================
$map_clone=&identifyProperFosmidClone($contig, $template, $position, $total, $map_clone, $contig_sequence);
$map_clone=&digestFosmid($map_clone);
&processDigestData($map_clone, $image_dir, $user_info);
#####Send Email and update job queue
#===============================================
&sendEmail($email, $message, $map_id, $assembly_id, $job_queue_id, $status) if ($opt_m);          ###only send e mail if email provided
CommonTask::updateJobStatus($user_info, $job_queue_id, $status, \@date_format);
exit(0);

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#processDigestData'>processDigestData; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#insertInSilicoDigestData'>insertInSilicoDigestData; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#compareXtoY'>compareXtoY; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#printInvalidGel'>printInvalidGel; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#printValidGel'>printValidGel; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeDrawing'>initializeDrawing; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeColor'>initializeColor; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#digestFosmid'>digestFosmid; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#identifyProperFosmidClone'>identifyProperFosmidClone; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getInfo'>getInfo; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#readConfigFile'>readConfigFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeRead'>reorganizeRead; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeTig'>reorganizeTig; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#sendEmail'>sendEmail; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#writeUltracontig'>writeUltracontig; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#orientJoin'>orientJoin; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#computeLayout'>computeLayout; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getDistance'>getDistance; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#collapseCoord'>collapseCoord; </A></strong></font><BR>

=over

=for html <A NAME='processDigestData'>

=item processDigestData

=for html
DIGEST FOSMID #########################################<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

################################################## DIGEST FOSMID #########################################

#-----------------------------------
sub processDigestData{

   my ($map_clone, $image_dir, $user_info)=@_;

   foreach my $clone (keys %$map_clone){

      my $image_file=$clone.".gif";

      my $specific_image_dir="$image_dir/$map_clone->{$clone}{'map_contig_number'}";

      if(! -d $specific_image_dir){
         my $cmd="mkdir $specific_image_dir";
         system($cmd);
      }

      if ($map_clone->{$clone}{'paired_ok'}){
         my $band_color_exp=&compareXtoY($map_clone->{$clone}{'experimental'}, $map_clone->{$clone}{'insilico'});
         my $band_color_sil=&compareXtoY($map_clone->{$clone}{'insilico'}, $map_clone->{$clone}{'experimental'});
         ###more analysis here
         &insertInSilicoDigestData($map_clone->{$clone}, $user_info, $image_file);##only insert for clones that could be digested
         &printValidGel($clone, $map_clone->{$clone}, $band_color_exp, $band_color_sil, $specific_image_dir, $image_file);
      }else{
         &printInvalidGel($clone, $map_clone->{$clone}, $specific_image_dir, $image_file);
      }
    }
}

=for html <A NAME='insertInSilicoDigestData'>

=item insertInSilicoDigestData

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub insertInSilicoDigestData{

   my ($map_clone_data, $user_info, $image_file)=@_;

   my $dbh=SQLdb::Initialize($user_info->{'password'}, $user_info->{'username'},$user_info->{'database'},$user_info->{'host_name'});

   ####check IDs in InSilico_Digest
   my $insilico_digest_id=SAMQueryAPI::getInSilicoDigestID($dbh, $map_clone_data->{'contig_id'}, $map_clone_data->{'map_layout_id'}); 

   if(! $insilico_digest_id){
      SAMQueryAPI::addInSilicoDigest($dbh, $map_clone_data, $image_file);
   }

   SQLdb::Disconnect($dbh);
}

=for html <A NAME='compareXtoY'>

=item compareXtoY

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub compareXtoY{

   my ($upper_array, $lower_array)=@_;

   my $band_color;
   foreach my $band_up(@$upper_array){
      my $match=0;
      my $high_boundary=$band_up+($ERROR*$band_up);   ###Error Allowed +/- $ERROR
      my $low_boundary=$band_up-($ERROR*$band_up);

      foreach my $band_lo(@$lower_array){
         if($band_lo >$low_boundary && $band_lo <$high_boundary){
            $match=1;  ###there's a band with close M.W., even if don't match in numbers
         }
      }
      if($match){$band_color->{$band_up}="black";}else{$band_color->{$band_up}="red";}
   }

   return $band_color;
}

=for html <A NAME='printInvalidGel'>

=item printInvalidGel

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub printInvalidGel{

   my ($clone, $map_clone_data, $image_dir, $image_file)=@_;

   my $image=&initializeDrawing($WIDTH,$LENGTH); #Define Drawing Object
   my $color=&initializeColor($image);

   open (IMG, ">$image_dir/$image_file") || die "can't open $image_dir/$image_file for writing.\n";
   binmode IMG;

   ####START MAKING GRAPHIC

   $image->string(gdLargeFont,$EXP_TXT_START, $TXT_HEAD1, $clone, $color->{$map_clone_data->{'buried'}{'color'}});
   $image->string(gdSmallFont,$SIL_START, $TXT_HEAD1, "$map_clone_data->{'buried'}{'name'} / $map_clone_data->{'enzyme'} Digest", $color->{$map_clone_data->{'buried'}{'color'}});

   $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD2, "$map_clone_data->{'comment'}", $color->{'black'});
   $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD3, "DATA NOT AVAILABLE", $color->{'red'});

   #-----------------------
   #Convert the image to PNG and print it on standard output
   print IMG $image->gif;#->png#(not supported in current version)
   close IMG;

}

=for html <A NAME='printValidGel'>

=item printValidGel

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub printValidGel{

    my ($clone, $map_clone_data, $band_color_exp, $band_color_sil, $image_dir, $image_file)=@_;

    my $experimental=$map_clone_data->{'experimental'};
    my $insilico=$map_clone_data->{'insilico'};

    my $image=&initializeDrawing($WIDTH,$LENGTH); #Define Drawing Object
    my $color=&initializeColor($image);

    open (IMG, ">$image_dir/$image_file") || die "can't open $image_dir/$image_file for writing.\n";
    binmode IMG;

    ####START MAKING GRAPHIC

    $image->string(gdLargeFont,$EXP_TXT_START, $TXT_HEAD1, $clone, $color->{$map_clone_data->{'buried'}{'color'}});
    $image->string(gdSmallFont,$SIL_START, $TXT_HEAD1, "$map_clone_data->{'buried'}{'name'} / $map_clone_data->{'enzyme'} Digest", $color->{$map_clone_data->{'buried'}{'color'}});
    $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD2, "Contig $map_clone_data->{'contig_number'} at $map_clone_data->{'left_coord'}-$map_clone_data->{'right_coord'} bp  InsertSize=$map_clone_data->{'stop'} bp", $color->{'black'});
    $image->dashedLine(0,$RESOLUTION_BOUNDARY,$WIDTH,$RESOLUTION_BOUNDARY, $color->{'grey'});
    
    #-----------------------
    my ($exp_band_seen, $silico_band_seen);

    $image->string(gdSmallFont,$EXP_TXT_START, $TXT_HEAD3, "EXPERIMENTAL", $color->{'navy'});
    foreach my $band(@$experimental){###experimental bands

       my $pixel = $M*(log10($band)-$K); ###will return migration distance for every fragment
       my $migration=$GEL_HEAD+$pixel;
       $image->filledRectangle($EXP_START,$migration,$EXP_END,$migration+$BAND_THICK, $color->{$band_color_exp->{$band}});

       if(defined $exp_band_seen->{$band}){
           my $current_count=$exp_band_seen->{$band}+1;
           my $previous=$band." ($exp_band_seen->{$band})";
           my $current=$band." ($current_count)";
           $image->string(gdTinyFont,$EXP_TXT_START, $migration, $previous, $color->{'white'});##erases previous entry
           $image->string(gdTinyFont,$EXP_TXT_START, $migration, $current, $color->{'black'});
       }else{
           $image->string(gdTinyFont,$EXP_TXT_START, $migration, $band, $color->{'black'});
       }
       
       $exp_band_seen->{$band}++; ###keeps track of multiplet bands
    }

    my $ladder=$TEXT_TAIL;
    $image->string(gdSmallFont,$SIL_START, $TXT_HEAD3, "IN-SILICO", $color->{'forrest'});

    my $alternate="salmon";
    my $flag=0;

    foreach my $band(@$insilico){###in silico digest
       my $pixel = $M*(log10($band)-$K); ###will return migration distance for every fragment
       my $migration=$GEL_HEAD+$pixel;

       $image->filledRectangle($SIL_START,$migration,$SIL_END,$migration+$BAND_THICK, $color->{$band_color_sil->{$band}});

       if($migration<=$RESOLUTION_BOUNDARY){
          $ladder-=$STEP_INCREASE;

          $image->line($SIL_ARROW_START,$migration,$SIL_SPC_START, $ladder, $color->{$alternate});      ####matching line
          $image->string(gdTinyFont,$SIL_SPC_START, $ladder, "($band) $map_clone_data->{'fragment_position'}{$band}", $color->{'black'}); ###more info
       
          if($flag){
             $flag=0;
             $alternate="salmon"
          }else{
             $flag=1;
             $alternate="lightblue";
          }
       }

       if(defined $silico_band_seen->{$band}){
           my $current_count=$silico_band_seen->{$band}+1;
           my $previous=$band." ($silico_band_seen->{$band})";
           my $current=$band." ($current_count)";
           $image->string(gdTinyFont,$SIL_TXT_START, $migration, $previous, $color->{'white'});##erases previous entry
           $image->string(gdTinyFont,$SIL_TXT_START, $migration, $current, $color->{'black'});
       }else{
           $image->string(gdTinyFont,$SIL_TXT_START, $migration, $band, $color->{'black'});
       }

       $image->string(gdTinyFont,$SIL_TXT_START, $migration, $band, $color->{'black'});   ####band size
       $silico_band_seen->{$band}++; ###keeps track of multiplet bands
    }

    #-----------------------
    #Convert the image to PNG and print it on standard output
    print IMG $image->gif;#->png#(not supported in current version)
    close IMG;

}

=for html <A NAME='initializeDrawing'>

=item initializeDrawing

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-------------------------------------
sub initializeDrawing{

   my ($width, $length)=@_;

   my $image = new GD::Image($width,$length);

   return $image;
}

=for html <A NAME='initializeColor'>

=item initializeColor

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$image</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub initializeColor{

   my $image=shift;
   my $color;

   # allocate some colors
   $color->{white} = $image->colorAllocate(255,255,255);
   $color->{black} = $image->colorAllocate(0,0,0);
   $color->{swamp} = $image->colorAllocate(150,150,30);
   $color->{blue} = $image->colorAllocate(0,102,204);
   $color->{yellow}= $image->colorAllocate(255,255,0);
   $color->{cyan}=$image->colorAllocate(0,255,255);
   $color->{purple}=$image->colorAllocate(255,0,255);
   $color->{green}=$image->colorAllocate(100,250,25);
   $color->{red}=$image->colorAllocate(250,25,75);
   $color->{forrest}= $image->colorAllocate(25,175,0);
   $color->{dirtyred}=$image->colorAllocate(200,0,120);
   $color->{navy}=$image->colorAllocate(0,0,150);
   $color->{dirtyyellow}=$image->colorAllocate(200,200,75);
   $color->{grey}=$image->colorAllocate(153, 153, 153);
   $color->{lightgrey}=$image->colorAllocate(220, 220, 220);
   $color->{salmon}=$image->colorAllocate(255,153,153);
   $color->{lightblue}=$image->colorAllocate(153,204,255);
   $color->{orange}=$image->colorAllocate(255,153,51);

   return $color;
}

=for html <A NAME='digestFosmid'>

=item digestFosmid

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>($map_clone)</B>():  [Default = <B>@_</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub digestFosmid{

   my ($map_clone)=@_;

   foreach my $clone (keys %$map_clone){

      if($map_clone->{$clone}{'paired_ok'}){

         my $enzyme=$map_clone->{$clone}{'enzyme'};
         my $pattern=$map_clone->{$clone}{'pattern'};
         my @experimental=split(" ", $map_clone->{$clone}{'digest'});
         my @sites=();
         while($map_clone->{$clone}{'sequence'} =~ /$pattern/ig){    ###look for position of specific pattern
            @sites = (@sites, pos $map_clone->{$clone}{'sequence'});
         }
         my $previous=$map_clone->{$clone}{'start'};
         my @all_fragment;
         foreach my $position(@sites){
            my $size=($position+$map_clone->{$clone}{'cut_site'})-$previous;
            push @all_fragment, $size;
            my $fragment_position;
            if(! $previous){###this is the first fragment
               $fragment_position="V_".($map_clone->{$clone}{'left_coord'}+$previous)."-".($map_clone->{$clone}{'left_coord'}+$position);
            }else{          ###other fragments
               $fragment_position=($map_clone->{$clone}{'left_coord'}+$previous)."-".($map_clone->{$clone}{'left_coord'}+$position);
            }

            $map_clone->{$clone}{'fragment_position'}{$size}=$fragment_position;
            $previous=$position+$map_clone->{$clone}{'cut_site'};
         }
         my $last_size=$map_clone->{$clone}{'stop'}-$previous;

         my $fragment_position=$map_clone->{$clone}{'left_coord'}+$previous."-".$map_clone->{$clone}{'right_coord'}."_V";
         $map_clone->{$clone}{'fragment_position'}{$last_size}=$fragment_position;

         push @all_fragment, $last_size;               ###convert band string into array
         my @insilico=sort { $a <=> $b} @all_fragment; ###order array

         $map_clone->{$clone}{'experimental'}=\@experimental;
         $map_clone->{$clone}{'insilico'}=\@insilico;

         foreach my $band(@insilico){
            $map_clone->{$clone}{'fragment_size_string'}.=$band." ";
            $map_clone->{$clone}{'fragment_position_string'}=$map_clone->{$clone}{'fragment_position'}{$band}." ";
         }
      }
   }
   return $map_clone;
}

=for html <A NAME='identifyProperFosmidClone'>

=item identifyProperFosmidClone

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-----------------------------------
sub identifyProperFosmidClone{

   my ($contig, $template, $position, $total, $map_clone, $contig_sequence)=@_;

   my $ct=0;
   my ($been_there, $orientation);

   ###graphical view of assembled reads
   $orientation->{0}="<--";$orientation->{1}="-->";

   TEMPLATE:
   foreach my $template_a (sort {$template->{$a}<=>$template->{$b}} keys %$template){
      $ct++;
      my $percent=($ct/$total*100);
      print "$ct out of $total ($percent%)\n";

      $been_there->{$template_a}++;###ensures that only check 1 paired-end

      my  $reverse;

      my $direction=$template->{$template_a}{'direction'};
      my $clone_name=$template->{$template_a}{'clone'};
      my $tig_a=$template->{$template_a}{'contig_number'};

      if ($direction eq 'F'){$reverse="R";}elsif($direction eq 'R'){$reverse="F";}
      my $template_b=$clone_name.$reverse;###get the partner

      if (! defined $been_there->{$template_b} && defined $template->{$template_b}{'contig_number'}){###the order here is important for speeding purposes
         my $tig_b=$template->{$template_b}{'contig_number'};

         if ($tig_a == $tig_b){####Paired Reads located on same contig
            ###check whether the reads are facing each other -->    <---
            ###if it's the case, grab the sequence, note the clone
                my $dir_a=$position->{$template_a}{'direction'};
                my $dir_b=$position->{$template_b}{'direction'};

                if ($dir_a != $dir_b){
          #         print "$template_a:$position->{$template_a}{'start'}-$position->{$template_a}{'stop'} $orientation->{$dir_a}\n";
          #         print "$template_b:$position->{$template_b}{'start'}-$position->{$template_b}{'stop'} $orientation->{$dir_b}\n";
                   my ($left_coord, $right_coord)=(0,0);
                   ####START READ
                   if ($position->{$template_a}{'start'}<$position->{$template_b}{'start'}){
                      $left_coord=$position->{$template_a}{'start'};
                   }else{
                      $left_coord=$position->{$template_b}{'start'};
                   }

                   ####END READ
                   if ($position->{$template_a}{'stop'}>$position->{$template_b}{'stop'}){
                      $right_coord=$position->{$template_a}{'stop'};
                   }else{
                      $right_coord=$position->{$template_b}{'stop'};
                   }
                   $map_clone->{$clone_name}{'stop'}=abs($right_coord-$left_coord);
                   $map_clone->{$clone_name}{'left_coord'}=$left_coord;
                   $map_clone->{$clone_name}{'right_coord'}=$right_coord;
                   $map_clone->{$clone_name}{'sequence'}=substr($contig_sequence->{$tig_a}, $left_coord, $map_clone->{$clone_name}{'stop'});
                   $map_clone->{$clone_name}{'start'}=0;
                   $map_clone->{$clone_name}{'contig_number'}=$tig_a;
                   $map_clone->{$clone_name}{'contig_id'}=$template->{$template_a}{'contig_id'};
                   $map_clone->{$clone_name}{'paired_ok'}=1;
                   $map_clone->{$clone_name}{'comment'}="Valid and logical paired-end reads.  Insert size is $map_clone->{$clone_name}{'stop'}.";
                }else{###both reads point in the same direction
                   $map_clone->{$clone_name}{'paired_ok'}=0; ###no sequence since problematic clone, but keep track of clone
                   $map_clone->{$clone_name}{'comment'}="Illogical:$template_a $orientation->{$dir_a}/$template_b $orientation->{$dir_b}";
                   next TEMPLATE;
                }
         }else{###paired on different contigs
            $map_clone->{$clone_name}{'paired_ok'}=0; ###no sequence since problematic clone fo insilico digests, but keep track of clone
            $map_clone->{$clone_name}{'comment'}="$template_a on tig$tig_a and $template_b on different tig$tig_b";
         }
      }else{###check pairs only once
         if(! defined $template->{$template_b}{'contig_number'}){
            $map_clone->{$clone_name}{'paired_ok'}=0; ###no sequence since problematic clone, but keep track of clone
            $map_clone->{$clone_name}{'comment'}="$template_b not found in the sequence assembly";
         }
      }
   }
   return $map_clone;
}

=for html <A NAME='getInfo'>

=item getInfo

=for html
Gather all the info pertaining to the map and seq. assembly<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#------------------------------------
sub getInfo{
   #Gather all the info pertaining to the map and seq. assembly

   my ($user_info, $assembly_id, $map_id)=@_;

   my $dbh=SQLdb::Initialize($user_info->{'password'}, $user_info->{'username'},$user_info->{'database'},$user_info->{'host_name'});

   my $tmp_tig=SAMQueryAPI::getContigNSupertigStats($dbh, $assembly_id);
   my $tmp_pos=SAMQueryAPI::getReadCoordBuried($dbh, $assembly_id, $map_id);  ###longer query but this has the advantage of grabbing only the clones found in the map
   my $tmp_config=SAMQueryAPI::getFileDataConfig($dbh, $assembly_id);

   SQLdb::Disconnect($dbh);

   #print Dumper($tmp_pos);

   my ($contig, $supertig)=&reorganizeTig($tmp_tig);
   my ($template, $position, $map_clone, $map, $read, $map_conversion, $total)=&reorganizeRead($tmp_pos);
   my ($contig_sequence, $web_dir)=&readConfigFile($tmp_config);

   return $contig, $supertig, $template, $position, $map_clone, $map, $read, $map_conversion, $total, $contig_sequence, $web_dir;
}

=for html <A NAME='readConfigFile'>

=item readConfigFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_config</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub readConfigFile{

   my $tmp_config=shift;

   my @record=@$tmp_config;
   my $web_dir=$record[6];
   my $build=$record[3]."_".$record[4];  
 
   my $contig_file=$record[0]."/".$record[1]."/".$build."/".$record[2]."/".$record[5];

   open (TIG, $contig_file) || die "can't open $contig_file\n";

   my $contig_sequence;
   my $sequence;
   my $contig_number=-1;
   while(<TIG>){
      chomp;

      if(/^\>\D+(\d+)(\D+)?/){
         my $new_tig=$1;
         if($contig_number != -1){
            $contig_sequence->{$contig_number}=$sequence;
         }
         $sequence='';
         $contig_number=$new_tig;

      }elsif(/^([agctnx]*[acgtnx])$/i){
         $sequence.=$1;
      }
   }
   if($contig_number != -1){
      $contig_sequence->{$contig_number}=$sequence;
      #print "$contig_number $sequence\n";
   }

   close TIG;
   return $contig_sequence, $web_dir;
}

=for html <A NAME='reorganizeRead'>

=item reorganizeRead

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_pos</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-----------------------------------
sub reorganizeRead{

   my $tmp_pos=shift;
   my @read_ref=@$tmp_pos;

   my $total=0;
   my ($template, $position, $map_clone);
   my ($map, $read, $map_conversion);
   foreach my $record (@read_ref){
      $total++;

      $map->{$record->[1]}{$record->[0]}{'order'}=$record->[2]; #Map contig->Clone Name->Order
      $map_conversion->{$record->[1]}{'id'}=$record->[8];

      $read->{$record->[3]}{$record->[0]}{$record->[4]}{'id'}=$record->[11];
      $read->{$record->[3]}{$record->[0]}{$record->[4]}{'start'}=$record->[5];
      $read->{$record->[3]}{$record->[0]}{$record->[4]}{'stop'}=$record->[6];
      $read->{$record->[3]}{$record->[0]}{$record->[4]}{'orientation'}=$record->[7];
      $read->{$record->[3]}{$record->[0]}{$record->[4]}{'alternate_name'}=$record->[9];
      $read->{$record->[3]}{$record->[0]}{$record->[4]}{'buried'}=$record->[10];
      #contig number->clone name->trace direction

      my ($buried_type, $buried_color);
      if($record->[10]){$buried_type="buried";$buried_color="grey";}else{$buried_type="not buried";$buried_color="dirtyred"}

      my $template_name=$record->[0].$record->[4];           ###RF00123aB02.F/R/U
      $template->{$template_name}{'contig_number'}=$record->[3];
      $template->{$template_name}{'contig_id'}=$record->[17];
      $template->{$template_name}{'direction'}=$record->[4]; ###F/R/U
      $template->{$template_name}{'clone'}=$record->[0];

      $position->{$template_name}{'start'}=$record->[5];
      $position->{$template_name}{'stop'}=$record->[6];
      $position->{$template_name}{'direction'}=$record->[7]; ###1/0 (1 being true, --> relative to contig orientation)

      $map_clone->{$record->[0]}{'digest'}=$record->[12];
      $map_clone->{$record->[0]}{'enzyme'}=$record->[13];
      $map_clone->{$record->[0]}{'pattern'}=$record->[14];          ###Enzyme Recognition site
      $map_clone->{$record->[0]}{'cut_site'}=$record->[15];         ###Position of enzyme cut site
      $map_clone->{$record->[0]}{'buried'}{'name'}=$buried_type;    ###Is it a buried clone?
      $map_clone->{$record->[0]}{'buried'}{'color'}=$buried_color;
      $map_clone->{$record->[0]}{'map_layout_id'}=$record->[16];    ###for writing to database
      $map_clone->{$record->[0]}{'map_contig_number'}=$record->[1]; ###for saving all the clone image files in map contig directories

      #print "$template_name $total\n";
   }
   return $template, $position, $map_clone, $map, $read, $map_conversion, $total;
}

=for html <A NAME='reorganizeTig'>

=item reorganizeTig

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tmp_tig</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------
sub reorganizeTig{

   my $tmp_tig=shift;
   my @tig_ref=@$tmp_tig;

   my ($contig, $supertig);
   foreach my $record (@tig_ref){
      $contig->{$record->[0]}{'supertig'}=$record->[1];
      $contig->{$record->[0]}{'order'}=$record->[3];
      $contig->{$record->[0]}{'size'}=$record->[4];
      $contig->{$record->[0]}{'gap'}=$record->[5];
      $contig->{$record->[0]}{'id'}=$record->[7];

      $supertig->{$record->[1]}{'size'}=$record->[2];
      $supertig->{$record->[1]}{'id'}=$record->[6];
   }

   return $contig, $supertig;
}

=for html <A NAME='sendEmail'>

=item sendEmail

=for html
MISC ##################################################<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$email</B>():  [Default = <B>shift</B>]<BR>
- <B>$message</B>():  [Default = <B>shift</B>]<BR>
- <B>$map_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$jobqueue_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$status</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>###must send the Job_Queue to the user</Font><BR>
</Font><BR>

=cut

################################################## MISC ##################################################

#-----------------------------
sub sendEmail{

   my $email=shift;
   my $message=shift;
   my $map_id=shift;
   my $assembly_id=shift;
   my $jobqueue_id=shift;
   my $status=shift;

   ###must send the Job_Queue to the user

   my $comment="Your Job#$jobqueue_id:\nBinding the fingerprint map (Map_ID=$map_id) to the sequence assembly (Assembly_ID=$assembly_id)  $status.\n$message";
   CommonTask::sendMail($email, $comment, $jobqueue_id) if ($email ne '');

}

=for html <A NAME='writeUltracontig'>

=item writeUltracontig

=for html
TIE MAP CONTIG ########################################<BR>coordinate the insertion of records in SAM database<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

############################################################## TIE MAP CONTIG ########################################

#--------------------------------
sub writeUltracontig{
   ####coordinate the insertion of records in SAM database
   my ($ultracontig, $finaltigcoord, $finalreadcoord, $order, $map, $map_conversion, $user_info)=@_;

   my ($status, $message);
   eval{

   my %ultracontig=%$ultracontig;
 
   my $dbh=SQLdb::Initialize($user_info->{'password'}, $user_info->{'username'},$user_info->{'database'},$user_info->{'host_name'});

   ####check IDs in Ultracontig, Contig_Coord, Read_Coord
   my $ultracontig_id=SAMQueryAPI::getMaxUltracontigID($dbh);
   my $readcoord_id=SAMQueryAPI::getMaxReadCoordID($dbh);
   my $contigcoord_id=SAMQueryAPI::getMaxContigCoordID($dbh);

   foreach my $buried (sort {$a<=>$b} keys %ultracontig){

     my $map_set=$ultracontig{$buried};
     my %map_set=%$map_set;

     foreach my $map_tig (sort {$a<=>$b} keys %map_set){
       my $tmp_super=$map_set{$map_tig};
       my %supercontig=%$tmp_super;
       my $map_id=$map_conversion->{$map_tig}{'id'};
       print "$map_id\n";       

       foreach my $sc_id (sort {$a<=>$b} keys %supercontig){

         ####check existence in Ultracontig
         my $exist_ultracontig_id=SAMQueryAPI::getUltracontigID($dbh, $map_id, $sc_id, $buried);
       
         if (! defined $exist_ultracontig_id){ #first time seen ultracontig

            SAMQueryAPI::addUltracontig($dbh, $map_id, $map_tig, $sc_id, $ultracontig, $buried);
            $ultracontig_id++;  #keep track of record_id (essential for Contig_Coord and Read_Coord tables)

            my $tmp_clone=$finalreadcoord->{$sc_id};   #ensures clone appear only once per map
            my $tmp_contig=$finaltigcoord->{$sc_id};

            ########################
            #Deal with all the reads
            ########################

            my %clone=%$tmp_clone;
            foreach my $clone_name(keys %clone){

             if (defined $map->{$map_tig}{$clone_name}){        ##clone exists for that map  

               my $tmp_primer=$clone{$clone_name};
               my %read=%$tmp_primer;
               my $clone_order=$order->{$clone_name};

               foreach my $trace_direction(keys %read){

                  ####check existence of Read in ReadCoord
                  my $exist_readcoord_id=SAMQueryAPI::getReadCoordID($dbh, $read{$trace_direction}{'id'}, $sc_id, $map_id);
                  my $fk_readcoord_id;
 
                  if(! defined $exist_readcoord_id){
                     SAMQueryAPI::addMapReadCoord($dbh, $read{$trace_direction}, $clone_name, $trace_direction, $clone_order);
                     $readcoord_id++;
                     $fk_readcoord_id=$readcoord_id;
                  }else{
                     $fk_readcoord_id=$exist_readcoord_id;
                  }
                        
                  SAMQueryAPI::addMapReadLayout($dbh, $ultracontig_id, $fk_readcoord_id);

               }
              }
            }

            ##########################
            #Deal with all the contigs
            ##########################

            my %contig=%$tmp_contig;

            foreach my $tig (keys %contig){
               ####check existence of Contig in ContigCoord
               my $exist_contigcoord_id=SAMQueryAPI::getContigCoordID($dbh, $contig{$tig}{'id'}, $sc_id, $map_id);
               my $fk_contigcoord_id;

               if(! defined $exist_contigcoord_id){
                     SAMQueryAPI::addMapContigCoord($dbh, $contig{$tig}, $tig);
                     $contigcoord_id++;
                     $fk_contigcoord_id=$contigcoord_id;
                  }else{
                     $fk_contigcoord_id=$exist_contigcoord_id;
                  }

                  SAMQueryAPI::addMapContigLayout($dbh, $ultracontig_id, $fk_contigcoord_id);
            }

         }#abort writing to Ultracontig table if defined (record exists)
       }
     }
   }
   SQLdb::Disconnect($dbh);

   };
   if($@){$status='failed';$message="Some problems writing to the database $@.  ";}else{$status='completed';$message="successfully updated SAMdb. ";}

   return $status, $message; 
}

=for html <A NAME='orientJoin'>

=item orientJoin

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-------------------------------
sub orientJoin{

   my ($layout, $orient, $tigcoord, $readcoord, $map, $supertig, $buried, $ultracontig)=@_;###tigcoord not useful at this stage cause not trying to determine gap size

   ####evaluate gap size if applicable
   ####orient contigs based on gap and/or trend of clone overlap
   my ($status, $message);

   eval{

   my %map=%$map;
   
   foreach my $map_tig (sort {$a<=>$b} keys %map){#sort by map number ascending

      my $superlay_ref=$layout->{$map_tig};   
      my @superlay=@$superlay_ref;
      my $order=0;
      print "\n\n>$map_tig\n";
     
      foreach my $sc (@superlay){###concentrate on layout for every map tig

         $ultracontig->{$buried}{$map_tig}{$supertig->{$sc}{'id'}}{'order'}=++$order;

         my $gap;

         if ($order==1){$gap=0;}else{$gap=$DEFAULT_GAP_SIZE;}

         $ultracontig->{$buried}{$map_tig}{$supertig->{$sc}{'id'}}{'gap'}=$gap;
         $ultracontig->{$buried}{$map_tig}{$supertig->{$sc}{'id'}}{'size'}=$supertig->{$sc}{'size'};

         if ($orient->{$map_tig}{$sc}{'rev'}>$orient->{$map_tig}{$sc}{'fwd'}){
            $ultracontig->{$buried}{$map_tig}{$supertig->{$sc}{'id'}}{'orientation'}=0;
            print " <-$sc($supertig->{$sc}{'id'} B:$buried)- ";
         }else{
            $ultracontig->{$buried}{$map_tig}{$supertig->{$sc}{'id'}}{'orientation'}=1;
            print " -$sc($supertig->{$sc}{'id'} B:$buried)-> ";
         }

      }
   }
   print "\n\n";
   };
   if($@){$status='failed';$message="Some problems orienting supercontigs $@.  ";}else{$status='completed';$message="Successfully oriented supercontigs. ";}
 
   return $ultracontig, $status, $message;
}

=for html <A NAME='computeLayout'>

=item computeLayout

=for html
brings supercontig together based on the fingerprint map<BR>tentatively orients supercontigs based on clone overlaps<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-------------------------------
sub computeLayout{

   #brings supercontig together based on the fingerprint map
   #tentatively orients supercontigs based on clone overlaps
    
   my ($tigcoord, $readcoord, $map, $supertig, $buried)=@_;
   my ($status, $message);
   my ($layout, $orient, $order);

   eval{

    
   foreach my $map_tig(sort {$a<=>$b} keys %$map){#sort by map number ascending
#   my $map_tig=4;
      
      my $clone=$map->{$map_tig};
      my %clone=%$clone;

      my ($past_start, $past_stop, $clone_order)=(0,0,0);
      my (%beenthere, %occurance);                       #building layout for each map contig
      my @superlay;                                      #really keeps track of the layout order
      my $gap;

      foreach my $template(sort {$clone{$a}{'order'}<=>$clone{$b}{'order'}} keys %clone){#sort by template order

        $clone_order++;
        $order->{$template}=$clone_order;

        my $tig_f=$readcoord->{$template}{'F'}{'contig'};
        my $tig_r=$readcoord->{$template}{'R'}{'contig'};
        my $dir_f=$readcoord->{$template}{'F'}{'orientation'};
        my $dir_r=$readcoord->{$template}{'R'}{'orientation'};

        next if ($readcoord->{$template}{'R'}{'buried'} && ! $buried);                                  #case where you don't want buried clones

        if (defined $tigcoord->{$tig_f}{'supertig'} && defined $tigcoord->{$tig_r}{'supertig'}){        #don't deal with not assembled/wouldn't give enough info for the merge

             my $sc_f=$tigcoord->{$tig_f}{'supertig'};
             my $sc_r=$tigcoord->{$tig_r}{'supertig'};
             
             #------------------------------------------
             if ($sc_f==$sc_r){

                #   (s)-->    <--(e)   get the start of the clone (s) and the end (e)
                my ($present_start, $present_stop)=(0,0);

                if ($dir_f && !$dir_r){
                   $present_start=$readcoord->{$template}{'F'}{'start'};$present_stop=$readcoord->{$template}{'R'}{'stop'};
                }elsif(!$dir_f && $dir_r){
                   $present_start=$readcoord->{$template}{'R'}{'start'};$present_stop=$readcoord->{$template}{'F'}{'stop'};
                }

                if ($past_start>$present_start){
                   $orient->{$map_tig}{$sc_f}{'rev'}++;
                }elsif($past_start<$present_start){
                   $orient->{$map_tig}{$sc_f}{'fwd'}++;
                }

                if ($past_stop>$present_stop){
                   $orient->{$map_tig}{$sc_f}{'rev'}++;
                }elsif($past_start<$present_start){
                   $orient->{$map_tig}{$sc_f}{'fwd'}++;
                }

                if(! defined $beenthere{$sc_f}){
                   $beenthere{$sc_f}++;
                   push @superlay, $sc_f;
                }

                $past_start=$present_start;
                $past_stop=$present_stop;

             #-----------------------------------------
             }else{#template cross different seq. supertig

                my $concat;
 
                if($sc_f<$sc_r){$concat=$sc_f."-".$sc_r;}else{$concat=$sc_r."-".$sc_f;}#make sure have a uniform labelling

                  my $for_start=$readcoord->{$template}{'F'}{'start'};
                  my $for_stop=$readcoord->{$template}{'F'}{'stop'};
                  my $rev_start=$readcoord->{$template}{'R'}{'start'};
                  my $rev_stop=$readcoord->{$template}{'R'}{'stop'};
                  my $sc_f_length=$supertig->{$sc_f}{'size'};
                  my $sc_r_length=$supertig->{$sc_r}{'size'};

                  if  (! defined $beenthere{$sc_f} && defined $beenthere{$sc_r}){###by deduction, sc_r is first before sc_f

                      $occurance{$map_tig}{$concat}++;

                       if ($occurance{$map_tig}{$concat}>=2){
                          $beenthere{$sc_f}++;
                          push @superlay, $sc_f;
                          $occurance{$map_tig}{$concat} = -9999;#makes sure it is not seen again          
                       }

                    #---------------------------------------------------------------
                    }elsif (! defined $beenthere{$sc_r} && defined $beenthere{$sc_f}){###by deduction, sc_f is first before sc_r
                
                      $occurance{$map_tig}{$concat}++;

                      if ($occurance{$map_tig}{$concat}>1){
                          $beenthere{$sc_r}++;
                          push @superlay, $sc_r;
                          $occurance{$map_tig}{$concat} = -9999;#makes sure it is not seen again
                       }

                    #-----------------------------------------------------------------
                    }elsif (! defined $beenthere{$sc_r} && ! defined $beenthere{$sc_f}){###none of them were seen

                       $occurance{$map_tig}{$concat}++;

                       if ($occurance{$map_tig}{$concat}>1){#2 rev-fwd links
                          $beenthere{$sc_f}++;
                          $beenthere{$sc_r}++;
                           
                          ####need to improve this code
                          #evaluate which one comes before the other based on 1 fwd-rev link (may not be sufficient)
                          my $start_F=$readcoord->{$template}{'F'}{'start'};
                          my $start_R=$readcoord->{$template}{'R'}{'start'};
                          if ($start_F>$start_R){push @superlay, $sc_r;push @superlay, $sc_f;}
                          else{push @superlay, $sc_f;push @superlay, $sc_r;}
                          $occurance{$map_tig}{$concat} = -9999;#makes sure it is not seen again
                          #####
                       }
                   }
             }
          }#both supertig defined 
      }#foreach template
       $layout->{$map_tig}=\@superlay;
    }#foreach map tig
    };
    if($@){print "$@";$status='failed';$message="Some problems binding fingerprint map to sequence assembly $@.  ";}else{$status='completed';$message="Successfully tied fingerprint to sequence assembly. ";} 
   
    return $layout, $orient, $order, $status, $message; 
}

=for html <A NAME='getDistance'>

=item getDistance

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$length_i</B>():  [Default = <B>shift</B>]<BR>
- <B>$start_i</B>():  [Default = <B>shift</B>]<BR>
- <B>$stop_j</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>#------  ---------</Font><BR>## <Font color=blue># i->        <-j</Font><BR>
</Font><BR>

=cut

#------------------------------------
sub getDistance{

   my $length_i=shift;
   my $start_i=shift;
   my $stop_j=shift;

   #------  ---------
   # i->        <-j

   my $v=($length_i-$start_i)+$stop_j;

   return $v;
}

=for html <A NAME='collapseCoord'>

=item collapseCoord

=for html
this sub transfers rds coords associated to specific contig to specific supertig<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#-------------------------------
sub collapseCoord{

   #this sub transfers rds coords associated to specific contig to specific supertig
   my ($contig, $read, $supertig)=@_;

   my ($status, $message);
   my ($tigcoord, $readcoord, $finalreadcoord, $finaltigcoord);

   eval{

   my %contig=%$contig;

   my $previous=0;

   ####convert the contig coord to a 'fake' contig coord
   foreach my $tig (sort {$contig{$a}{'supertig'}<=>$contig{$b}{'supertig'} || $contig{$a}{'order'}<=>$contig{$b}{'order'}} keys %contig){###go thru tig layout for each sc
      
      $previous=1 if ($contig{$tig}{'order'}==1);   #always start at pos+1 for new supercontig

      $tigcoord->{$tig}{'supertig'}=$contig{$tig}{'supertig'};
      $tigcoord->{$tig}{'start'}=$contig{$tig}{'gap'}+$previous;
      $previous=$tigcoord->{$tig}{'start'}+$contig{$tig}{'size'};
      $tigcoord->{$tig}{'stop'}=$previous;

      ####This is very important for database purposes 
      my $sc_id=$supertig->{$contig{$tig}{'supertig'}}{'id'};              #id is extracted from supertig from sc number

      $finaltigcoord->{$sc_id}{$tig}{'start'}=$tigcoord->{$tig}{'start'};
      $finaltigcoord->{$sc_id}{$tig}{'stop'}=$tigcoord->{$tig}{'stop'};
      $finaltigcoord->{$sc_id}{$tig}{'id'}=$contig{$tig}{'id'};

      ####get the read coord for all these tigs
      if (defined $read->{$tig}){                                          #not every tigs contain fosmid/BAC clones
         my $tmp_template=$read->{$tig};
         my %template=%$tmp_template;

         foreach my $clone (keys %template){

            my $tmp_direction=$template{$clone};
            my %direction=%$tmp_direction;

            foreach my $trace_direction(keys %direction){#F/R
               $readcoord->{$clone}{$trace_direction}{'contig'}=$tig;
               $readcoord->{$clone}{$trace_direction}{'start'}=$tigcoord->{$tig}{'start'}+$direction{$trace_direction}{'start'};
               $readcoord->{$clone}{$trace_direction}{'stop'}=$tigcoord->{$tig}{'start'}+$direction{$trace_direction}{'stop'};
               $readcoord->{$clone}{$trace_direction}{'orientation'}=$direction{$trace_direction}{'orientation'};
               $readcoord->{$clone}{$trace_direction}{'buried'}=$direction{$trace_direction}{'buried'};
             
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'id'}=$direction{$trace_direction}{'id'};
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'contig'}=$tig;
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'start'}=$readcoord->{$clone}{$trace_direction}{'start'};
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'stop'}=$readcoord->{$clone}{$trace_direction}{'stop'};
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'orientation'}=$readcoord->{$clone}{$trace_direction}{'orientation'};
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'alternate_name'}=$direction{$trace_direction}{'alternate_name'};
               $finalreadcoord->{$sc_id}{$clone}{$trace_direction}{'buried'}=$direction{$trace_direction}{'buried'};
            }
         }
      }
   }
   };
   if($@){$status='failed';$message="Some problem transforming coordinates $@.  ";}else{$status='completed';$message="Coordinates successfully transformed. ";}

   return $tigcoord, $readcoord, $finaltigcoord, $finalreadcoord, $status, $message;
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>

=cut

