#Parses Ace file supplied and generate the XML equivalent in directories provided
#Rene Warren 
#November 2002
package AnalyzeAssembly;

##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

AnalyzeAssembly.pm - Parses Ace file supplied and generate the XML equivalent in directories provided

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
Parses Ace file supplied and generate the XML equivalent in directories provided<BR>Rene Warren <BR>November 2002<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item strict

=item Data::Dumper

=item SequenceAnalysis

=item CreateAssemblyXML

=item SQLdb

=item Math::Complex;1

=back

=cut

use strict;
use Data::Dumper;
use SequenceAnalysis;
use CreateAssemblyXML;
use SQLdb;
use Math::Complex;1;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#getGenomicAssemblyStats'>getGenomicAssemblyStats; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getSinglet'>getSinglet; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseAce'>parseAce; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#calculateUnpaddedReadCoordinate'>calculateUnpaddedReadCoordinate; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachneContigQuality'>parseArachneContigQuality; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachneContig'>parseArachneContig; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachneLinkFile'>parseArachneLinkFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachne2LinkFile'>parseArachne2LinkFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachneReadFile'>parseArachneReadFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#parseArachne2ReadFile'>parseArachne2ReadFile; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#N_fifty'>N_fifty; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#GetErrRate'>GetErrRate; </A></strong></font><BR>

=over

=for html <A NAME='getGenomicAssemblyStats'>

=item getGenomicAssemblyStats

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
sub getGenomicAssemblyStats{

   my $user_info=shift;
   my @user_info=@$user_info;
   my $condition=shift;
   my @condition=@$condition;
   my $stats=shift;
   my %stats=%$stats;

   ####Build date condition
   my $date_condition="Trace_Date_Sequenced>=\'$condition[0]\' AND Trace_Date_Sequenced<=\'$condition[1] 23:59:59\'";

   ####Build library condition
   my @chosen_library=split (/,/, $condition[2]);
   my $library_count=0;
   my $library_condition;
   foreach my $lib (sort @chosen_library){
      $library_condition.=" OR " if ($library_count>0);
      $library_condition.="Library_Name=\'$lib\'";
      $library_count++;
   }

   ####re-initialize database connection
   my $dbh=SQLdb::Initialize($user_info[0], $user_info[1], $user_info[2], $user_info[3]);

   ####Get Average Phred20 bases for list of analyzed reads (assembled reads not in at his point)
   my @field=("avg(Trace_Quality_Length)", "count(Trace_ID)");
   my @table=("Trace","Clone", "Library");
   my @condition=("Library_ID=FK_Library__ID", "Clone_ID=FK_Clone__ID", "$date_condition", "$library_condition");
   my $select_phred=SQLdb::GetRecord(\@field, \@table, \@condition);
   my $tmp_record=SQLdb::ExecuteUnique($dbh, $select_phred);
   my @phred_record=@$tmp_record;

   ####Get Genome Size as Indicated in Project Table
   @field=("Project_Genome_Size");
   @table=("Project");
   @condition=();
   my $select_genomesize=SQLdb::GetRecord(\@field, \@table, \@condition);
   $tmp_record=SQLdb::ExecuteUnique($dbh, $select_genomesize);
   my @genome_size=@$tmp_record;

   SQLdb::Disconnect($dbh);
  
   ####Analyzed Reads
   $stats{'read_analyzed'}=$phred_record[1];

   ####NL
   my $assembled_bases=($stats{'assembled_reads'}*$phred_record[0]);

   ####NL--Analyzed
   my $analyzed_bases=($phred_record[1]*$phred_record[0]);

   ####Get Genome Coverage N*L/G
   $stats{'genome_coverage'}=$assembled_bases/$genome_size[0] if ($genome_size[0]>0);

   ####Coverage from Analyzed Read N*L/G
   $stats{'genome_coverage2'}=$analyzed_bases/$genome_size[0] if ($genome_size[0]>0);

   ####Get Contig Coverage
   $stats{'contig_coverage'}=$assembled_bases/$stats{'tigsize'} if ($stats{'tigsize'});

   ####Poisson Estimated Genome Size
   ####G=(L/ln(P/N))*(-1*N);
   $stats{'poisson_genome_size'}=($phred_record[0]/ln($stats{'contigs'}/$stats{'assembled_reads'}))*(-1*$stats{'assembled_reads'}) if ($stats{'assembled_reads'});

   ####Poisson Genome Coverage
   $stats{'poisson_genome_coverage'}=$assembled_bases/$stats{'poisson_genome_size'} if ($stats{'poisson_genome_size'});

   ####Poisson Genome Coverage -analyzed Reads
   $stats{'poisson_genome_coverage2'}=$analyzed_bases/$stats{'poisson_genome_size'} if ($stats{'poisson_genome_size'});

   ####Estimated Contigs no if know size
   ####P=N*e(-1*(NL/G));
   $stats{'poisson_contig'}=$stats{'assembled_reads'}*exp(-1*($stats{'assembled_reads'}*$phred_record[0]/$genome_size[0])) if ($genome_size[0]);

   #print "genome coverage:$stats{'genome_coverage'}\ncontig coverage:$stats{'contig_coverage'}\npoisson genome size:$stats{'poisson_genome_size'}\nPoisson coverage:$stats{'poisson_genome_coverage'}\nPoisson Contigs:$stats{'poisson_contig'}\n";

   return \%stats;

}

=for html <A NAME='getSinglet'>

=item getSinglet

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
- <B>$singlet_file</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
sub getSinglet{

   my $user_info=shift;
   my @user_info=@$user_info;
   my $condition=shift;
   my @condition=@$condition;
   my $stats=shift;
   my %stats=%$stats;
   my $singlet_file=shift;

   my $bad_quality=0;
   my $singleton=0;
   my $masked_read=0;

   ####Build date condition
   my $date_condition="Trace_Date_Sequenced>=\'$condition[0]\' AND Trace_Date_Sequenced<=\'$condition[1] 23:59:59\'";

   ####Build library condition
   my @chosen_library=split (/,/, $condition[2]);
   my $library_count=0;
   my $library_condition;
   foreach my $lib (sort @chosen_library){
      $library_condition.=" OR " if ($library_count>0);
      $library_condition.="Library_Name=\'$lib\'";
      $library_count++;
   }

   ####re-initialize database connection
   my $dbh=SQLdb::Initialize($user_info[0], $user_info[1], $user_info[2], $user_info[3]);

   ####Get Read Info
   my @field=("Trace_Name", "Trace_Length", "Trace_Vector_Quality", "Trace_Quality_Length");
   my @table=("Trace", "Clone", "Library");
   my @condition=("Clone_ID=FK_Clone__ID", "Library_ID=FK_Library__ID", "$date_condition", "$library_condition");
   my $select_trace=SQLdb::GetRecord(\@field, \@table, \@condition);
   my $trace_record=SQLdb::ExecuteMany($dbh, $select_trace);

   my @trace_record=@$trace_record;

   SQLdb::Disconnect($dbh);

   ####Reorganize Read info
   my %trace;
   foreach my $ref(@trace_record){
      my @record=@$ref;
      $trace{$record[0]}=\@record;
   }

   open(SINGLETS,$singlet_file) or die "Can't open $singlet_file\n";

   my @singlet_list;

   while(<SINGLETS>){
      #if($_=~/[\s+|\>](\S+\.ab1)\s+/){#needs to come from abi sequencers!
      if($_=~/[\s+|\>](\S+)\s+/){
        push @singlet_list, $1;
     }
   }
   close SINGLETS;

   foreach my $read(@singlet_list){
     #print "***$read***\t";

     if(defined $trace{$read}){
       my $array_ref=$trace{$read};
       my @record=@$array_ref;
       my $qual_bases=$record[3];
       my $masked_bases=$record[2];
       my $read_length=$record[1];

       my $fraction=($qual_bases/$read_length)*100 if ($read_length !=0);
       #print "$qual_bases...$masked_bases...$read_length...$fraction\n";

       if ($fraction >= 20){#cut-off at least 20% quality/total
          my $fraction_masked = ($masked_bases / $qual_bases)*100 if ($qual_bases !=0);#take quality vector/quality total

          #print "****$fraction_masked\n";

          if ($fraction_masked > 5){
             $masked_read++;
          }else{
             $singleton++;
          }
       }
       else{
          $bad_quality++;
       }
     }
   }
   #print "$bad_quality, $singleton, $masked_read\n";
   $stats{'bad_quality'}=$bad_quality;
   $stats{'read_singleton'}=$singleton;
   $stats{'masked_read'}=$masked_read;

   return \%stats;
}

=for html <A NAME='parseAce'>

=item parseAce

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$ace_file</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
- <B>$tig_length</B>():  [Default = <B>shift</B>]<BR>
- <B>$data_config</B>():  [Default = <B>shift</B>]<BR>
- <B>$super_tig</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#--------------------------------------------
sub parseAce{

    my $ace_file = shift;
    my $stats=shift;
    my %stats=%$stats;
    my $tig_length=shift;
    my %tig_length=%$tig_length;
    my $data_config=shift;
    my $super_tig=shift;

    my %super_tig;
    my $read_per_tig;

    if($super_tig){#some analysis script won't pass these
       %super_tig=%$super_tig;
    }

    my $tmp;
    my $l;
    my %library;
    my @tig;
    my @count;
    my %read_assembled;
    my $tmptig;

    open(ACE,$ace_file) or die "Can't open $ace_file.\n";;
    my $rdcount=0;
    my $current_contig_number=-1;
    my @read;
   
    #print "$ace_file\n";
 
    while (<ACE>){
	#print "$_\n";
	my $sequence;
	my $qualities;

	if (/^CO\D+(\d+(\.\d+)?)\s+(\d+)\s+(\d+)/){          #change Contig for \D (non-digit)
            
            print "***$1***$4***\n";
            my $tig_number=$1;
            my $read_ct=$4;

            if ($current_contig_number != -1){          #take care of previous data
               my ($perca, $perct, $percc, $percg, $percgc);
               ($perca, $perct, $percc, $percg, $percgc)=SequenceAnalysis::calculateComposition($tmptig->{'sequence'});     #call to module to get the nucleotide composition
               $tmptig->{'perca'}=$perca;
               $tmptig->{'perct'}=$perct;
               $tmptig->{'percc'}=$percc;
               $tmptig->{'percg'}=$percg;
               $tmptig->{'percgc'}=$percgc;
               CreateAssemblyXML::writeRecord($tmptig, $data_config, $read_per_tig);
               %$tmptig=();
               @read=();                 
            }
           
            $current_contig_number=$tig_number;

            $stats{"contigs"}++;          #sup up the number of contigs
            $stats{"assembled_reads"}+=$read_ct;#sum up the total number of reads

            #read per contig
            if ($read_ct==1){$stats{'1'}++;}elsif ($read_ct==2){$stats{'2'}++;}elsif ($read_ct==3){$stats{'3'}++;}elsif ($read_ct==4){$stats{'4'}++;}elsif ($read_ct==5){$stats{'5'}++;}
            elsif ($read_ct>=6 && $read_ct <10){
               $stats{'6+'}++;
            }
            elsif ($read_ct>=10 && $read_ct <20){
               $stats{'10+'}++;
            }
            elsif ($read_ct>=20){
               $stats{'20+'}++;
            }

            $tmptig->{'number'}=$tig_number;
            $tmptig->{'reads'}=$read_ct;

	    $_=<ACE>;
         
	    until ($_=~/^\s+$/){ 
		chomp $_;
                $sequence .= $_;  
		$_=<ACE>;
	    }
	    $_=<ACE>;

            until ($_=~/^BQ$/){
                chomp $_;
                $_=<ACE>;
            }
            $_=<ACE>;

	    until ($_=~/^\s+$/){ 
                chomp $_;
		$qualities .= $_;  
		$_=<ACE>;
	    }

            ###############sequence with pads 
            $tmptig->{'padded_sequence'}=$sequence;

            ###############sequence no pads
            $sequence=~s/\*//g;             #remove pads globally from sequence
	    $tmptig->{'sequence'}=$sequence;

            ###############size
            my $tig_length=length($sequence);#calculates the contig length without pads
            $tmptig->{'size'}=$tig_length;

            $stats{"totalbp"}+=$tig_length;
            $stats{"tigsize"}+=$tig_length if ($tig_length>1000 && $read_ct>1);#size cdt for >1k tig with +2rds
            $stats{"biggest_tig"}=$tig_length if ($tig_length > $stats{"biggest_tig"});  #get the biggest contig
            $stats{"smallest_tig"}=$tig_length if ($tig_length < $stats{"smallest_tig"});#get the smallest contig
            #needs to be defined as $stats{"smallest_tig"}=1000000 in program that called this module

            $tig_length{$current_contig_number}=$tig_length;#essential to build an hash of contig lengths to calculate N50 values

            ################SUPERCONTIG Ensure we can use the map...create 1 supercontig for every contig if none were designated..
            if (defined $super_tig{$tig_number}{'number'}){
               $tmptig->{'supertig_number'}=$super_tig{$tig_number}{'number'};
            }else{
               $tmptig->{'supertig_number'}=$tmptig->{'number'};
            }

            if (defined $super_tig{$tig_number}{'rank'}){
               $tmptig->{'contig_order'}=$super_tig{$tig_number}{'rank'};
            }else{
               $tmptig->{'contig_order'}=1;
            }

            if (defined $super_tig{$tig_number}{'gap'}){
               $tmptig->{'contig_gap'}=$super_tig{$tig_number}{'gap'};
            }else{
               $tmptig->{'contig_gap'}=0;
            }

            if (defined $super_tig{$tig_number}{'number'}){
               $tmptig->{'supertig_number'}=$super_tig{$tig_number}{'number'};
            }else{
               $tmptig->{'supertig_size'}=$tmptig->{'size'};
            }
            ################base + consensus
            my ($all_base, $tmp_consensus);
            ($tmp_consensus, $all_base)=SequenceAnalysis::countBase($sequence);
            my @all_base=@$all_base;

            $stats{'all_a'}+=$all_base[0]; 
            $stats{'all_t'}+=$all_base[1];
            $stats{'all_g'}+=$all_base[2];
            $stats{'all_c'}+=$all_base[3];

            $stats{'consensus'}+=$tmp_consensus;#sum up the consensus sequence

            $tmptig->{'consensus'}=$tmp_consensus;

            #################qualities

            $tmptig->{'qual'}=$qualities;
	    my @base_quality=split(" ", $qualities);
            #print "@base_quality\n\n\n";

	    my ($err,$phred, $ph20, $ph30, $ph40);
	    ($err, $phred, $ph20, $ph30, $ph40) = GetErrRate(\@base_quality);
	    
            $stats{'error_rate'}+=$err;
	    $stats{'average_phred'}+=$phred;
            $stats{'phred20'}+=$ph20;
            $stats{'phred30'}+=$ph30;
            $stats{'phred40'}+=$ph40;  

            $tmptig->{'error_rate'}=sprintf "%.2f", $err;
            $tmptig->{'average_phred'}=$phred;
            $tmptig->{'phred20'}=$ph20;
            $tmptig->{'phred30'}=$ph30;
            $tmptig->{'phred40'}=$ph40;             

	}
        elsif (/^AF\s+(\S+)\s+([CU])\s+(\-?\d+)/gis){
           my $read=$1;
           my $direction=1;
           if ($2 eq 'C'){$direction=0;}
           my $padded_start=$3;
           $padded_start=1 if ($padded_start<=0);                                      #fix problem if bad quality read all together
           $read_per_tig->{$current_contig_number}{$read}{'padded_start'}=$padded_start;
           $read_per_tig->{$current_contig_number}{$read}{'direction'}=$direction
        }
        elsif (/^RD\s+(\S+)\s+(\d+)/gis){   
           my $read=$1;
           my $padded_read;
 
           $_=<ACE>;

            until ($_=~/^\s+$/){
                chomp $_;
                $padded_read .= $_;
                $_=<ACE>;
            }

            $read_per_tig=&calculateUnpaddedReadCoordinate($read_per_tig, $current_contig_number, $read, $tmptig, $padded_read);

        }
    }
    if ($current_contig_number != -1){          #take care of previous data
               my ($perca, $perct, $percc, $percg, $percgc);
               ($perca, $perct, $percc, $percg, $percgc)=SequenceAnalysis::calculateComposition($tmptig->{'sequence'});     #call to module to get the nucleotide composition
               $tmptig->{'perca'}=$perca;
               $tmptig->{'perct'}=$perct;
               $tmptig->{'percc'}=$percc;
               $tmptig->{'percg'}=$percg;
               $tmptig->{'percgc'}=$percgc;
               print "Creating XML...\n";
               CreateAssemblyXML::writeRecord($tmptig, $data_config, $read_per_tig);
               %$tmptig=();
               @read=();
    }
   close ACE;

   #$stats{'mean_tig'}=int($stats{'totalbp'}/$stats{'contigs'}) if ($stats{'contigs'} != 0);
   ####call to N50 calculation
   #my $N50=AnalyzeAssembly::N_fifty(\%tig_length);
   #$stats{"N50_length"}=$N50;

   return \%stats, \%tig_length;
}

=for html <A NAME='calculateUnpaddedReadCoordinate'>

=item calculateUnpaddedReadCoordinate

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$read_per_tig</B>():  [Default = <B>shift</B>]<BR>
- <B>$current_contig_number</B>():  [Default = <B>shift</B>]<BR>
- <B>$read</B>():  [Default = <B>shift</B>]<BR>
- <B>$tmptig</B>():  [Default = <B>shift</B>]<BR>
- <B>$padded_read</B>():  [Default = <B>shift;</B>]<BR>
## <Font color=blue>###count the amount of pads in read sequence</Font><BR>
</Font><BR>

=cut

#-------------------------------------------------------------------------------
sub calculateUnpaddedReadCoordinate{

  my $read_per_tig=shift;
  my $current_contig_number=shift;
  my $read=shift;
  my $tmptig=shift;
  my $padded_read=shift;;

  ###count the amount of pads in read sequence
  my $count_read_pads = ($padded_read=~ tr/\*//);

  ####count the amount of pad on contig up to the read start
  my $index_start=$read_per_tig->{$current_contig_number}{$read}{'padded_start'};
  my $padded_sequence=$tmptig->{'padded_sequence'};
  my $count_pads_before_start = () = (substr($padded_sequence,0, $index_start) =~/\*/g);
  
  my $read_length=length($padded_read)-$count_read_pads;
  my $read_start=$read_per_tig->{$current_contig_number}{$read}{'padded_start'}-$count_pads_before_start;
  $read_start=1 if ($read_per_tig->{$current_contig_number}{$read}{'padded_start'}<0);
  $read_start=$tmptig->{'size'} if ($read_start>$tmptig->{'size'});

  my $read_stop=($read_start+$read_length)-1;
  $read_stop=1 if ($read_stop<0);
  $read_stop=$tmptig->{'size'} if ($read_stop>$tmptig->{'size'});                   #doesn't allow read stop position beyond contig length

  $read_per_tig->{$current_contig_number}{$read}{'start'}=$read_start;
  $read_per_tig->{$current_contig_number}{$read}{'stop'}=$read_stop;

  return $read_per_tig;
}

=for html <A NAME='parseArachneContigQuality'>

=item parseArachneContigQuality

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------------------------------------------
sub parseArachneContigQuality{

   my $file=shift;
   my $stats=shift;
   my %stats=%$stats;

   my %qual;
   my $tmptig;

   open (OUT, "$file") or die "Can't open $file";

   SCAN:
   while(<OUT>){
      chomp;
      my $contig_number;
      my $qualities;     
 
      if(/\D+(\d+)/){
         $contig_number=$1;
      }
      $_=<OUT>;      

      until ($_=~/^\s+$/){#until blank line
         chomp $_;
         $qualities .= $_ . " ";
         $_=<OUT>;
      }
 
      my @base_quality=split(" ",$qualities);
      my ($err,$phred, $ph20, $ph30, $ph40);
      ($err, $phred, $ph20, $ph30, $ph40) = GetErrRate(\@base_quality);

      $stats{'error_rate'}+=$err;
      $stats{'average_phred'}+=$phred;
      $stats{'phred20'}+=$ph20;
      $stats{'phred30'}+=$ph30;
      $stats{'phred40'}+=$ph40;

      $qual{$contig_number}{'qual'}=$qualities;##Added May3rd, 2004
      $qual{$contig_number}{'error_rate'}=sprintf "%.2f", $err;
      $qual{$contig_number}{'average_phred'}=$phred;
      $qual{$contig_number}{'phred20'}=$ph20;
      $qual{$contig_number}{'phred30'}=$ph30;
      $qual{$contig_number}{'phred40'}=$ph40;

   }
   close OUT;

   return \%stats, \%qual;
}#end sub

=for html <A NAME='parseArachneContig'>

=item parseArachneContig

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$read_per_tig</B>():  [Default = <B>shift</B>]<BR>
- <B>$qual</B>():  [Default = <B>shift</B>]<BR>
- <B>$super_tig</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
- <B>$data_config</B>():  [Default = <B>shift</B>]<BR>
<FONT color=red>## parameters are also passed on to <A Href=#N_fifty>N_fifty</A> ##</Font><BR>
</Font><BR>

=cut

#-------------------------------------------------------------------------------
sub parseArachneContig{#somehow equivalent to parse Ace

   my $file=shift;
   my $read_per_tig=shift;
   my $qual=shift;
   my %qual=%$qual;
   my $super_tig=shift;
   my %super_tig=%$super_tig;
   my $stats=shift;
   my %stats=%$stats;
   my $data_config=shift;  
 
   open (OUT, "$file") or die "Can't open $file";
   
   my %tig_length;

   my $tig_seq;
   my $contig_number=0;
   my $old_contig_number=-1;
   my $sequence='';

   ###############Scan contig file
   while(<OUT>){
 
      chomp;

      if(/\D+(\d+)/){
         $contig_number=$1;
	 $stats{"contigs"}++;

         if ($old_contig_number != -1){
            $tig_seq->{$old_contig_number}=$sequence;
            $sequence='';
         }
         $old_contig_number=$contig_number;

      }elsif(/^([AGCTNX]*[ACGTNX])$/){
         $sequence .= $1;
      }

    }#while
    close OUT;
    if ($old_contig_number != -1){###Last contig seen
         $tig_seq->{$old_contig_number}=$sequence;
         $sequence='';
    }

    ################End of a contig record scan
    my $read_ct;
    my $tmptig;
    ################Reads 

    foreach my $contig_number (keys %$tig_seq){

      my $tmp_read=$read_per_tig->{$contig_number};
      my %read_in_contig=%$tmp_read;      

      foreach my $read (keys %read_in_contig){
         $read_ct++;
      }

      if ($read_ct==1){$stats{'1'}++;}elsif ($read_ct==2){$stats{'2'}++;}
      elsif ($read_ct==3){$stats{'3'}++;}elsif ($read_ct==4){$stats{'4'}++;}
      elsif ($read_ct==5){$stats{'5'}++;}
      elsif ($read_ct>=6 && $read_ct <10){$stats{'6+'}++;}
      elsif ($read_ct>=10 && $read_ct <20){$stats{'10+'}++;}
      elsif ($read_ct>=20){$stats{'20+'}++;}

      $tmptig->{'number'}=$contig_number;
      $tmptig->{'reads'}=$read_ct;

      $tmptig->{'supertig_number'}=$super_tig{$contig_number}{'number'} if (defined $super_tig{$contig_number}{'number'});
      $tmptig->{'supertig_size'}=$super_tig{$contig_number}{'size'} if (defined $super_tig{$contig_number}{'size'});
      $tmptig->{'contig_order'}=$super_tig{$contig_number}{'rank'} if (defined $super_tig{$contig_number}{'rank'});
      $tmptig->{'contig_gap'}=$super_tig{$contig_number}{'gap'} if (defined $super_tig{$contig_number}{'gap'});

      ###############qualities
      $tmptig->{'error_rate'}=$qual{$contig_number}{'error_rate'};
      $tmptig->{'average_phred'}=$qual{$contig_number}{'average_phred'};
      $tmptig->{'phred20'}=$qual{$contig_number}{'phred20'};
      $tmptig->{'phred30'}=$qual{$contig_number}{'phred30'};
      $tmptig->{'phred40'}=$qual{$contig_number}{'phred40'};
      $tmptig->{'qual'}=$qual{$contig_number}{'qual'};

      ###############size
      $tmptig->{'sequence'}=$tig_seq->{$contig_number};
      my $tig_length=length($tig_seq->{$contig_number});#calculates the contig length without pads
      $tmptig->{'size'}=$tig_length;
      $stats{"totalbp"}+=$tig_length;
      $stats{"tigsize"}+=$tig_length if ($tig_length>1000 && $read_ct>1);#size cdt for >1k tig with +2rds
      $stats{"biggest_tig"}=$tig_length if ($tig_length > $stats{"biggest_tig"});  #get the biggest contig
      $stats{"smallest_tig"}=$tig_length if ($tig_length < $stats{"smallest_tig"});#get the smallest contig
      #needs to be defined as $stats{"smallest_tig"}=1000000 in program that called this module
      $tig_length{$contig_number}=$tig_length;#essential to build an hash of contig lengths to calculate N50 values

      ################base + consensus
      my ($all_base, $tmp_consensus);
      ($tmp_consensus, $all_base)=SequenceAnalysis::countBase($tig_seq->{$contig_number});
      my @all_base=@$all_base;
      $stats{'all_a'}+=$all_base[0];
      $stats{'all_t'}+=$all_base[1];
      $stats{'all_g'}+=$all_base[2];
      $stats{'all_c'}+=$all_base[3];

      $stats{'consensus'}+=$tmp_consensus;#sum up the consensus sequence

      $tmptig->{'consensus'}=$tmp_consensus;

      my ($perca, $perct, $percc, $percg, $percgc);
      ($perca, $perct, $percc, $percg, $percgc)=SequenceAnalysis::calculateComposition($tig_seq->{$contig_number});     #call to module to get the nucleotide composition
      $tmptig->{'perca'}=$perca;
      $tmptig->{'perct'}=$perct;
      $tmptig->{'percc'}=$percc;
      $tmptig->{'percg'}=$percg;
      $tmptig->{'percgc'}=$percgc;
      CreateAssemblyXML::writeRecord($tmptig, $data_config, $read_per_tig);
      %$tmptig=();

   }

   ####Mean Tig Size
   $stats{'mean_tig'}=int($stats{'totalbp'}/$stats{'contigs'}) if ($stats{'contigs'} != 0);

   ####call to N50 calculation
   my $N50=AnalyzeAssembly::N_fifty(\%tig_length);
   $stats{"N50_length"}=$N50;

   return \%stats;

}#end sub

=for html <A NAME='parseArachneLinkFile'>

=item parseArachneLinkFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------------------------------------------
sub parseArachneLinkFile{

   my $file=shift;
   my $stats=shift;
   my %stats=%$stats;

   my $super;
   my ($size, $rank_ct)=(0,0);
   my %super_tig;
   my ($superct, $supersz);

   open (OUT, "$file") or die "Can't open $file";
   
   while(<OUT>){
      chomp;
      if (/supercontig\s+(\d+)\s+\D+(\d+).*\:$/){#extracts supercontig number
         $rank_ct=0;
         $super=$1;
         $size=$2;
         $superct++;
         $supersz+=$size;
      }
      elsif(/(\D+\d+\D+)?(\d+)\D+\d+/){#extracts contig number
         $rank_ct++;
         my $complex=$1;
         my $tig=$2;

         $super_tig{$tig}{'number'}=$super;
         $super_tig{$tig}{'size'}=$size;
         $super_tig{$tig}{'rank'}=$rank_ct;
         if ($complex =~ /\((-?\d+)\D+/){
             $super_tig{$tig}{'gap'}=$1;
         }else{
             $super_tig{$tig}{'gap'}=0;
         }

      }
   }
   close OUT;

   $stats{'supertig'}=$superct;
   $stats{'supertig_size'}=$supersz;
   ####Mean Supertig Size
   $stats{'mean_supertig'}=int($supersz/$superct) if ($superct != 0);

   return \%super_tig, \%stats;

}#end sub

=for html <A NAME='parseArachne2LinkFile'>

=item parseArachne2LinkFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------------------------------------------------
sub parseArachne2LinkFile{###this accounts for Arachne v2.0.1 change in assembly.link output

   my $file=shift;
   my $stats=shift;

   my ($size, $rank_ct, $superct, $supersz)=(0,0,0,0);
   my ($super_tig, $been_there);

   open (OUT, "$file") or die "Can't open $file";
  
   while(<OUT>){
      chomp;
      ##   sc id   bp    num tig  order   tig id  tig bp  prec gap after
      if(/^(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\S+)\s+\S+/){
         my ($sc_id, $sc_size, $tig_amount, $tig_rank, $tig_id, $tig_size, $tig_prev_gap)=($1, $2, $3, $4, $5, $6, $7);

         if (! defined $been_there->{$sc_id}){###the supercontig info appears for each constituent contig
            $superct++;
            $supersz+=$sc_size;      
            $been_there->{$sc_id}=$sc_size;
         }      
         $super_tig->{$tig_id}{'number'}=$sc_id;
         $super_tig->{$tig_id}{'size'}=$sc_size;
         $super_tig->{$tig_id}{'rank'}=$tig_rank;
         $super_tig->{$tig_id}{'gap'}=$tig_prev_gap;
       }
      
   }
   close OUT;

   $stats->{'supertig'}=$superct;
   $stats->{'supertig_size'}=$supersz;
   ####Mean Supertig Size
   $stats->{'mean_supertig'}=int($supersz/$superct) if ($superct != 0);

   return $super_tig, $stats;

}#end sub

=for html <A NAME='parseArachneReadFile'>

=item parseArachneReadFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------------------------------------------------
sub parseArachneReadFile{

   my $file=shift;
   my $stats=shift;
   my %stats=%$stats;

   my $read_per_tig;
   my $assembled_read=0;

   open (OUT, $file) or die "CAN'T OPEN $file";
   SCAN:
   while (<OUT>){
      chomp;
      my $read;  

      if (/^(\S+)$/){#depends on arachne's output

         #print "$assembled_read--$1\n";
         $read=$1; 
         $assembled_read++;
         $_=<OUT>;
         chomp;
         if (/\D+\d+\D+(\d+)(\D+)(\d+)\D+(\d+)\D+(\d+)/){
            my $tig=$3;
            my $start=$5;
            my $length=$1;
            my $stop=$start+$length;
            my $direction=1;
            if ($2=~/rc/){
               $direction=0;
            }                       

            $read_per_tig->{$tig}{$read}{'start'}=$start;
            $read_per_tig->{$tig}{$read}{'stop'}=$stop;
            $read_per_tig->{$tig}{$read}{'direction'}=$direction            
            #print "$tig...$read...$start\n";
            #print "$tig...$read...$stop\n";
         }
         else{
            next SCAN;
         }
      }
  }
  close OUT;

  $stats{'assembled_reads'}=$assembled_read;
  return $read_per_tig, \%stats;
}

=for html <A NAME='parseArachne2ReadFile'>

=item parseArachne2ReadFile

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$file</B>():  [Default = <B>shift</B>]<BR>
- <B>$stats</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------------------------------------------------
sub parseArachne2ReadFile{###this accounts for Arachne v2.0.1 change in assembly.link output

   my $file=shift;
   my $stats=shift;

   my $read_per_tig;
   my $assembled_read=0;

   open (OUT, $file) or die "CAN'T OPEN $file";
   SCAN:
   while (<OUT>){
      chomp;
      my $read;  
      ###    read  status untrim  coord   length  tig id  tig l   start   stop    direction
      if (/^(\S+)\s+[MST]?\s+?(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+([\+\-])/){#depends on arachne's output

         my ($read, $length, $tig, $start, $stop, $read_direction)=($1, $4, $5, $7, $8, $9);
         $assembled_read++;
         
         my $orientation;

         if ($read_direction eq '+'){
            $orientation=1;
         }else{
            $orientation=0;
         }

         $read_per_tig->{$tig}{$read}{'start'}=$start;
         $read_per_tig->{$tig}{$read}{'stop'}=$stop;
         $read_per_tig->{$tig}{$read}{'direction'}=$orientation
         #print "$tig...$read...$start\n";
         #print "$tig...$read...$stop\n";
      }
  }
  close OUT;

  $stats->{'assembled_reads'}=$assembled_read;
  return $read_per_tig, $stats;
}

=for html <A NAME='N_fifty'>

=item N_fifty

=for html
N50 length = the largest length such that 50% of all base-pairs are contained in contigs of this length or larger<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$bins</B>(): reference to an array of contig sizes  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------------------------------------------
sub N_fifty{
    # N50 length = the largest length such that 50% of all base-pairs are contained in contigs of this length or larger

    my $bins=shift; # reference to an array of contig sizes
    my %bins=%$bins;

    my ($N50,$total_size, $contig_number, $tig_size); # total_size = total bases in assembly

    # get total size
    foreach my $tig (sort {$bins{$a}<=>$bins{$b}} keys %bins){
        $total_size+=$bins{$tig};
    }

    my $half_size = $total_size/2;
    my $running_count=0;

    # look for largest size contig containing 50% of bases

    foreach my $tig (sort {$bins{$b}<=>$bins{$a}} keys  %bins){
        $tig_size=$bins{$tig};

        $running_count+=$tig_size;
        if ($running_count >= $half_size){
            $N50=$tig_size;
            last;
        }
    }

    return $N50;
}#end sub

=for html <A NAME='GetErrRate'>

=item GetErrRate

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$BQ</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------------------------------------------
sub GetErrRate{

    my $BQ=shift;

    my $err;
    my $err10kb;
    my $pesum;
    my $total_bq;
    my $count=0;
    my $ph20=0;
    my $ph30=0;
    my $ph40=0;

    foreach my $bq (@$BQ){
	$total_bq+=$bq;
	$count++;
        my $pe=10**($bq/-10);
        $ph20++ if ($bq >=20);
        $ph30++ if ($bq >=30);
        $ph40++ if ($bq >=40);
        $pesum+=$pe;
    }
    my $avgphred = $total_bq / $count if ($count>0);
    $avgphred = sprintf "%0.0f", $avgphred;
   
    my $bps = $#$BQ+1;
    if ($bps == 0){
	return 0;
    }
    $err = $pesum/($bps);
    $err10kb = $err*10000;
    return $err10kb,$avgphred, $ph20, $ph30, $ph40;
}#end sub   

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut


return 1;
