#Generate fasta dump of sequence in supplied directory
#Generate qual dump of base qualities in supplied directory
#Above, same sub routine
#Generate an XML fasta dump in specified directory
#Rene Warren
#October 30th, 2002
#Performs common tasks between different assemblers, hence the original name of this module
package CommonTask;

##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

CommonTask.pm - Generate fasta dump of sequence in supplied directory

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
Generate fasta dump of sequence in supplied directory<BR>Generate qual dump of base qualities in supplied directory<BR>Above, same sub routine<BR>Generate an XML fasta dump in specified directory<BR>Rene Warren<BR>October 30th, 2002<BR>Performs common tasks between different assemblers, hence the original name of this module<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item strict

=item SQLdb

collection of query statements

=item Data::Dumper

=item Date::Manip

=back

=cut

use strict;
use SQLdb; #collection of query statements
use Data::Dumper;
use Date::Manip;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
require Mail::Send;
#-------------------------
1;

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#updateJobStatus'>updateJobStatus; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#sendMail'>sendMail; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reArrangeFasta'>reArrangeFasta; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#dumpOrf'>dumpOrf; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#dumpEveryOrf'>dumpEveryOrf; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#XmlDump'>XmlDump; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#FastaDump'>FastaDump; </A></strong></font><BR>

=over

=for html <A NAME='updateJobStatus'>

=item updateJobStatus

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$job_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$status</B>():  [Default = <B>shift</B>]<BR>
- <B>$date_format</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
sub updateJobStatus{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $job_id=shift;
   my $status=shift;
   my $date_format=shift;
   my @date_format=@$date_format;

   my $unixdate=`date`;
   chomp($unixdate);
   my $date=&UnixDate($unixdate,@date_format);

   ####re-initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ###Update Status to current status 
   my $uniquetable="Job_Queue";
   my @field=("Job_Queue_Status", "Job_Queue_DateEnd");
   my @value=("$status", "$date");
   my @condition=("Job_Queue_ID=$job_id");
   SQLdb::UpdateRecord($dbh, $uniquetable, \@field, \@value, \@condition);

   SQLdb::Disconnect($dbh);

}

=for html <A NAME='sendMail'>

=item sendMail

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$email</B>():  [Default = <B>shift</B>]<BR>
- <B>$comment</B>():  [Default = <B>shift</B>]<BR>
- <B>$jobqueue_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
sub sendMail{

   my $email=shift;
   my $comment=shift;
   my $jobqueue_id=shift;

   my $msg = new Mail::Send;
   $msg = new Mail::Send;
   my $subject="Your Job on SAM (job#$jobqueue_id)";   

   $msg->to($email);
   $msg->subject($subject);
   my $fh = $msg->open('sendmail'); # explicit

   print $fh "$comment\nHave a nice day.";

   $fh->close;

}

=for html <A NAME='reArrangeFasta'>

=item reArrangeFasta

=for html
Remove bad character globaly<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$input</B>():  [Default = <B>shift</B>]<BR>
- <B>$output</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
#Remove bad character globaly
sub reArrangeFasta{

   my $input=shift;
   my $output=shift;

   open(INPUT,"$input") or die "Can't open $input\n";
   open(OUTPUT, ">$output");

   while(<INPUT>){
      chomp;
      if(/^([atgcnx].*)$/i){
          my $bases = $1;
          $bases=uc($bases);
          $bases=~tr/X/N/;   #pressdb won't run if sequence contains x or other characters
          print OUTPUT "$bases\n";
      }else{
         print OUTPUT "$_\n";
      }
   }
   close INPUT;
   close OUTPUT;
}

=for html <A NAME='dumpOrf'>

=item dumpOrf

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$aa_output</B>():  [Default = <B>shift</B>]<BR>
- <B>$cds_output</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$gene_finder_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------
sub dumpOrf{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $aa_output=shift;
   my $cds_output=shift;
   my $assembly_id=shift;
   my $gene_finder_id=shift;

   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get Assembly Information
   my @field=("Orf_ID", "Blast_Accession", "Blast_Hit_Identity", "Blast_Protein_Name", "Blast_Organism", "Orf_Protein_Sequence", "Orf_CDS_Sequence", "Assembler_Name", "Assembly_Build_ID", "Contig_Number", "Orf_Origin_Strand", "Orf_Origin_Start", "Orf_Origin_Stop", "Assembly_Type");
   my @table=("Blast", "Contig", "Orf", "Orf_Origin", "Assembly", "Orf_Context", "Assembler");
   my @condition=("Orf_Context_ID=FK_Orf_Context__ID", "Orf_ID=Blast.FK_Orf__ID", "Orf_ID=Orf_Context.FK_Orf__ID", "Contig_ID=FK_Contig__ID", "Assembly_ID=FK_Assembly__ID", "Assembly_ID=$assembly_id", "FK_Gene_Finder__ID=$gene_finder_id", "Assembler_ID=FK_Assembler__ID");
   my $special="Order by Orf_ID";
   my $select_orf=SQLdb::GetRecord(\@field, \@table, \@condition, $special);
   my $tmp_orf=SQLdb::ExecuteMany($dbh, $select_orf);

   print "$select_orf\n";
   SQLdb::Disconnect($dbh);

   my @orf_ref=@$tmp_orf;

   open (OUTPUT, ">$aa_output") or die "Can't open $aa_output for writing.\n";
   open (CDS, ">$cds_output") or die "Can't open $cds_output for writing.\n";

   foreach my $reference (@orf_ref){
      my @record=@$reference;
      print OUTPUT ">$record[0]***$record[1]***identity:$record[2]%***$record[3]***$record[4]***$record[7] Build$record[8]-$record[13]***Tig:$record[9] Strand:$record[10] ($record[11]-$record[12])\n$record[5]\n";###protein
      print CDS ">$record[0]***$record[1]***identity:$record[2]%***$record[3]***$record[4]***$record[7] Build$record[8]-$record[13]***Tig:$record[9] Strand:$record[10] ($record[11]-$record[12])\n$record[6]\n";   ###nuc acid
   } 
   close OUTPUT;
   close CDS;
}

=for html <A NAME='dumpEveryOrf'>

=item dumpEveryOrf

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$everyorf_output</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------
sub dumpEveryOrf{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $everyorf_output=shift;

   ####Initialize database connection
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Isolate Unique Orfs, regardless of their origin
   my @field=("Orf_ID", "Blast_Accession", "Blast_Hit_Identity", "Blast_Protein_Name", "Blast_Organism", "Orf_CDS_Sequence");
   my @table=("Blast", "Orf");
   my @condition=("Orf_ID=Blast.FK_Orf__ID");
   my $special="Order by Orf_ID";
   my $select_orf=SQLdb::GetRecord(\@field, \@table, \@condition, $special);
   my $tmp_orf=SQLdb::ExecuteMany($dbh, $select_orf);
   SQLdb::Disconnect($dbh);

   my @orf_ref=@$tmp_orf;

   open (ALL, ">$everyorf_output") or die "Can't open $everyorf_output for writing.\n";
   foreach my $reference (@orf_ref){
      my @record=@$reference;
      print ALL ">$record[0]***$record[1]***identity:$record[2]%***$record[3]***$record[4]***\n$record[5]\n";###nucleotide
   }

   close ALL;

}

=for html <A NAME='XmlDump'>

=item XmlDump

=for html
get sequence information and produce an xml file in the specified directory<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$config</B>():  [Default = <B>shift</B>]<BR>
- <B>$trace_condition</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
sub XmlDump{
#get sequence information and produce an xml file in the specified directory

   my $dbh=shift;
   my $config=shift;
   my %config=%$config;
   my $trace_condition=shift;
   my @trace_condition=@$trace_condition;   

   #GET CENTER NAME
   my @center_field=("Center_Name");
   my @center_table=("Center");
   my @center_condition=();
   my $select_center=SQLdb::GetRecord(\@center_field, \@center_table, \@center_condition);
   my $center=SQLdb::ExecuteUnique($dbh, $select_center);
   my @center=@$center;

   if (! -d "$config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Xml_Dir'}"){

       print "$config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Xml_Dir'} does not exist...creating directory\n";
       `mkdir $config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Xml_Dir'}`;
   }

   open (IN, ">$config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Xml_Dir'}/$config{'Config_Xml_File'}") or die "Can't open $config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Xml_Dir'}/$config{'Config_Xml_File'}\n";
  
   #ALL TRACE INFO
   my @trace_field=("Trace_Name", "Trace_Direction", "Trace_Well", "Trace_Type","Trace_Template", "Clone_Name", "Library_Name");
   my @trace_table=("Trace", "Clone", "Library");
   my $spec="ORDER BY Trace_Name DESC"; 
   my $select_trace=SQLdb::GetRecord(\@trace_field, \@trace_table, \@trace_condition, $spec);
   my $seq_record=SQLdb::ExecuteMany($dbh, $select_trace);
   my @seq_record=@$seq_record;

   print IN "<?xml version=\"1.0\"?>\n";
   print IN "<trace_volume>\n";

   foreach my $record (@seq_record){
      my @field=@$record;  
      my ($plate_id, $well_id);
     
      if ($field[0]=~/^(\w+)\..*\_(\D\d+)$/){###this is intended to fix a problem arachne has with pairing reads from same template
         $plate_id=$1;
         $well_id=$2;
      }

      print IN "\t<trace>\n";
      print IN "\t\t<CENTER_NAME>$center[0]</CENTER_NAME>\n";
      print IN "\t\t<TRACE_NAME>$field[0]</TRACE_NAME>\n";                                     #required
      print IN "\t\t<PLATE_ID>$plate_id</PLATE_ID>\n";                                         #required
      print IN "\t\t<SUBMISSION_TYPE>NEW</SUBMISSION_TYPE>\n";
      print IN "\t\t<TEMPLATE_ID>$field[4]</TEMPLATE_ID>\n";                                   #required if type=paired production
      print IN "\t\t<TRACE_END>$field[1]</TRACE_END>\n";# if ($field[1] ne 'U');               #if trace direction is undefined
      print IN "\t\t<WELL_ID>$well_id</WELL_ID>\n";                                            #required
      print IN "\t\t<LIBRARY_ID>$field[6]</LIBRARY_ID>\n";                                     #highly recommended
      print IN "\t\t<TYPE>$field[3]</TYPE>\n";
      print IN "\t</trace>\n";
   }

   print IN "</trace_volume>\n";
   close IN;

}#end sub

=for html <A NAME='FastaDump'>

=item FastaDump

=for html
GET SEQUENCE NAME,SEQUENCE, QUALITY<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$config</B>():  [Default = <B>shift</B>]<BR>
- <B>$trace_condition</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------
sub FastaDump{

  #GET SEQUENCE NAME,SEQUENCE, QUALITY
  my $dbh=shift;
  my $config=shift;
  my %config=%$config;
  my $trace_condition=shift;
  my @trace_condition=@$trace_condition;#very important if want to selectively get some reads

  my @trace_field=("Trace_Name", "Trace_Sequence", "Trace_Quality");
  my @trace_table=("Trace", "Clone", "Library");   
  my $spec="ORDER BY Trace_Name DESC";
  my $select_table=SQLdb::GetRecord(\@trace_field, \@trace_table, \@trace_condition, $spec);

  print "$select_table\n";

  my $seq_record=SQLdb::ExecuteMany($dbh, $select_table);
  my @seq_record=@$seq_record;

   open (SEQ, ">$config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Read_File'}") or die "Can't open $config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Read_File'}\n";
   open (QUAL, ">$config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Qual_File'}") or die "Can't open $config{'Config_Data_Dir'}/$config{'project_name'}/$config{'Config_Qual_File'}\n";

   foreach my $record (@seq_record){
      my @field=@$record;

      print SEQ ">$field[0]\n";
      print SEQ "$field[1]\n";
    
      print QUAL ">$field[0]\n";

      my @quality=unpack ("C*", $field[2]);
      print QUAL "@quality\n";
   }
   close SEQ;
   close QUAL;   

}#end sub

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut


return 1;
