#CustomAnalysis.pm module
#Calculates gap size, read overlap, contig orientation and insert size distribution
#Rene Warren
#March 2003
package CustomAnalysis;

##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

CustomAnalysis.pm - CustomAnalysis.pm module

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
CustomAnalysis.pm module<BR>Calculates gap size, read overlap, contig orientation and insert size distribution<BR>Rene Warren<BR>March 2003<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item strict

=item SQLdb

=item Statistics::Descriptive

=back

=cut

use strict;
use SQLdb;
use Statistics::Descriptive;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
#------------------------------------
1;

##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#cloneAnalysis'>cloneAnalysis; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#updateLibraryAnalysis'>updateLibraryAnalysis; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#fillTable'>fillTable; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#searchTemplate'>searchTemplate; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#initializeDirection'>initializeDirection; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getDistance'>getDistance; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getTemplateData'>getTemplateData; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#libraryAnalysis'>libraryAnalysis; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#fillLibraryAnalysis'>fillLibraryAnalysis; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#analyzeReadDistribution'>analyzeReadDistribution; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getLibraryReadCount'>getLibraryReadCount; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#reorganizeDistributionData'>reorganizeDistributionData; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#analyzeInsertDistribution'>analyzeInsertDistribution; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#calculateStat'>calculateStat; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#generateArray'>generateArray; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#defineRange'>defineRange; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#getAssemblyClone'>getAssemblyClone; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#calculateInsertSize'>calculateInsertSize; </A></strong></font><BR>

=over

=for html <A NAME='cloneAnalysis'>

=item cloneAnalysis

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>####Get Template Data</Font><BR>
</Font><BR>

=cut

#------------------------------------
sub cloneAnalysis{

   my $user_info=shift;
   my $assembly_id=shift;

   ####Get Template Data
   my ($template, $contig, $position, $library, $clone, $total)=getTemplateData($user_info, $assembly_id);

   my $init_date=`date`;

   ####Search for Gap-Spanning & Overlapping Clones
   my ($spanning_clone, $overlapping_clone, $pairing_info)=&searchTemplate($template, $contig, $position, $library, $clone, $total);
   my $date=`date`;
   print "$init_date\n";
   print "$date\n";

   ####Fill the Gap_Clone table
   my $gap_table="Gap";
   my @gap_field=("Gap_ID", "FK_Assembly__ID", "FK_Clone__ID","Gap_Left_Contig_Number", "Gap_Left_Contig_Size", "Gap_Right_Contig_Number", "Gap_Right_Contig_Size", "Gap_Left_Read_Name", "Gap_Left_Read_Start", "Gap_Left_Read_Stop", "Gap_Left_Read_Direction", "Gap_Right_Read_Name", "Gap_Right_Read_Start", "Gap_Right_Read_Stop", "Gap_Right_Read_Direction", "Gap_Left_Contig_Inversion", "Gap_Right_Contig_Inversion", "Gap_Size", "Gap_Clone_Misassembly");
   &fillTable($user_info, $spanning_clone, $assembly_id, $gap_table, \@gap_field);

   ####Fill the Overlap_Clone table
   my $overlap_table="Overlap";
   my @overlap_field=("Overlap_ID", "FK_Assembly__ID", "FK_Clone__ID","Overlap_Contig_Number", "Overlap_Contig_Size", "Overlap_Left_Read_Name", "Overlap_Left_Read_Start", "Overlap_Left_Read_Stop", "Overlap_Left_Read_Direction", "Overlap_Right_Read_Name", "Overlap_Right_Read_Start", "Overlap_Right_Read_Stop", "Overlap_Right_Read_Direction", "Overlap_Size", "Overlap_Read_Inclusion");

   &fillTable($user_info, $overlapping_clone, $assembly_id, $overlap_table, \@overlap_field);
   &updateLibraryAnalysis($user_info, $assembly_id, $pairing_info);
}

=for html <A NAME='updateLibraryAnalysis'>

=item updateLibraryAnalysis

=for html
specific info about read pairing in assembly<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$pairing_info</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
#specific info about read pairing in assembly
sub updateLibraryAnalysis{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembly_id=shift;
   my $pairing_info=shift;
   my %pairing_info=%$pairing_info;

   ####Connecting to Database
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});
   
   my $uniquetable="Library_Analysis";
   my @field=("Library_Analysis_Paired_Same", "Library_Analysis_Paired_Different", "Library_Analysis_Unpaired");

   foreach my $library_id (keys %pairing_info){
      my @value=("$pairing_info{$library_id}{'same'}", "$pairing_info{$library_id}{'different'}", "$pairing_info{$library_id}{'unpaired'}");
      my @condition=("FK_Assembly__ID=$assembly_id", "FK_Library__ID=$library_id");
      SQLdb::UpdateRecord($dbh, $uniquetable, \@field, \@value, \@condition);
   }

   SQLdb::Disconnect($dbh);
}

=for html <A NAME='fillTable'>

=item fillTable

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$ref_array</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
- <B>$unique_table</B>():  [Default = <B>shift</B>]<BR>
- <B>$field_array</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub fillTable{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $ref_array=shift;
   my @reference_array=@$ref_array;
   my $assembly_id=shift;
   my $unique_table=shift;
   my $field_array=shift;

   ####Connecting to Database
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});
   
   foreach my $reference (@reference_array){
      my @record=@$reference;
      my @value=("", "$assembly_id", @record);
    
      SQLdb::InsertRecord($dbh, $unique_table, $field_array, \@value);
   }

   SQLdb::Disconnect($dbh);
}

=for html <A NAME='searchTemplate'>

=item searchTemplate

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$template</B>():  [Default = <B>shift</B>]<BR>
- <B>$contig</B>():  [Default = <B>shift</B>]<BR>
- <B>$position</B>():  [Default = <B>shift</B>]<BR>
- <B>$library</B>():  [Default = <B>shift</B>]<BR>
- <B>$clone</B>():  [Default = <B>shift</B>]<BR>
- <B>$total</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub searchTemplate{

   my $template=shift;
   my %template=%$template;
   my $contig=shift;
   my %contig=%$contig;
   my $position=shift;
   my %position=%$position;
   my $library=shift;
   my %library=%$library;
   my $clone=shift;
   my %clone=%$clone;
   my $total=shift;
 
   my (%been_there, %pairing_info);
   my (@spanning_clone, @overlapping_clone);   
   my ($gapct, $overct)=(0,0);

   my $orientation=&initializeDirection();
   my %orientation=%$orientation;

   my $ct=0;
   foreach my $template_a (sort {$template{$a}<=>$template{$b}} keys %template){
      $ct++;
      my $percent=($ct/$total*100);
      print "$ct out of $total ($percent%)\n";

      $been_there{$template_a}++;###ensures that only check 1 paired-end

      my  $reverse;

      my $direction=$template{$template_a}{'direction'};
      my $clone_name=$template{$template_a}{'clone'};

      my $library_id=$library{$clone_name}{'id'};
      my $tig_a=$template{$template_a}{'contig'};

      if ($direction eq 'F'){$reverse="R";}elsif($direction eq 'R'){$reverse="F";}

      my $template_b=$clone_name.$reverse;###get the partner

      ####It is WAY TOO EXPENSIVE to have sub routine calls, the previous of this module was written that way 4 days-->30sec
      if (! defined $been_there{$template_b} && defined $template{$template_b}{'contig'}){###the order here is important for speeding purposes
          my $tig_b=$template{$template_b}{'contig'};
          if ($tig_a != $tig_b){####Gap Clones, paired-end reads located on <> contigs
             ####paired in different contig 
             $pairing_info{$library_id}{'same'}++;

              my ($distance, $min, $max_limit, $misassembly, $layout);
              my ($tig_l, $template_l, $tig_r, $template_r);

              ###Be Careful: A means Left and B means Right!!!!
              if ($tig_a<$tig_b){
                 $tig_l=$tig_a;
                 $template_l=$template_a;
                 $tig_r=$tig_b;
                 $template_r=$template_b;
              }else{
                 $tig_l=$tig_b;
                 $template_l=$template_b;
                 $tig_r=$tig_a;
                 $template_r=$template_a;
              }

              ####Define Variable
              my $tig_l_length=$contig{$tig_l}{'length'};
              my $tig_r_length=$contig{$tig_r}{'length'};
              my $read_l_start=$position{$template_l}{'start'};
              my $read_r_start=$position{$template_r}{'start'};
              my $read_l_stop=$position{$template_l}{'stop'};
              my $read_r_stop=$position{$template_r}{'stop'};

              ####Determine most likely possibility
              if ($position{$template_l}{'direction'}){
                 if (!$position{$template_r}{'direction'}){####->   <-
                    $layout="LR";
                    $distance=&getDistance($tig_l_length, $read_l_start, $read_r_stop);
                 }else{####->  ->
                    $layout="LrR";
                    my $rev_readr_stop=($tig_r_length-$read_r_start);
                    $distance=&getDistance($tig_l_length, $read_l_start, $rev_readr_stop);
                 }
             }else{
                if (!$position{$template_r}{'direction'}){####<-  ->
                   my $fwd_readl_start=($tig_l_length-$read_l_stop);
                   $layout="rLR";
                   $distance=&getDistance($tig_l_length, $fwd_readl_start, $read_r_stop);
                }else{####<-  ->
                   my $rev_readr_stop=($tig_r_length-$read_r_start);
                   my $fwd_readl_start=($tig_l_length-$read_l_stop);

                   $layout="rLrR";
                   $distance=&getDistance($tig_l_length, $fwd_readl_start, $rev_readr_stop);
                }
             }

             $min=100000000;
             $max_limit=(1.35*$library{$clone_name}{'insertsize'});#allow up to 35%

             my $gap_size=($library{$clone_name}{'insertsize'}-$distance);#gap>0, overlap<0
             #print "$template_a --- $library{$clone_name}{'insertsize'} --- $distance --- $clone_name\n";    

             my $allowed_size=($max_limit-$distance);

             if ($allowed_size >=0){
                $misassembly=0;
             }else{
                $misassembly=1;
             }

             my @record;
             @record=("$clone{$clone_name}{'id'}", "$tig_l", "$tig_l_length", "$tig_r", "$tig_r_length", "$position{$template_l}{'name'}", "$read_l_start", "$read_l_stop", "$position{$template_l}{'direction'}", "$position{$template_r}{'name'}","$read_r_start", "$read_r_stop", "$position{$template_r}{'direction'}", "$orientation{$layout}{'L'}", "$orientation{$layout}{'R'}", "$gap_size", "$misassembly");

             $gapct++; 
             push @spanning_clone, \@record;

          }else{###Clone, paired-end reads located on the same contig
             ####paired in same tig
             $pairing_info{$library_id}{'different'}++;
             my $tig_length=$contig{$tig_a}{'length'};        ###tig_a==tig_b
             my $read_a_start=$position{$template_a}{'start'};
             my $read_b_start=$position{$template_b}{'start'};
             my $read_a_stop=$position{$template_a}{'stop'};
             my $read_b_stop=$position{$template_b}{'stop'};

             my ($start_limit, $end_limit, $overlap, $inclusion)=(0,0,0,0);
             my @record;

             if ($read_a_start>=$read_b_start && $read_a_start<=$read_b_stop){
                $start_limit=$read_a_start;
                if ($read_b_stop<=$read_a_stop){
                   $end_limit=$read_b_stop; #  ---   a
                }else{#inclusion             ---     b
                   $end_limit=$read_a_stop; #  ---   a
                   $inclusion=1             #------- b
                }
                $overlap=$end_limit-$start_limit;

                @record=("$clone{$clone_name}{'id'}", "$tig_a", "$tig_length", "$position{$template_a}{'name'}", "$read_a_start", "$read_a_stop", "$position{$template_a}{'direction'}", "$position{$template_b}{'name'}","$read_b_start", "$read_b_stop", "$position{$template_b}{'direction'}", "$overlap", "$inclusion");
                push @overlapping_clone, \@record;
                $overct++;
            }elsif($read_b_start>=$read_a_start && $read_b_start<=$read_a_stop){
                #same in reverse
                $start_limit=$read_b_start;
               if ($read_a_stop<=$read_b_stop){
                   $end_limit=$read_a_stop; #  ---   b
               }else{#inclusion              ---     a
                   $end_limit=$read_b_stop; #  ---   b
                   $inclusion=1;            #------- a
               }                            
               $overlap=$end_limit-$start_limit;

               @record=("$clone{$clone_name}{'id'}", "$tig_a", "$tig_length", "$position{$template_b}{'name'}", "$read_b_start", "$read_b_stop", "$position{$template_b}{'direction'}", "$position{$template_a}{'name'}","$read_a_start", "$read_a_stop", "$position{$template_a}{'direction'}", "$overlap", "$inclusion");
               push @overlapping_clone, \@record;
               $overct++;
            }
         }

      }elsif(! defined $template{$template_b} && ! defined $been_there{$template_b}){
         ####unpaired
         $pairing_info{$library_id}{'unpaired'}++;
      }

   }
   print "Gaps:$gapct\tOverlaps:$overct\n";
   return \@spanning_clone, \@overlapping_clone, \%pairing_info;
}

=for html <A NAME='initializeDirection'>

=item initializeDirection

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
- None<BR>
<BR>

=cut

#------------------------------------
sub initializeDirection{

   my %direction;

   $direction{'LR'}{'L'}=0;
   $direction{'LR'}{'R'}=0;
   $direction{'rLrR'}{'L'}=1;
   $direction{'rLrR'}{'R'}=1;
   $direction{'LrR'}{'L'}=0;
   $direction{'LrR'}{'R'}=1;
   $direction{'rLR'}{'L'}=1;
   $direction{'rLR'}{'R'}=0;

   return \%direction;
}

=for html <A NAME='getDistance'>

=item getDistance

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$length_i</B>():  [Default = <B>shift</B>]<BR>
- <B>$start_i</B>():  [Default = <B>shift</B>]<BR>
- <B>$stop_j</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>#------  ---------</Font><BR>## <Font color=blue># i->        <-j</Font><BR>
</Font><BR>

=cut

#------------------------------------
sub getDistance{

   my $length_i=shift;
   my $start_i=shift;
   my $stop_j=shift;

   #------  ---------
   # i->        <-j

   my $v=($length_i-$start_i)+$stop_j;

   return $v;

}

=for html <A NAME='getTemplateData'>

=item getTemplateData

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub getTemplateData{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembly_id=shift;
   
   my (%contig, %length, %position, %library, %clone, %template);

   ####Connecting to Database
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});
   
   ####get Template, Contig, Contig Size, Start & Stop values
   ####Get all library Assembled for a given Assembly
   my @field=("Clone_Name", "Trace_Direction", "Contig_Number", "Contig_Size", "Assembled_Read_Start", "Assembled_Read_Stop", "Library_Insert_Size", "Contig_ID", "Clone_ID", "Assembled_Direction", "Trace_Name", "Library_ID");
   my @table=("Assembly","Contig", "Assembled", "Trace", "Clone", "Library");
   my @condition=("Assembly_ID=FK_Assembly__ID","Contig_ID=FK_Contig__ID","Trace_ID=FK_Trace__ID","Clone_ID=FK_Clone__ID","Library_ID=FK_Library__ID","Assembly_ID=$assembly_id");
   my $select=SQLdb::GetRecord(\@field, \@table, \@condition);
   my $tmp_reference=SQLdb::ExecuteMany($dbh, $select);
   SQLdb::Disconnect($dbh);

   my @clone_reference=@$tmp_reference;

   my $total=0;

   ####Fill data structures  
   foreach my $reference(@clone_reference){
      my @record=@$reference;
      my $template=$record[0].$record[1];
      $template{$template}{'contig'}=$record[2];
      $template{$template}{'direction'}=$record[1];
      $template{$template}{'clone'}=$record[0];

      $contig{$record[2]}{'length'}=$record[3];
      $contig{$record[2]}{'id'}=$record[7];

      $position{$template}{'start'}=$record[4];
      $position{$template}{'stop'}=$record[5];
      $position{$template}{'direction'}=$record[9];
      $position{$template}{'name'}=$record[10];

      $library{$record[0]}{'insertsize'}=$record[6];
      $library{$record[0]}{'id'}=$record[11];

      $clone{$record[0]}{'id'}=$record[8]; 
      $total++;
   }
   
   return \%template, \%contig, \%position, \%library, \%clone, $total;
}

=for html <A NAME='libraryAnalysis'>

=item libraryAnalysis

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>####read distribution</Font><BR>## <Font color=blue>#print "Read Distribution\n";</Font><BR>
</Font><BR>

=cut

#=====================================================================================================================================

#------------------------------------
sub libraryAnalysis{

   my $user_info=shift;
   my $assembly_id=shift;

   ####read distribution
   #print "Read Distribution\n";
   my ($contig_range, $contig_data)=&analyzeReadDistribution($user_info, $assembly_id);

   ####insert size distribution
   #print "Insert Size Distr\n";
   my ($range, $value, $statistics)=&analyzeInsertDistribution($user_info, $assembly_id);

   ####fill database
   #print "Fill db\n";
   &fillLibraryAnalysis($user_info, $range, $value, $statistics, $contig_range, $contig_data, $assembly_id);

}

=for html <A NAME='fillLibraryAnalysis'>

=item fillLibraryAnalysis

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$range</B>():  [Default = <B>shift</B>]<BR>
- <B>$value</B>():  [Default = <B>shift</B>]<BR>
- <B>$statistics</B>():  [Default = <B>shift</B>]<BR>
- <B>$contig_range</B>():  [Default = <B>shift</B>]<BR>
- <B>$contig_data</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub fillLibraryAnalysis{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $range=shift;
   my %range=%$range;
   my $value=shift;
   my %value=%$value;
   my $statistics=shift;
   my %statistics=%$statistics;
   my $contig_range=shift;
   my %contig_range=%$contig_range;
   my $contig_data=shift;   
   my %contig_data=%$contig_data;
   my $assembly_id=shift;

   ####Connecting to Database
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});
   my $uniquetable="Library_Analysis";  
   my @assembled_field=("Library_Analysis_ID", "FK_Assembly__ID", "FK_Library__ID", "Library_Analysis_Insert_Range", "Library_Analysis_Insert_Value", "Library_Analysis_Insert_Clone", "Library_Analysis_Insert_Mean", "Library_Analysis_Insert_StdDev", "Library_Analysis_Insert_Max", "Library_Analysis_Insert_Min", "Library_Analysis_Contig_Range", "Library_Analysis_Contig_Value");

   foreach my $library_id (sort keys %range){
      my $tmp_range=$range{$library_id};
      my $tmp_value=$value{$library_id};
      my $tmp_statistics=$statistics{$library_id};
      my $tmp_contig_range=$contig_range{$library_id};
      my $tmp_contig_data=$contig_data{$library_id};

      my @range=@$tmp_range;
      my @value=@$tmp_value;
      my @contig_range=@$tmp_contig_range;
      my @contig_value=@$tmp_contig_data;
      my %lib_stats=%$tmp_statistics;

      my @assembled_value=("","$assembly_id","$library_id", "@range", "@value", "$lib_stats{'count'}", "$lib_stats{'mean'}","$lib_stats{'stdev'}","$lib_stats{'max'}","$lib_stats{'min'}","@contig_range","@contig_value");
      SQLdb::InsertRecord($dbh, $uniquetable, \@assembled_field, \@assembled_value);

   } 
 
   SQLdb::Disconnect($dbh);

}

=for html <A NAME='analyzeReadDistribution'>

=item analyzeReadDistribution

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub analyzeReadDistribution{

    my $user_info=shift;
    my $assembly_id=shift;

    my ($contig_range, $contig_data);

    ####Get Library Read count
    ($contig_range, $contig_data)=&getLibraryReadCount($user_info, $assembly_id);

    return $contig_range, $contig_data;
}

=for html <A NAME='getLibraryReadCount'>

=item getLibraryReadCount

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub getLibraryReadCount{

   my $user_info=shift; 
   my %user_info=%$user_info;
   my $assembly_id=shift;

   my %contig_range;
   my %contig_data;

   ####Connecting to Database 
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get all library Assembled for a given Assembly
   my @lib_field=("Library_ID", "Library_Insert_Size");
   my @lib_table=("Assembly","Contig", "Assembled", "Trace", "Clone", "Library");
   my @lib_condition=("Assembly_ID=FK_Assembly__ID","Contig_ID=FK_Contig__ID","Trace_ID=FK_Trace__ID","Clone_ID=FK_Clone__ID","Library_ID=FK_Library__ID","Assembly_ID=$assembly_id");
   my $lib_instruction="GROUP BY Library_ID";
   my $select_library=SQLdb::GetRecord(\@lib_field, \@lib_table, \@lib_condition, $lib_instruction);
   my $library_reference=SQLdb::ExecuteMany($dbh, $select_library);
   my @library_ref=@$library_reference;

   ####Get Number of contigs for a given assembly
   my @contig_field=("Contig_Number");
   #my @contig_field=("min(Contig_Number)", "max(Contig_Number)");
   my @contig_table=("Contig", "Assembly");
   my @contig_condition=("Assembly_ID=FK_Assembly__ID", "Assembly_ID=$assembly_id");
   my $instructions="ORDER BY Contig_Number";
   my $select_contig=SQLdb::GetRecord(\@contig_field, \@contig_table, \@contig_condition, $instructions);
   #my $tmp_contig=SQLdb::ExecuteUnique($dbh, $select_contig);
   my $tmp_contig=SQLdb::ExecuteMany($dbh, $select_contig);
   
   #my @contig_value=@$tmp_contig;
   #my @contig_array=($contig_value[0]..$contig_value[1]);##clever, but won't do with floats
   my @contig_array;

   foreach my $ref(@$tmp_contig){
      my @record=@$ref;
      push @contig_array, $record[0];
   }

   ####Get Data for distinct libraries
   foreach my $reference(@library_ref){
      my @lib_arr=@$reference;
      my @field=("Contig_Number", "count(Library_ID)");
      my @table=("Assembly", "Contig", "Assembled", "Trace", "Clone", "Library");
      my @condition=("Assembly_ID=FK_Assembly__ID","Contig_ID=FK_Contig__ID","Trace_ID=FK_Trace__ID","Clone_ID=FK_Clone__ID","Library_ID=FK_Library__ID","Assembly_ID=$assembly_id", "Library_ID='$lib_arr[0]'");
      my $special_instruction="Group By Contig_Number";
      my $select_distribution=SQLdb::GetRecord(\@field, \@table, \@condition, $special_instruction);
      my $tmp_distribution=SQLdb::ExecuteMany($dbh, $select_distribution);

      my $read_value=&reorganizeDistributionData($tmp_distribution, \@contig_array, $lib_arr[0]);#Generate independent arrays for every library

      $contig_range{$lib_arr[0]}=\@contig_array;
      $contig_data{$lib_arr[0]}=$read_value; 
   }
   SQLdb::Disconnect($dbh);

   return \%contig_range, \%contig_data; 
}

=for html <A NAME='reorganizeDistributionData'>

=item reorganizeDistributionData

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$array_of_reference</B>():  [Default = <B>shift</B>]<BR>
- <B>$contig_list</B>():  [Default = <B>shift</B>]<BR>
- <B>$library</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub reorganizeDistributionData{

   my $array_of_reference=shift;
   my @array_of_reference=@$array_of_reference;
   my $contig_list=shift;
   my @contig_list=@$contig_list;
   my $library=shift;

   my @read_value;
   my %contig;

   ####Reorganize the data into hash
   foreach my $reference(@array_of_reference){
      my @value=@$reference;
      $contig{$value[0]}=$value[1];  ###this is awfully specific to the query: 1)contig 2)amount
   }

   ####read count for every contig
   foreach my $contig_number (@contig_list){
      if (defined $contig{$contig_number}){
         push @read_value, $contig{$contig_number};
      }else{
         push @read_value, 0; #if query returned nothing for that contig, 0 reads from that lib are in
      }
   }

   return \@read_value;
}

=for html <A NAME='analyzeInsertDistribution'>

=item analyzeInsertDistribution

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
## <Font color=blue>####Get Clone-Specific Info</Font><BR>
</Font><BR>

=cut

#------------------------------------
sub analyzeInsertDistribution{

    my $user_info=shift;
    my $assembly_id=shift;

    ####Get Clone-Specific Info
    my ($contig_composition, $trace_position, $length, $trace_direction)=&getAssemblyClone($user_info, $assembly_id);

    ####Get Range
    my $library_range=&defineRange($length);

    ####Insert Size Distribution    
    my ($distribution, $all_insert)=&calculateInsertSize($contig_composition, $trace_position, $library_range, $trace_direction);
    
    ####Generate Data Array
    my $library_value=&generateArray($library_range, $distribution);
    
    ####Calculate Stats
    my $statistics=&calculateStat($all_insert);
  
    return $library_range, $library_value, $statistics;
}

=for html <A NAME='calculateStat'>

=item calculateStat

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$all_insert</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub calculateStat{

   my $all_insert=shift;
   my %all_insert=%$all_insert;

   my %statistics;
 
   foreach my $library (keys %all_insert){
      my $tmp_insert=$all_insert{$library};
      my @insert=@$tmp_insert;

      my $stat=Statistics::Descriptive::Full->new();
      $stat->add_data(@insert);
      $statistics{$library}{'mean'}=$stat->mean();
      $statistics{$library}{'max'}=$stat->max();
      $statistics{$library}{'min'}=$stat->min();
      $statistics{$library}{'count'}=$stat->count();
      $statistics{$library}{'stdev'}=$stat->standard_deviation();
   }
   return \%statistics;
}

=for html <A NAME='generateArray'>

=item generateArray

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$library_range</B>():  [Default = <B>shift</B>]<BR>
- <B>$distribution</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub generateArray{

   my $library_range=shift;
   my %library_range=%$library_range;
   my $distribution=shift;
   my %distribution=%$distribution;

   my %library_value;

   foreach my $library (keys %library_range){
      my @insert;   
      my $tmp_range=$library_range{$library};
      my @range=@$tmp_range; 

      foreach my $xvalue (@range){
         if(defined $distribution{$library}{$xvalue}){
            push @insert, $distribution{$library}{$xvalue};
         }else{
            push @insert, 0;
         }
      }
      #print "$library @range\n";
      #print "$library @insert\n";

      $library_value{$library}=\@insert;
   }

   return \%library_value;
}

=for html <A NAME='defineRange'>

=item defineRange

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$length</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub defineRange{

   my $length=shift;
   my %length=%$length;

   my %library_range;

   my $chunk=200;

   foreach my $library(keys %length){
  
      my $length=$length{$library};
      my $tmp_chunk=$chunk;
      my $factor=1;                             #small & medium plasmid library

      if ($length>9000 && $length<20000){       #large plasmid library
         $factor=5;
         $tmp_chunk=$chunk*$factor;
      }elsif($length>=20000 && $length<60000){  #fosmid libraries
         $factor=10;
         $tmp_chunk=$chunk*$factor;
      }elsif($length>=60000){                   #BAC libraries
         $factor=100;
         $tmp_chunk=$chunk*$factor;
      }

      my @range;
      my $low=$length-(3000*$factor);
      my $high=$length+(3000*$factor);

      $low=0 if ($low<0);

      for (my $value=$low;$value<=$high;$value+=$tmp_chunk){
         push @range, $value;
      }
      
      $library_range{$library}=\@range;
      #print "$library $length $factor @range\n";
   }

   return \%library_range;
}

=for html <A NAME='getAssemblyClone'>

=item getAssemblyClone

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$user_info</B>():  [Default = <B>shift</B>]<BR>
- <B>$assembly_id</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub getAssemblyClone{

   my $user_info=shift;
   my %user_info=%$user_info;
   my $assembly_id=shift;

   my %contig_composition;
   my %trace_position;
   my %length;
   my %trace_direction;

   ####Initialize Database
   my $dbh=SQLdb::Initialize($user_info{'password'}, $user_info{'username'}, $user_info{'database'}, $user_info{'host_name'});

   ####Get all library Assembled for a given Assembly
   my @lib_field=("Library_ID", "Library_Insert_Size");
   my @lib_table=("Assembly","Contig", "Assembled", "Trace", "Clone", "Library");
   my @lib_condition=("Assembly_ID=FK_Assembly__ID","Contig_ID=FK_Contig__ID","Trace_ID=FK_Trace__ID","Clone_ID=FK_Clone__ID","Library_ID=FK_Library__ID","Assembly_ID=$assembly_id");
   my $lib_instruction="GROUP BY Library_ID";
   my $select_library=SQLdb::GetRecord(\@lib_field, \@lib_table, \@lib_condition, $lib_instruction);
   #print "$select_library\n";
   my $library_reference=SQLdb::ExecuteMany($dbh, $select_library);
   my @library_ref=@$library_reference;

   foreach my $reference(@library_ref){
      my @lib_arr=@$reference;

      $length{$lib_arr[0]}=$lib_arr[1];
      #print "$lib_arr[0]...$lib_arr[1]\n";

      ####Get Clones
      my @field=("Contig_Number","Clone_Name", "Trace_Name", "Trace_Direction", "Assembled_Read_Start","Assembled_Read_Stop", "Assembled_Direction");
      my @table=("Assembly","Contig", "Assembled", "Trace", "Clone", "Library");
      my @condition=("Assembly_ID=FK_Assembly__ID","Contig_ID=FK_Contig__ID","Trace_ID=FK_Trace__ID","Clone_ID=FK_Clone__ID","Library_ID=FK_Library__ID","Assembly_ID=$assembly_id", "Library_ID='$lib_arr[0]'");
      my $special_instruction="Order by Contig_Number";
      my $select_clone=SQLdb::GetRecord(\@field, \@table, \@condition, $special_instruction);
      my $clone_reference=SQLdb::ExecuteMany($dbh, $select_clone);

      #print "$select_clone\n";

      ####Reorganize Data
      my @clone_array=@$clone_reference;
      foreach my $reference (@clone_array){  
         my @record=@$reference;
         my @position=($record[4], $record[5]);
         $contig_composition{$lib_arr[0]}{$record[0]}{$record[1]}{$record[2]}=$record[3];
         $trace_position{$record[2]}=\@position;
         $trace_direction{$record[2]}=$record[6];
      }
   }
   SQLdb::Disconnect($dbh);

   return \%contig_composition, \%trace_position, \%length, \%trace_direction;
}

=for html <A NAME='calculateInsertSize'>

=item calculateInsertSize

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$tig_composition</B>():  [Default = <B>shift</B>]<BR>
- <B>$trace_position</B>():  [Default = <B>shift</B>]<BR>
- <B>$library_range</B>():  [Default = <B>shift</B>]<BR>
- <B>$trace_direction</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#------------------------------------
sub calculateInsertSize{ 

   my $tig_composition=shift;
   my %tig_composition=%$tig_composition;
   my $trace_position=shift;
   my %trace_position=%$trace_position;
   my $library_range=shift;
   my %library_range=%$library_range;
   my $trace_direction=shift;
   my %trace_direction=%$trace_direction;
   
   my (%all_insert, %distribution);

   foreach my $library(keys %tig_composition){ 

      my $tmp_tig_layer=$tig_composition{$library};
      my %tig_layer=%$tmp_tig_layer;
      my @insert;

      foreach my $contig (keys %tig_layer){
        #print "$contig\t";
        my $tmp_clone=$tig_layer{$contig};
        my %clone_layer=%$tmp_clone;
        foreach my $clone(keys %clone_layer){
          my $tmp_read=$clone_layer{$clone};
          my %read_layer=%$tmp_read;  
          my $count_clone=0;
          my $previous_read='';
          my $previous_direction='';
          #print "$clone\t";
          foreach my $read (keys %read_layer){#R/F/U
             #print "$read\n";    
             my $direction=$read_layer{$read};

             if ($count_clone){#had seen a read already

#                print "**** $read ne $previous_read && $direction ne $previous_direction && $trace_direction{$read} ne $trace_direction{$previous_read}****\n" if ($clone eq 'RM00459aB05');
                if ($read ne $previous_read && $direction ne $previous_direction && $trace_direction{$read} ne $trace_direction{$previous_read}){#A->F and (B->R or A1->R)
                    ####positions
                    my ($read_start, $read_stop, $prev_start, $prev_stop);

                    my $tmp_read_position=$trace_position{$read};
                    my $tmp_prev_position=$trace_position{$previous_read};
                    my @read_position=@$tmp_read_position;
                    my @prev_position=@$tmp_prev_position;

                    ($read_start, $read_stop)=@read_position;
                    ($prev_start, $prev_stop)=@prev_position;

                    my $tmp_range=$library_range{$library};
                    my @range=@$tmp_range;

                    ###takes into account overlapping paired-end reads!!!
                    my $insert_size=0;
                    if ($read_start<=$prev_start && $read_stop<=$prev_stop){###will take into account overlaps
                       $insert_size=$prev_stop-$read_start;
                    }elsif($prev_start<=$read_start && $prev_stop<=$read_stop){
                       $insert_size=$read_stop-$prev_start;
                    }elsif($prev_start>$read_start && $prev_stop<$read_stop){#inclusion
                       $insert_size=$read_stop-$read_start;
#                       print "$read read start:$read_start read stop:$read_stopprev start:$prev_start prev stop:$prev_stop\n";
                    }elsif($read_start>$prev_start && $read_stop<$prev_stop){#inclusion
                       $insert_size=$prev_stop-$prev_start;
                    #   print "!\n";
                    }
                    my $oldvalue=0;
                    #print "* $library $read read start:$read_start read stop:$read_stop  $trace_direction{$read}  $previous_read prev start:$prev_start prev stop:$prev_stop   $trace_direction{$previous_read}\n>insert===$insert_size\n" if ($insert_size<=0);
                    push @insert, $insert_size;
                    foreach my $value(@range){
                       if ($insert_size>$oldvalue && $insert_size<$value){
                          $distribution{$library}{$value}++;
                       }
                       $oldvalue=$value;
                    }
#                    print "$read ne $previous_read -->$insert_size\n" if ($clone eq 'RM00459aB05');
                }#elsif($read ne $previous_read && $direction eq $previous_direction){#A->U B->U

                 #  if($direction eq 'U'){#undefined read pairing/transposon

                 #  }#else is just a different version of same read
                #}elsif($read ne $previous_read && $direction ne $previous_direction && $trace_direction{$read} ne $trace_direction{$previous_read}){

                #}#reads are mate pairs, but point in same direction

             }
             $previous_read=$read;
             $previous_direction=$direction;
             $count_clone++;
          }
       }
    }
    $all_insert{$library}=\@insert;
  }

  return \%distribution, \%all_insert;
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut


return 1;
