=head1 NAME <UPLINK>

Plate.pm - Well-conversion module

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
Well-conversion module<BR>Plate.pm is a module to facilitate the conversion of wells on laboratory plastic ware. <BR>It currently includes two methods, used to transform well notation 384->96 or 96->384 depending on the method chosen <BR>

=cut

#Well-conversion module
#Plate.pm is a module to facilitate the conversion of wells on laboratory plastic ware. 
#It currently includes two methods, used to transform well notation 384->96 or 96->384 depending on the method chosen 

package Plate;

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

use strict;
use Number::Format;

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

=head2 ATTRIBUTES <UPLINK>

None

=cut

=head2 CONSTRUCTOR <UPLINK>

=over

=item new

=for html
constructor for the plate object  <BR>

=for html
Arguments:<BR>
- <B>($class, $object_ref)</B>(): [Default = @_]<BR>
<BR>

=cut

#--------------------------------------------------
sub new{

   #constructor for the plate object  
   my ($class, $object_ref)=@_; 

   bless $object_ref, $class;
  
   return $object_ref; 
}

=back

=cut

=head2 PUBLIC FUNCTIONS <UPLINK>

=over

=item transform96to384

=for html
Method for transforming a Plate object used to convert a 96-well into 384-well notation (it will output a quadrant followed by a well)<BR>To use this method, simply point the object to the well and quadrant of interest<BR>This is because a 384-well could be split into 4 quadrants a,b,c,d representing 4X96-well plates on a single 384-well<BR>

=for html
Arguments:<BR>
- <B>($this, @arg)</B>(): receives the object, no arguments [Default = @_]<BR>
<BR>

=cut

#--------------------------------------------------
sub transform96to384{

    #Method for transforming a Plate object used to convert a 96-well into 384-well notation (it will output a quadrant followed by a well)
    #To use this method, simply point the object to the well and quadrant of interest
    #This is because a 384-well could be split into 4 quadrants a,b,c,d representing 4X96-well plates on a single 384-well
    my ($this, @arg)=@_;#receives the object, no arguments

    my @vertical=('A'..'H');
    my @horizontal=(1..12);
    my @quadrant=('a'..'d');

    my @alpha=(1,'A',2,'B',3,'C',4,'D',5,'E',6,'F',7,'G',8,'H',9,'I',10,'J',11,'K',12,'L',13,'M',14,'N',15,'O',16,'P',17,'Q',18,'R',19,'S',20,'T',21,'U',22,'V',23,'W',24,'X',25,'Y',26,'Z');
    my %alpha=@alpha;

    my $vertpos=1;
    

    foreach my $vert (@vertical){

         my $horizpos=1;

         foreach my $horiz (@horizontal){

            my ($well, $newwell);
            my ($number, $letter, $letternum);

            if ($horiz<10){
               $well=$vert."0".$horiz;
            }else{
               $well=$vert.$horiz;
            }

            foreach my $quad (@quadrant){  #must see how data is stored for a 384 well plate

                 if ($quad eq 'a'){
                     $number=$horizpos;
                     $letternum=$vertpos;
                }
                 elsif($quad eq 'b'){
                     $number=$horizpos+1;
                     $letternum=$vertpos;
                 }
                 elsif($quad eq 'c'){
                     $number=$horizpos;
                     $letternum=$vertpos+1;
                 }
                 elsif($quad eq 'd'){
                     $number=$horizpos+1;
                     $letternum=$vertpos+1;
                 }
                 my $letter=$alpha{$letternum};

                 if ($number<10){
                    $newwell=$letter."0".$number;
                 }else{
                    $newwell=$letter.$number;
                 }

                 $this->{$well}{$quad}=$newwell;
                 #$position{$well}{$quad}=$newwell;

                 #print "$well$quad\t$newwell\n";

            }
            $horizpos=2*$horiz;
            $horizpos++;
        }
        $vertpos+=2;
   }
   return $this;
}

=item transform384to96

=for html
Method for transforming a Plate object used to tranform a 384-well into a 96-well plate<BR>This is because a 384-well could be split into 4 quadrants a,b,c,d representing 4X96-well plates on a single 384-well<BR>

=for html
Arguments:<BR>
- <B>($this, @arg)</B>(): receives the object, no arguments [Default = @_]<BR>
<BR>

=cut

#--------------------------------------------------
sub transform384to96{

   #Method for transforming a Plate object used to tranform a 384-well into a 96-well plate
   #This is because a 384-well could be split into 4 quadrants a,b,c,d representing 4X96-well plates on a single 384-well
   my ($this, @arg)=@_;#receives the object, no arguments
   
   my @vertical=('A'..'P');
   my @horizontal=(1..24);
   
   my @new_vert=('A','A','B','A','C','B','D','B','E','C','F','C','G','D','H','D','I','E','J','E','K','F','L','F','M','G','N','G','O','H','P','H');
   my %new_vert=@new_vert;

   my @new_horiz=(1,1,2,1,3,2,4,2,5,3,6,3,7,4,8,4,9,5,10,5,11,6,12,6,13,7,14,7,15,8,16,8,17,9,18,9,19,10,20,10,21,11,22,11,23,12,24,12);
   my %new_horiz=@new_horiz;

   my @alpha=('A',1,'B',2,'C',3,'D',4,'E',5,'F',6,'G',7,'H',8,'I',9,'J',10,'K',11,'L',12,'M',13,'N',14,'O',15,'P',16,'Q',17,'R',18,'S',19,'T',20,'U',21,'V',22,'W',23,'X',24,'Y',25,'Z',26);
   my %alpha=@alpha;

   foreach my $vert (@vertical){

      foreach my $horiz (@horizontal){
    
         my ($quadrant, $well_96, $well_384);

         if ($horiz<10){
             $well_384=$vert."0".$horiz;
         }else{
             $well_384=$vert.$horiz;
         }

         my $horiz_even=($horiz%2);#modulus 0 if even
         my $vert_even=($alpha{$vert}%2);

         if (!$horiz_even && !$vert_even){
            $quadrant='d';
         }
         elsif ($horiz_even && $vert_even){
            $quadrant='a';
         }
         elsif ($horiz_even && !$vert_even){
            $quadrant='c';
         }
         elsif (!$horiz_even && $vert_even){
            $quadrant='b';
         }

         my $horizontal_96=$new_horiz{$horiz};
         my $vertical_96=$new_vert{$vert};

         if ($horizontal_96<10){
            $well_96=$quadrant.$vertical_96."0".$horizontal_96;
         }
         else{
            $well_96=$quadrant.$vertical_96.$horizontal_96;
         }

         #print "***$well_384***$well_96***\n";
         $this->{$well_384}=$well_96;
      }
   }
   return $this;
}

=back

=cut

1;

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

Rene Warren, 2003/10/10

=head1 CREATED <UPLINK>

2003-07-31

=head1 REVISION <UPLINK>

$Id: Plate.pm,v 1.1 2003/10/10 21:35:00 rwarren Exp $ (Release: $Name:  $)

=cut

