#SQLdb.pm - Database connection/interaction module
#generic, could be used with any database, tables, even allows incorporation of binary and multiple conditions
#Rene Warren, OCT2002
package SQLdb;

##############################
# perldoc_header             #
##############################

=head1 NAME <UPLINK>

SQLdb.pm - SQLdb.pm - Database connection/interaction module

=head1 SYNOPSIS <UPLINK>

	<<SYNOPSIS>>

=head1 DESCRIPTION <UPLINK>

=for html
SQLdb.pm - Database connection/interaction module<BR>generic, could be used with any database, tables, even allows incorporation of binary and multiple conditions<BR>Rene Warren, OCT2002<BR>

=cut

##############################
# superclasses               #
##############################

=head2 SUPERCLASSES <UPLINK>

=over

=back

=cut

##############################
# system_variables           #
##############################
##############################
# standard_modules_ref       #
##############################

=head2 STANDARD MODULES REFERENCE <UPLINK>

=over

=item strict

=item DBI;1

=back

=cut

use strict;
use DBI;1;

##############################
# custom_modules_ref         #
##############################

=head2 CUSTOM MODULES REFERENCE <UPLINK>

=over

=back

=cut

##############################
# global_vars                #
##############################

=head2 GLOBAL VARIABLES <UPLINK>

=over

=back

=cut

##############################
# modular_vars               #
##############################
##############################
# constants                  #
##############################
##############################
# main_header                #
##############################
##############################
# constructor                #
##############################
##############################
# public_methods             #
##############################
##############################
# public_functions           #
##############################

=head2 PUBLIC FUNCTIONS <UPLINK>

=for html <font size='-1'><strong><A href='#Initialize'>Initialize; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#Disconnect'>Disconnect; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#deleteRecord'>deleteRecord; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#UpdateRecord'>UpdateRecord; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#InsertRecord'>InsertRecord; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#GetRecord'>GetRecord; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#ExecuteUnique'>ExecuteUnique; </A></strong></font><BR>

=for html <font size='-1'><strong><A href='#ExecuteMany'>ExecuteMany; </A></strong></font><BR>

=over

=for html <A NAME='Initialize'>

=item Initialize

=for html
Initialize database connection<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$password</B>():  [Default = <B>shift</B>]<BR>
- <B>$login</B>():  [Default = <B>shift</B>]<BR>
- <B>$database</B>():  [Default = <B>shift</B>]<BR>
- <B>$host_name</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub Initialize{
####Initialize database connection

   my $password=shift;
   my $login=shift;
   my $database=shift;
   my $host_name=shift;

   my $dsn="DBI:mysql:$database:$host_name";
   my $dbh;

   ####Do not proceed until connected to db
   until ($dbh = DBI ->connect($dsn, $login, $password, {PrintError => 0, RaiseError =>0})){
      #PrintError doesn't make the script die if error detected 0=>silent, 1=>verbose
      warn "Can't connect: $DBI::errstr";
      sleep 3;#give it a 3 seconds rest
   }

   return $dbh;
}

=for html <A NAME='Disconnect'>

=item Disconnect

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub Disconnect{

   my $dbh=shift;
   
   $dbh->disconnect();

}

=for html <A NAME='deleteRecord'>

=item deleteRecord

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$table</B>():  [Default = <B>shift</B>]<BR>
- <B>$condition</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub deleteRecord{

   my $dbh=shift;
   my $table=shift;
   my $condition=shift;
   my @condition=@$condition;

   my $select="DELETE From $table ";
   $select.="WHERE ";
   for (my $index=0;$index<=$#condition;$index++){
      $select.=$condition[$index];
      $select.=" and " if ($index<$#condition);   #only add ' and ' if not last element in array
   }

   my $sth = $dbh->prepare($select);
   $sth->execute();

   $sth->finish();

}

=for html <A NAME='UpdateRecord'>

=item UpdateRecord

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$table</B>():  [Default = <B>shift</B>]<BR>
- <B>$field</B>():  [Default = <B>shift</B>]<BR>
- <B>$value</B>():  [Default = <B>shift</B>]<BR>
- <B>$condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$binary</B>(): takes into account possible binaries  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub UpdateRecord{

   my $dbh=shift;
   my $table=shift;
   my $field=shift;
   my @field=@$field;
   my $value=shift;
   my @value=@$value;
   my $condition=shift;
   my @condition=@$condition;
   my $binary=shift;   #takes into account possible binaries
   my @binary=@$binary if ($binary);

   my $select="UPDATE $table ";
   $select.="SET ";
   for(my $index=0;$index<=$#field;$index++){
      $select.=$field[$index]."=";
      if ($value[$index] eq '?'){
          $select.=$value[$index];
      }
      else{
          $select.='"'.$value[$index].'"';
          #$select.=$value[$index];
      }
      $select.=", " if ($index<$#value);   #only add coma if not last element in array
   }
   $select.=" WHERE ";
   for (my $index=0;$index<=$#condition;$index++){
      $select.=$condition[$index];
      $select.=" and " if ($index<$#condition);   #only add ' and ' if not last element in array
   }
   #print "$select\n";
   my $sth = $dbh->prepare($select);

   if (@binary){$sth->execute(@binary);}else{$sth->execute();}

   $sth->finish();
}

=for html <A NAME='InsertRecord'>

=item InsertRecord

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$table</B>():  [Default = <B>shift</B>]<BR>
- <B>$field</B>():  [Default = <B>shift</B>]<BR>
- <B>$value</B>():  [Default = <B>shift</B>]<BR>
- <B>$binary</B>(): takes into account possible binaries  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub InsertRecord{

   my $dbh=shift;
   my $table=shift;
   my $field=shift;
   my @field=@$field; 
   my $value=shift;
   my @value=@$value;
   my $binary=shift;   #takes into account possible binaries
   my @binary=@$binary if ($binary);

   my $select="INSERT INTO $table (";
   for(my $index=0;$index<=$#field;$index++){
      $select.=$field[$index];
      $select.=", " if ($index<$#field);   #only add coma if not last element in array
   }
   $select.=") VALUES(";
   for (my $index=0;$index<=$#value;$index++){
      if ($value[$index] eq '?'){
          $select.=$value[$index];
      }
      else{
          $select.='"'.$value[$index].'"';
      }
      $select.=", " if ($index<$#value);   #only add coma if not last element in array
   }
   $select.=")";

   #print "$select";

   my $sth = $dbh->prepare($select);
  
 
   if (@binary){$sth->execute(@binary);}else{$sth->execute();}
   $sth->finish();

}

=for html <A NAME='GetRecord'>

=item GetRecord

=for html
select statement builder for retrieving records (could be many fields) from 1 or multiple table with 1 or more cdt<BR>

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$field</B>():  [Default = <B>shift</B>]<BR>
- <B>$table</B>():  [Default = <B>shift</B>]<BR>
- <B>$condition</B>():  [Default = <B>shift</B>]<BR>
- <B>$special_instruction</B>():  [Default = <B>shift</B>]<BR>
- <B>$logic</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub GetRecord{
#select statement builder for retrieving records (could be many fields) from 1 or multiple table with 1 or more cdt

   my $field=shift;
   my @field=@$field;
   my $table=shift;
   my @table=@$table;
   my $condition=shift;
   my @condition=@$condition;
   my $special_instruction=shift;
   my $logic=shift;
   
   $logic="AND" if (! defined $logic);#set default to AND for logical operator

   my $select="SELECT ";
   for(my $index=0;$index<=$#field;$index++){
      $select.=$field[$index];
      $select.=", " if ($index<$#field);   #only add coma if not last element in array
   }
   $select.=" FROM ";
   for (my $index=0;$index<=$#table;$index++){
      $select.=$table[$index];
      $select.=", " if ($index<$#table);   #only add coma if not last element in array
   }
  
   if ($condition[0]){#WHERE clause is NOT always present if there are no conditions
  
      $select.=" WHERE ";
      for (my $index=0;$index<=$#condition;$index++){
         $select.="(".$condition[$index].")";
         $select.=" $logic " if ($index<$#condition);   #only add ' and ' if not last element in array
      } 
   }
   
   if($special_instruction){
      $select.=" $special_instruction";
   }
   #print "$select\n";

   return $select;

}

=for html <A NAME='ExecuteUnique'>

=item ExecuteUnique

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$select</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub ExecuteUnique{

   my $dbh=shift;
   my $select=shift;
   
   my $sth = $dbh->prepare($select);
   $sth->execute();
   my @ary=$sth->fetchrow_array();
   $sth->finish();

   return \@ary;
}

=for html <A NAME='ExecuteMany'>

=item ExecuteMany

=for html

=for html
<B>Arguments:</B><BR>
********************<BR>
<Font size =-2>- <B>$dbh</B>():  [Default = <B>shift</B>]<BR>
- <B>$select</B>():  [Default = <B>shift</B>]<BR>
</Font><BR>

=cut

#-------------------------------------
sub ExecuteMany{

   my $dbh=shift;
   my $select=shift;
   
   my @multiple_record;

   my $sth = $dbh->prepare($select);
   $sth->execute();

   while (my @record=$sth->fetchrow_array()){
      push @multiple_record, \@record;
   }
   $sth->finish();

   return \@multiple_record;
}

=back

=cut

##############################
# private_methods            #
##############################
##############################
# private_functions          #
##############################
##############################
# main_footer                #
##############################
##############################
# perldoc_footer             #
##############################

=head1 KNOWN ISSUES <UPLINK>

<<KNOWN ISSUES>>

=head1 FUTURE IMPROVEMENTS <UPLINK>

<<FUTURE IMPROVEMENTS>>

=head1 AUTHORS <UPLINK>

<<AUTHORS>>

=head1 CREATED <UPLINK>

2004-08-18

=head1 REVISION <UPLINK>


=cut


return 1;
